@@ -3,7 +3,7 @@
  *                      but exported to plug-ins
  *****************************************************************************
  * Copyright (C) 1999-2002 VideoLAN
- * $Id: input_ext-plugins.h,v 1.42 2003/05/05 22:23:31 gbazin Exp $
+ * $Id: input_ext-plugins.h,v 1.46 2003/11/30 22:47:55 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -55,12 +55,21 @@ VLC_EXPORT( int,  input_UnselectES,( input_thread_t *, es_descriptor_t * ) );
 /*****************************************************************************
  * Prototypes from input_dec.c
  *****************************************************************************/
-decoder_fifo_t * input_RunDecoder( input_thread_t *, es_descriptor_t * );
+decoder_t * input_RunDecoder( input_thread_t *, es_descriptor_t * );
 void input_EndDecoder( input_thread_t *, es_descriptor_t * );
-VLC_EXPORT( void, input_DecodePES, ( decoder_fifo_t *, pes_packet_t * ) );
+
+VLC_EXPORT( void, input_DecodePES, ( decoder_t *, pes_packet_t * ) );
+VLC_EXPORT( void, input_DecodeBlock,( decoder_t *, block_t * ) );
+
 void input_EscapeDiscontinuity( input_thread_t * );
 void input_EscapeAudioDiscontinuity( input_thread_t * );
-VLC_EXPORT( void, input_NullPacket, ( input_thread_t *, es_descriptor_t * ) );
+
+/*****************************************************************************
+ * Prototypes from es_out.c
+ *****************************************************************************/
+/* input internal use only */
+es_out_t *input_EsOutNew( input_thread_t * );
+void      input_EsOutDelete( es_out_t * );
 
 /*****************************************************************************
  * Prototypes from input_clock.c
@@ -76,6 +85,7 @@ VLC_EXPORT( mtime_t, input_ClockGetTS, ( input_thread_t *, pgrm_descriptor_t *,
 VLC_EXPORT( input_info_category_t *, input_InfoCategory, ( input_thread_t *, char * ) );
 VLC_EXPORT( int, input_AddInfo, ( input_info_category_t *, char *, char *, ... ) );
 int input_DelInfo( input_thread_t * p_input ); /* no need to export this */
+
 /*****************************************************************************
  * Prototypes from input_ext-plugins.h (buffers management)
  *****************************************************************************/
@@ -96,9 +106,6 @@ VLC_EXPORT( int, input_AccessInit,      ( input_thread_t * ) );
 VLC_EXPORT( void, input_AccessReinit,   ( input_thread_t * ) );
 VLC_EXPORT( void, input_AccessEnd,      ( input_thread_t * ) );
 
-/* no need to export this one */
-data_packet_t *input_NewPacketForce( input_buffers_t *, size_t );
-
 /*
  * Optional standard file descriptor operations (input_ext-plugins.h)
  */
