@@ -3,7 +3,7 @@
  * Declaration and extern access to global program object.
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001, 2002 VideoLAN
- * $Id: main.h,v 1.52 2003/01/19 03:16:24 sam Exp $
+ * $Id: main.h,v 1.56 2003/10/29 01:33:27 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -64,6 +64,7 @@ struct libvlc_t
     SIGNALOBJECTANDWAIT    SignalObjectAndWait;
     vlc_bool_t             b_fast_mutex;
     int                    i_win9x_cv;
+    char *                 psz_vlcpath;
 #endif
 };
 
@@ -80,9 +81,7 @@ struct vlc_t
     int                    i_argc;           /* command line arguments count */
     char **                ppsz_argv;              /* command line arguments */
     char *                 psz_homedir;             /* user's home directory */
-
-    /* Generic settings */
-    mtime_t                i_desync;   /* relative desync of the audio ouput */
+    char *                 psz_configfile;        /* location of config file */
 
     /* Fast memcpy plugin used */
     module_t *             p_memcpy_module;
@@ -96,9 +95,20 @@ struct vlc_t
 
     /* Shared data - these structures are accessed directly from p_vlc by
      * several modules */
-    input_channel_t *      p_channel;                /* channel library data */
 
     /* Locks */
     vlc_mutex_t            config_lock;          /* lock for the config file */
+#ifdef SYS_DARWIN
+    vlc_mutex_t            quicktime_lock;          /* QT is not thread safe on OSX */
+#endif
+
+    /* Structure storing the action name / key associations */
+    struct hotkey
+    {
+        const char *psz_action;
+        int i_action;
+        int i_key;
+
+    } *p_hotkeys;
 };
 
