@@ -2,7 +2,7 @@
  * stream_output.h : stream output module
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: stream_output.h,v 1.12 2003/08/09 14:59:24 gbazin Exp $
+ * $Id: stream_output.h,v 1.18 2003/12/07 17:09:33 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -27,6 +27,8 @@
  * sout_instance_t: stream output thread descriptor
  *****************************************************************************/
 
+#include "vlc_es.h"
+
 /*
  * i_allocated_size: size of allocated buffer
  * p_allocated_buffer: where data has been allocated
@@ -64,26 +66,6 @@ struct sout_buffer_t
     struct sout_buffer_t    *p_next;
 };
 
-struct sout_format_t
-{
-    int             i_cat;
-    vlc_fourcc_t    i_fourcc;
-
-    /* audio */
-    int             i_sample_rate;
-    int             i_channels;
-    int             i_block_align;
-
-    /* video */
-    int             i_width;
-    int             i_height;
-
-    int             i_bitrate;
-    int             i_extra_data;
-    uint8_t         *p_extra_data;
-
-};
-
 struct sout_fifo_t
 {
     vlc_mutex_t         lock;                         /* fifo data lock */
@@ -101,7 +83,7 @@ struct sout_input_t
 {
     sout_instance_t *p_sout;
 
-    sout_format_t   *p_fmt;
+    es_format_t     *p_fmt;
     sout_fifo_t     *p_fifo;
 
     void            *p_sys;
@@ -113,7 +95,7 @@ struct sout_packetizer_input_t
 
     sout_instance_t     *p_sout;
 
-    sout_format_t       *p_fmt;
+    es_format_t         *p_fmt;
 
     sout_stream_id_t    *id;
 };
@@ -142,6 +124,7 @@ struct sout_access_out_t
     int                     (* pf_write )( sout_access_out_t *,
                                            sout_buffer_t * );
 };
+
 /*
  * i_query parameter of pf_mux_capacity
  */
@@ -149,6 +132,9 @@ struct sout_access_out_t
 #define SOUT_MUX_CAP_GET_ADD_STREAM_ANY_TIME    0x01
 /* SOUT_MUX_CAP_GET_STREAMABLE:             p_args=NULL, p_answer=&boolean */
 #define SOUT_MUX_CAP_GET_STREAMABLE             0x02
+/* SOUT_MUX_CAP_GET_ADD_STREAM_WAIT:        p_args=NULL, p_answer=&boolean */
+#define SOUT_MUX_CAP_GET_ADD_STREAM_WAIT        0x03
+
 /*
  * return error code
  */
@@ -187,9 +173,11 @@ struct  sout_mux_t
     /* mux private */
     sout_mux_sys_t          *p_sys;
 
-//    /* creater private */
-//    void                    *p_sys_owner;
-
+#if 0
+    /* creater private */
+    void                    *p_sys_owner;
+#endif
+    
     /* XXX private to stream_output.c */
     /* if muxer doesn't support adding stream at any time then we first wait
      *  for stream then we refuse all stream and start muxing */
@@ -222,7 +210,7 @@ struct sout_stream_t
     char                    *psz_next;
 
     /* add, remove a stream */
-    sout_stream_id_t *      (*pf_add) ( sout_stream_t *, sout_format_t * );
+    sout_stream_id_t *      (*pf_add) ( sout_stream_t *, es_format_t * );
     int                     (*pf_del) ( sout_stream_t *, sout_stream_id_t * );
 
     /* manage a packet */
@@ -252,54 +240,6 @@ struct sout_instance_t
     sout_instance_sys_t     *p_sys;
 };
 
-/* some macro */
-#define TAB_APPEND( count, tab, p )             \
-    if( (count) > 0 )                           \
-    {                                           \
-        (tab) = realloc( (tab), sizeof( void ** ) * ( (count) + 1 ) ); \
-    }                                           \
-    else                                        \
-    {                                           \
-        (tab) = malloc( sizeof( void ** ) );    \
-    }                                           \
-    (void**)(tab)[(count)] = (void*)(p);        \
-    (count)++
-
-#define TAB_FIND( count, tab, p, index )        \
-    {                                           \
-        int _i_;                                \
-        (index) = -1;                           \
-        for( _i_ = 0; _i_ < (count); _i_++ )    \
-        {                                       \
-            if((void**)(tab)[_i_]==(void*)(p))  \
-            {                                   \
-                (index) = _i_;                  \
-                break;                          \
-            }                                   \
-        }                                       \
-    }
-
-#define TAB_REMOVE( count, tab, p )             \
-    {                                           \
-        int i_index;                            \
-        TAB_FIND( count, tab, p, i_index );     \
-        if( i_index >= 0 )                      \
-        {                                       \
-            if( count > 1 )                     \
-            {                                   \
-                memmove( ((void**)tab + i_index),    \
-                         ((void**)tab + i_index+1),  \
-                         ( (count) - i_index - 1 ) * sizeof( void* ) );\
-            }                                   \
-            else                                \
-            {                                   \
-                free( tab );                    \
-                (tab) = NULL;                   \
-            }                                   \
-            (count)--;                          \
-        }                                       \
-    }
-
 static inline sout_cfg_t *sout_cfg_find( sout_cfg_t *p_cfg, char *psz_name )
 {
     while( p_cfg && strcmp( p_cfg->psz_name, psz_name ) )
@@ -341,7 +281,7 @@ VLC_EXPORT( sout_buffer_t *, sout_FifoShow,       ( sout_fifo_t * ) );
 
 
 #define sout_InputNew( a, b ) __sout_InputNew( VLC_OBJECT(a), b )
-VLC_EXPORT( sout_packetizer_input_t *, __sout_InputNew,       ( vlc_object_t *, sout_format_t * ) );
+VLC_EXPORT( sout_packetizer_input_t *, __sout_InputNew,       ( vlc_object_t *, es_format_t * ) );
 VLC_EXPORT( int,            sout_InputDelete,      ( sout_packetizer_input_t * ) );
 VLC_EXPORT( int,            sout_InputSendBuffer,  ( sout_packetizer_input_t *, sout_buffer_t* ) );
 
@@ -358,7 +298,7 @@ VLC_EXPORT( int,                 sout_AccessOutSeek,   ( sout_access_out_t *, of
 VLC_EXPORT( int,                 sout_AccessOutWrite,  ( sout_access_out_t *, sout_buffer_t * ) );
 
 VLC_EXPORT( sout_mux_t *,       sout_MuxNew,          ( sout_instance_t*, char *, sout_access_out_t * ) );
-VLC_EXPORT( sout_input_t *,     sout_MuxAddStream,    ( sout_mux_t *, sout_format_t * ) );
+VLC_EXPORT( sout_input_t *,     sout_MuxAddStream,    ( sout_mux_t *, es_format_t * ) );
 VLC_EXPORT( void,               sout_MuxDeleteStream, ( sout_mux_t *, sout_input_t * ) );
 VLC_EXPORT( void,               sout_MuxDelete,       ( sout_mux_t * ) );
 VLC_EXPORT( void,               sout_MuxSendBuffer, ( sout_mux_t *, sout_input_t  *, sout_buffer_t * ) );
