@@ -120,9 +120,11 @@
 #if !defined( WIN32 ) && !defined( UNDER_CE )
 #   define DVD_DEVICE "/dev/dvd"
 #   define VCD_DEVICE "/dev/cdrom"
+#   define CDAUDIO_DEVICE "/dev/cdrom"
 #else
 #   define DVD_DEVICE "D:"
 #   define VCD_DEVICE "D:"
+#   define CDAUDIO_DEVICE "D:"
 #endif
 
 /*****************************************************************************
@@ -202,10 +204,6 @@
  * joined. */
 #define VOUT_MAX_AREAS                  5
 
-/* Default fonts */
-#define VOUT_DEFAULT_FONT               "default8x9.psf"
-#define VOUT_LARGE_FONT                 "default8x16.psf"
-
 /* Statistics are displayed every n loops (=~ pictures) */
 #define VOUT_STATS_NB_LOOPS             100
 
