@@ -2,7 +2,7 @@
  * vlc_objects.h: vlc_object_t definition.
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: vlc_objects.h,v 1.15 2003/01/27 17:41:01 ipkiss Exp $
+ * $Id: vlc_objects.h,v 1.21 2003/11/24 00:39:00 fenrir Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -21,6 +21,16 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
+/**
+ * \file
+ * This file defines the vlc_object_t structure and object types
+ */
+
+/** 
+ * \defgroup vlc_object Objects
+ * @{
+ */
+
 /* Object types */
 #define VLC_OBJECT_ROOT       (-1)
 #define VLC_OBJECT_VLC        (-2)
@@ -33,6 +43,11 @@
 #define VLC_OBJECT_VOUT       (-9)
 #define VLC_OBJECT_AOUT      (-10)
 #define VLC_OBJECT_SOUT      (-11)
+#define VLC_OBJECT_HTTPD     (-12)
+#define VLC_OBJECT_PACKETIZER (-13)
+#define VLC_OBJECT_ENCODER   (-14)
+#define VLC_OBJECT_DIALOGS   (-15)
+
 #define VLC_OBJECT_GENERIC  (-666)
 
 /* Object search mode */
@@ -45,6 +60,7 @@
 /*****************************************************************************
  * The vlc_object_t type. Yes, it's that simple :-)
  *****************************************************************************/
+/** The main vlc_object_t structure */
 struct vlc_object_t
 {
     VLC_COMMON_MEMBERS
@@ -64,6 +80,8 @@ VLC_EXPORT( void, __vlc_object_release, ( vlc_object_t * ) );
 VLC_EXPORT( vlc_list_t *, __vlc_list_find, ( vlc_object_t *, int, int ) );
 VLC_EXPORT( void, vlc_list_release, ( vlc_list_t * ) );
 
+/*}@*/
+
 #define vlc_object_create(a,b) \
     __vlc_object_create( VLC_OBJECT(a), b )
 
