@@ -4,7 +4,7 @@
  * Copyright (C) 1998-2003 VideoLAN
  *
  * Authors: Johan Bilien <jobi@via.ecp.fr>
- *          Jean-Paul Saman <saman@natlab.research.philips.com>
+ *          Jean-Paul Saman <jpsaman@wxs.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,10 @@ int E_(Open) ( vlc_object_t *p_this )
     char                dvr[] = DVR;
     char                frontend[] = FRONTEND;
     int                 i_len = 0;
+    vlc_value_t         val;
+    int                 i_test;
 
+    
     /* parse the options passed in command line : */
     psz_parser = strdup( p_input->psz_name );
 
@@ -112,6 +115,22 @@ int E_(Open) ( vlc_object_t *p_this )
     u_adapter = config_GetInt( p_input, "adapter" );
     u_device  = config_GetInt( p_input, "device" );
   
+    /* Get antenna configuration options */
+    b_diseqc = config_GetInt( p_input, "diseqc" );
+    u_lnb_lof1 = config_GetInt( p_input, "lnb-lof1" );
+    u_lnb_lof2 = config_GetInt( p_input, "lnb-lof2" );
+    u_lnb_slof = config_GetInt( p_input, "lnb-slof" );
+
+    /*Get modulation parameters*/
+    i_bandwidth = config_GetInt( p_input, "bandwidth");
+    i_code_rate_HP = config_GetInt(p_input, "code-rate-hp");
+    i_code_rate_LP = config_GetInt(p_input, "code-rate-lp");
+    i_modulation  = config_GetInt(p_input, "modulation");
+    i_transmission = config_GetInt(p_input, "transmission");
+    i_guard = config_GetInt(p_input, "guard");
+    i_hierarchy = config_GetInt(p_input, "hierarchy");
+
+
     /* Determine frontend device information and capabilities */
     b_probe = config_GetInt( p_input, "probe" );
     if (b_probe)
@@ -143,8 +162,12 @@ int E_(Open) ( vlc_object_t *p_this )
         else if (strncmp( p_input->psz_access, "terrestrial",11) ==0)
             frontend_info.type = FE_OFDM;
 
-        frontend_info.frequency_max =   12999000;
-        frontend_info.frequency_min =    9750000;
+//        frontend_info.frequency_max =   12999000; /* in KHz, lnb_lof2 */
+//        frontend_info.frequency_min =    9750000; /* lnb_lof1 */
+
+        frontend_info.frequency_max = u_lnb_lof2; /* in KHz, lnb_lof2 */
+        frontend_info.frequency_min = u_lnb_lof1; /* lnb_lof1 */
+
         frontend_info.symbol_rate_max = 30000000;
         frontend_info.symbol_rate_min =  1000000;
     }
@@ -155,35 +178,186 @@ int E_(Open) ( vlc_object_t *p_this )
     p_input->pf_set_area = SatelliteSetArea;
     p_input->pf_seek = SatelliteSeek;
 
-    u_freq = (unsigned int)strtol( psz_parser, &psz_next, 10 );
-    if( *psz_next )
+    i_test = strtol( psz_parser, &psz_next, 10 );
+    if ( psz_next == psz_parser )
     {
-        psz_parser = psz_next + 1;
-        b_polarisation = (vlc_bool_t)strtol( psz_parser, &psz_next, 10 );
+        for ( ;; )
+        {
+            if( !strncmp( psz_parser, "frequency=",
+                               strlen( "frequency=" ) ) )
+            {
+                u_freq =
+                  strtol( psz_parser + strlen( "frequency=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "polarization=",
+                               strlen( "polarization=" ) ) )
+            {
+                char *psz_parser_init;
+                psz_parser += strlen( "polarization=" );
+                psz_parser_init = psz_parser;
+                while ( *psz_parser != ':')
+                {
+                psz_parser++;
+                }
+                if ((!strncmp( psz_parser_init, "V" ,
+                     psz_parser - psz_parser_init ) ) || 
+                   (!strncmp( psz_parser_init, "V" ,
+                     psz_parser - psz_parser_init ) ) )
+                {
+                    b_polarisation = VLC_FALSE;
+                }
+                else if ((!strncmp( psz_parser_init, "H" ,
+                     psz_parser - psz_parser_init ) ) ||
+                   (!strncmp( psz_parser_init, "h" ,
+                     psz_parser - psz_parser_init ) ) )
+
+                {
+                    b_polarisation = VLC_TRUE;
+                }
+            }
+            else if( !strncmp( psz_parser, "fec=",
+                               strlen( "fec=" ) ) )
+            {
+                i_fec =
+                (int)strtol( psz_parser + strlen( "fec=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "srate=",
+                               strlen( "srate=" ) ) )
+            {
+                u_srate =
+                (unsigned int)strtol( psz_parser + strlen( "srate=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "program=",
+                               strlen( "program=" ) ) )
+            {
+                val.i_int = (int)strtol( psz_parser + strlen( "program=" ),
+                            &psz_parser, 0 );
+                var_Set( p_input, "program", val );
+            }
+            else if( !strncmp( psz_parser, "diseqc",
+                               strlen( "disecq" ) ) )
+            {
+                psz_parser += strlen("disecq");
+                b_diseqc = VLC_TRUE;
+            }
+            else if( !strncmp( psz_parser, "lnb-lof1=",
+                               strlen( "lnb-lof1=" ) ) )
+            {
+                u_lnb_lof1 =
+                (unsigned int)strtol( psz_parser + strlen( "lnb_lof1=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "lnb-lof2=",
+                               strlen( "lnb-lof2=" ) ) )
+            {
+                u_lnb_lof2 =
+                (unsigned int)strtol( psz_parser + strlen( "lnb_lof2=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "lnb-slof=",
+                               strlen( "lnb-slof=" ) ) )
+            {
+                u_lnb_slof =
+                (unsigned int)strtol( psz_parser + strlen( "lnb_slof=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "device=",
+                               strlen( "device=" ) ) )
+            {
+                u_device =
+                (unsigned int)strtol( psz_parser + strlen( "device=" ),
+                            &psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "adapter=",
+                               strlen( "adapter=" ) ) )
+            {
+                u_adapter =
+                (unsigned int)strtol( psz_parser + strlen( "adapter=" ),
+                            &psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "modulation=",
+                               strlen( "modulation=" ) ) )
+            {
+                i_modulation = (int)strtol( psz_parser + strlen( "modulation=" ),
+                            &psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "bandwidth=",
+                               strlen( "bandwidth=" ) ) )
+            {
+                i_bandwidth = (int)strtol( psz_parser + strlen( "bandwidth=" ),
+                            &psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "guard=",
+                               strlen( "guard=" ) ) )
+            {
+                i_guard = (int)strtol( psz_parser + strlen( "guard=" ),
+                            &psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "transmission=",
+                               strlen( "transmission=" ) ) )
+            {
+                i_transmission = (int)strtol( psz_parser + 
+                            strlen( "transmission=" ),&psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "hierarchy=",
+                               strlen( "hierarchy=" ) ) )
+            {
+                i_hierarchy = (int)strtol( psz_parser + 
+                                strlen( "hierarchy=" ),&psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "code-rate-HP=",
+                               strlen( "code-rate-HP=" ) ) )
+            {
+                i_code_rate_HP = (int)strtol( psz_parser + 
+                                strlen( "code-rate-HP=" ),&psz_parser, 0 );
+            }
+            else if (!strncmp( psz_parser, "code-rate-LP=",
+                               strlen( "code-rate-LP=" ) ) )
+            {
+                i_code_rate_LP = (int)strtol( psz_parser +
+                                strlen( "code-rate-LP=" ),&psz_parser, 0 );
+            }
+            else if( !strncmp( psz_parser, "probe",
+                               strlen( "probe" ) ) )
+            {
+                psz_parser += strlen("probe");
+                b_probe = VLC_TRUE;
+            }
+            if( *psz_parser )
+                psz_parser++;
+            else
+            break;
+        }
+    }
+    else
+    {
+        msg_Warn(p_input,"DVB Input syntax has changed, please see documentation for further informations");
+        u_freq = (unsigned int)i_test;
         if( *psz_next )
         {
             psz_parser = psz_next + 1;
-            i_fec = (int)strtol( psz_parser, &psz_next, 10 );
+            b_polarisation = (vlc_bool_t)strtol( psz_parser, &psz_next, 10 );
             if( *psz_next )
             {
                 psz_parser = psz_next + 1;
-                u_srate = (unsigned int)strtol( psz_parser, &psz_next, 10 );
+                i_fec = (int)strtol( psz_parser, &psz_next, 10 );
+                if( *psz_next )
+                {
+                    psz_parser = psz_next + 1;
+                    u_srate = (unsigned int)strtol( psz_parser, &psz_next, 10 );
+                }
             }
         }
     }
-
-    /* Workaround for backwards compatibility */
-    if (strncmp( p_input->psz_access, "satellite",9 ) ==0)
-    {
-        msg_Warn( p_input, "invalid frequency %d possibly in kHz, trying value *1000 Hz", u_freq );
-        u_freq *= 1000;
-    }
-    
+   
     /* Validating input values */
     if ( ((u_freq) > frontend_info.frequency_max) ||
          ((u_freq) < frontend_info.frequency_min) )
     {
-        msg_Warn( p_input, "invalid frequency %d (Hz), using default one", u_freq );
+        msg_Warn( p_input, "invalid frequency %d (KHz), using default one", u_freq );
         u_freq = config_GetInt( p_input, "frequency" );
         if ( ((u_freq) > frontend_info.frequency_max) ||
              ((u_freq) < frontend_info.frequency_min) )
@@ -196,7 +370,7 @@ int E_(Open) ( vlc_object_t *p_this )
     /* Workaround for backwards compatibility */
     if (strncmp( p_input->psz_access, "satellite", 9 ) ==0)
     {
-        msg_Warn( p_input, "invalid symbol rate %d possibly specified in kHz, trying value *1000 Hz", u_freq );
+        msg_Warn( p_input, "invalid symbol rate %d possibly specified in MHz, trying value *1000 KHz", u_freq );
         u_srate *= 1000;
     }
     
@@ -235,47 +409,45 @@ int E_(Open) ( vlc_object_t *p_this )
         }
     }
 
-    /* Get antenna configuration options */
-    b_diseqc = config_GetInt( p_input, "diseqc" );
-    u_lnb_lof1 = config_GetInt( p_input, "lnb-lof1" );
-    u_lnb_lof2 = config_GetInt( p_input, "lnb-lof2" );
-    u_lnb_slof = config_GetInt( p_input, "lnb-slof" );
-
     /* Setting frontend parameters for tuning the hardware */      
+    msg_Dbg( p_input, "Trying to tune to channel ...");
     switch( frontend_info.type )
     {
         /* DVB-S: satellite and budget cards (nova) */
         case FE_QPSK:
-            fep.frequency = u_freq;
+            fep.frequency = u_freq; /* KHz */
             fep.inversion = dvb_DecodeInversion(p_input, (int) b_polarisation);
             fep.u.qpsk.symbol_rate = u_srate;
             fep.u.qpsk.fec_inner = dvb_DecodeFEC(p_input, i_fec); 
             msg_Dbg( p_input, "satellite (QPSK) frontend found on %s", frontend_info.name );
+
+            if (ioctl_SetQPSKFrontend (p_input, fep, b_polarisation,
+                               u_lnb_lof1, u_lnb_lof2, u_lnb_slof,
+                               u_adapter, u_device )<0)
+            {
+                msg_Err( p_input, "DVB-S frontend returned a failure event" );
+                return -1;
+            }
             break;
             
         /* DVB-C */
         case FE_QAM:
-            i_modulation  = config_GetInt(p_input, "modulation");
-
-            fep.frequency = u_freq;
+            fep.frequency = u_freq; /* KHz */
             fep.inversion = dvb_DecodeInversion(p_input, (int) b_polarisation);
             fep.u.qam.symbol_rate = u_srate;
             fep.u.qam.fec_inner = dvb_DecodeFEC(p_input, i_fec); 
             fep.u.qam.modulation = dvb_DecodeModulation(p_input, i_modulation); 
             msg_Dbg( p_input, "cable (QAM) frontend found on %s", frontend_info.name );
+            if (ioctl_SetQAMFrontend (p_input, fep, u_adapter, u_device )<0)
+            {
+                msg_Err( p_input, "DVB-C frontend returned a failure event" );
+                return -1;
+            }
             break;
 
         /* DVB-T */
         case FE_OFDM:
-            i_bandwidth = config_GetInt( p_input, "bandwidth");
-            i_code_rate_HP = config_GetInt(p_input, "code-rate-hp");
-            i_code_rate_LP = config_GetInt(p_input, "code-rate-lp");
-            i_modulation  = config_GetInt(p_input, "modulation");
-            i_transmission = config_GetInt(p_input, "transmission");
-            i_guard = config_GetInt(p_input, "guard");
-            i_hierarchy = config_GetInt(p_input, "hierarchy");
-            
-            fep.frequency = u_freq;
+            fep.frequency = u_freq; /* KHz */
             fep.inversion = dvb_DecodeInversion(p_input, (int) b_polarisation);
             fep.u.ofdm.bandwidth = dvb_DecodeBandwidth(p_input, i_bandwidth);
             fep.u.ofdm.code_rate_HP = dvb_DecodeFEC(p_input, i_code_rate_HP); 
@@ -285,12 +457,17 @@ int E_(Open) ( vlc_object_t *p_this )
             fep.u.ofdm.guard_interval = dvb_DecodeGuardInterval(p_input, i_guard);
             fep.u.ofdm.hierarchy_information = dvb_DecodeHierarchy(p_input, i_hierarchy);
             msg_Dbg( p_input, "terrestrial (OFDM) frontend found on %s", frontend_info.name );
+            if (ioctl_SetOFDMFrontend (p_input, fep,u_adapter, u_device )<0)
+            {
+                msg_Err( p_input, "DVB-T frontend returned a failure event" );
+                return -1;
+            }
             break;
-
         default:
             msg_Err( p_input, "Could not determine frontend type on %s", frontend_info.name );
             return -1;
     }
+    msg_Dbg( p_input, "Tuning done.");
 
     /* Initialise structure */
     p_satellite = malloc( sizeof( input_socket_t ) );
@@ -324,42 +501,9 @@ int E_(Open) ( vlc_object_t *p_this )
         return -1;
     }
 
-    /* Initialize the Satellite Card */
-    switch (ioctl_SetFrontend (p_input, fep, b_polarisation,
-                               u_lnb_lof1, u_lnb_lof2, u_lnb_slof,
-                               u_adapter, u_device ))
-    {
-        case -2:
-            msg_Err( p_input, "frontend returned an unexpected event" );
-            close( p_satellite->i_handle );
-            free( p_satellite );
-            return -1;
-        case -3:
-            msg_Err( p_input, "frontend returned no event" );
-            close( p_satellite->i_handle );
-            free( p_satellite );
-            return -1;
-        case -4:
-            msg_Err( p_input, "frontend: timeout when polling for event" );
-            close( p_satellite->i_handle );
-            free( p_satellite );
-            return -1;
-        case -5:
-            msg_Err( p_input, "an error occured when polling frontend device" );
-            close( p_satellite->i_handle );
-            free( p_satellite );
-            return -1;
-        case -1:
-            msg_Err( p_input, "frontend returned a failure event" );
-            close( p_satellite->i_handle );
-            free( p_satellite );
-            return -1;
-        default:
-            break;
-    }
-
     msg_Dbg( p_input, "setting filter on PAT" );
 
+    /* Set Filter on PAT packet */
     if ( ioctl_SetDMXFilter(p_input, 0, &i_fd, 21, u_adapter, u_device ) < 0 )
     {
 #   ifdef HAVE_ERRNO_H
@@ -410,7 +554,7 @@ void E_(Close) ( vlc_object_t *p_this )
                 i_es_index ++ )
         {
 #define p_es p_input->stream.p_selected_program->pp_es[i_es_index]
-            if ( p_es->p_decoder_fifo )
+            if ( p_es->p_dec )
             {
                 ioctl_UnsetDMXFilter(p_input, p_es->i_demux_fd );
             }
@@ -486,7 +630,7 @@ int SatelliteSetProgram( input_thread_t    * p_input,
     unsigned int u_video_type = 1; /* default video type */
     unsigned int u_audio_type = 2; /* default audio type */
 
-    // Get adapter and device number to use for this dvb card
+    /* Get adapter and device number to use for this dvb card */
     u_adapter = config_GetInt( p_input, "adapter" );
     u_device  = config_GetInt( p_input, "device" );
 
@@ -497,11 +641,11 @@ int SatelliteSetProgram( input_thread_t    * p_input,
                 i_es_index ++ )
         {
 #define p_es p_input->stream.p_selected_program->pp_es[i_es_index]
-            if ( p_es->p_decoder_fifo )
+            if ( p_es->p_dec )
             {
                 input_UnselectES( p_input , p_es );
             }
-            if ( p_es->i_demux_fd )
+            if ( p_es->i_demux_fd > 0)
             {
                 ioctl_UnsetDMXFilter(p_input, p_es->i_demux_fd );
                 p_es->i_demux_fd = 0;
