@@ -2,7 +2,7 @@
  * http.c: HTTP access plug-in
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: http.c,v 1.32 2003/03/30 18:14:35 gbazin Exp $
+ * $Id: http.c,v 1.47 2003/12/21 23:32:58 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -25,7 +25,6 @@
  * Preamble
  *****************************************************************************/
 #include <stdlib.h>
-#include <string.h>
 #include <vlc/vlc.h>
 #include <vlc/input.h>
 
@@ -71,13 +70,13 @@ static ssize_t Read    ( input_thread_t *, byte_t *, size_t );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define PROXY_TEXT N_("specify an HTTP proxy")
+#define PROXY_TEXT N_("Specify an HTTP proxy")
 #define PROXY_LONGTEXT N_( \
     "Specify an HTTP proxy to use. It must be in the form " \
-    "http://myproxy.mydomain:myport. If none is specified, the HTTP_PROXY " \
+    "http://myproxy.mydomain:myport/. If none is specified, the HTTP_PROXY " \
     "environment variable will be tried." )
 
-#define CACHING_TEXT N_("caching value in ms")
+#define CACHING_TEXT N_("Caching value in ms")
 #define CACHING_LONGTEXT N_( \
     "Allows you to modify the default caching value for http streams. This " \
     "value should be set in miliseconds units." )
@@ -86,6 +85,8 @@ vlc_module_begin();
     add_category_hint( N_("http"), NULL, VLC_FALSE );
     add_string( "http-proxy", NULL, NULL, PROXY_TEXT, PROXY_LONGTEXT, VLC_FALSE );
     add_integer( "http-caching", 4 * DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT, CACHING_LONGTEXT, VLC_TRUE );
+    add_string( "http-user", NULL, NULL, "HTTP user name", "HTTP user name for Basic Authentification", VLC_FALSE );
+    add_string( "http-pwd", NULL , NULL, "HTTP password", "HTTP password for Basic Authentification", VLC_FALSE );
     set_description( _("HTTP input") );
     set_capability( "access", 0 );
     add_shortcut( "http" );
@@ -108,6 +109,7 @@ typedef struct _input_socket_s
     char *              psz_network;
     network_socket_t    socket_desc;
     char                psz_buffer[MAX_QUERY_SIZE];
+    char                psz_auth_string[MAX_QUERY_SIZE];
     char *              psz_name;
 } _input_socket_t;
 
@@ -120,6 +122,7 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
     _input_socket_t * p_access_data;
     module_t * p_network;
     char * psz_parser, * psz_value, * psz_answer;
+    byte_t * p_bytes;
     int i_code, i_ret, i, i_size;
 
     enum { HTTP_PROTOCOL, ICY_PROTOCOL } i_protocol;
@@ -145,15 +148,21 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
          snprintf( psz_buffer, MAX_QUERY_SIZE,
                    "%s"
                    "Range: bytes="I64Fd"-\r\n"
-                   HTTP_USERAGENT HTTP_END,
-                   p_access_data->psz_buffer, i_tell );
+                   HTTP_USERAGENT
+                   "%s"
+                   "Connection: Close\r\n"
+                   HTTP_END,
+                   p_access_data->psz_buffer, i_tell, p_access_data->psz_auth_string );
     }
     else
     {
          snprintf( psz_buffer, MAX_QUERY_SIZE,
                    "%s"
-                   HTTP_USERAGENT HTTP_END,
-                   p_access_data->psz_buffer );
+                   HTTP_USERAGENT
+                   "%s"
+                   "Connection: Close\r\n"
+                   HTTP_END,
+                   p_access_data->psz_buffer, p_access_data->psz_auth_string );
     }
     psz_buffer[MAX_QUERY_SIZE - 1] = '\0';
 
@@ -188,7 +197,8 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
     }
 
     /* Get the HTTP returncode */
-    i_size = input_Peek( p_input, (byte_t**)&psz_parser, MAX_ANSWER_SIZE );
+    i_size = input_Peek( p_input, &p_bytes, MAX_ANSWER_SIZE );
+    psz_parser = (char *)p_bytes;
 
     if( i_size <= 0 )
     {
@@ -225,6 +235,17 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
         return VLC_EGENERIC;
     }
 
+    /* Check for buggy Icecast servers */
+    if( strstr( psz_parser , "x-audiocast") )
+    {
+        i_protocol = ICY_PROTOCOL;
+        if( !p_input->psz_demux || !*p_input->psz_demux  )
+        {
+            msg_Info( p_input, "ICY server found, mp3 demuxer selected" );
+            p_input->psz_demux = "mp3";    // FIXME strdup ?
+        }
+    }
+
     /* Check the HTTP return code */
     i_code = atoi( (char*)psz_parser );
     msg_Dbg( p_input, "%s server replied: %i",
@@ -260,7 +281,8 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
     {
         char psz_line[MAX_ANSWER_SIZE];
 
-        i_size = input_Peek( p_input, (byte_t**)&psz_parser, MAX_ANSWER_SIZE );
+        i_size = input_Peek( p_input, &p_bytes, MAX_ANSWER_SIZE );
+        psz_parser = (char *)p_bytes;
 
         if( i_size <= 0 )
         {
@@ -303,17 +325,8 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
         if( !strcasecmp( psz_line, "Content-Length" ) )
         {
             off_t i_size = 0;
-#ifdef HAVE_ATOLL
+
             i_size = i_tell + atoll( psz_value );
-#else
-            psz_parser = psz_value;
-            while( psz_parser[0] >= '0' && psz_parser[0] <= '9' )
-            {
-                i_size *= 10;
-                i_size += psz_parser[0] - '0';
-            }
-            i_size += i_tell;
-#endif
             msg_Dbg( p_input, "stream size is "I64Fd, i_size );
 
             vlc_mutex_lock( &p_input->stream.stream_lock );
@@ -337,7 +350,7 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
                               i_code, psz_answer, psz_value );
             p_playlist->pp_items[p_playlist->i_index]->b_autodeletion
                                                                   = VLC_TRUE;
-            playlist_Add( p_playlist, psz_value,
+            playlist_Add( p_playlist, psz_value, NULL, 0,
                           PLAYLIST_INSERT | PLAYLIST_GO,
                           p_playlist->i_index + 1 );
             vlc_object_release( p_playlist );
@@ -376,12 +389,74 @@ static int HTTPConnect( input_thread_t * p_input, off_t i_tell )
     {
         p_input->stream.b_seekable = VLC_FALSE;
     }
+    if( i_code != 206 )
+    {
+        p_input->stream.b_seekable = VLC_FALSE;
+    }
     p_input->stream.b_changed = VLC_TRUE;
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
     return VLC_SUCCESS;
 }
 
+/*****************************************************************************
+ * Encode a string in base64
+ * Code borrowed from Rafael Steil
+ *****************************************************************************/
+void encodeblock( unsigned char in[3], unsigned char out[4], int len )
+{
+    static const char cb64[]
+        = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+    out[0] = cb64[ in[0] >> 2 ];
+    out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
+    out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] : '=');
+    out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
+}
+
+char *str_base64_encode(char *psz_str, input_thread_t *p_input )
+{
+    unsigned char in[3], out[4];
+    unsigned int i, len, blocksout = 0, linesize = strlen(psz_str);
+    char *psz_tmp = psz_str;
+    char *psz_result = (char *)malloc( linesize / 3 * 4 + 5 );
+
+    if( !psz_result )
+    {
+        msg_Err( p_input, "out of memory" );
+        return NULL;
+    }
+
+    while( *psz_tmp )
+    {
+        len = 0;
+
+        for( i = 0; i < 3; i++ )
+        {
+            in[i] = (unsigned char)*psz_tmp;
+
+            if (*psz_tmp)
+                len++;
+            else
+                in[i] = 0;
+
+            psz_tmp++;
+        }
+
+        if( len )
+        {
+            encodeblock( in, out, len );
+
+            for( i = 0; i < 4; i++ )
+            {
+                psz_result[blocksout++] = out[i];
+            }
+        }
+    }
+
+    psz_result[blocksout] = '\0';
+    return psz_result;
+}
+
 /*****************************************************************************
  * Open: parse URL and open the remote file at the beginning
  *****************************************************************************/
@@ -390,12 +465,14 @@ static int Open( vlc_object_t *p_this )
     input_thread_t *    p_input = (input_thread_t *)p_this;
     _input_socket_t *   p_access_data;
     char *              psz_name = strdup(p_input->psz_name);
-    char *              psz_parser = psz_name;
+    char *              psz_parser = psz_name, * psz_auth_parser;
     char *              psz_server_addr = "";
     char *              psz_server_port = "";
     char *              psz_path = "";
     char *              psz_proxy, *psz_proxy_orig;
+    char *              psz_user = NULL, *psz_pwd = NULL;
     int                 i_server_port = 0;
+    vlc_value_t         val;
 
     p_access_data = malloc( sizeof(_input_socket_t) );
     p_input->p_access_data = (access_sys_t *)p_access_data;
@@ -408,11 +485,17 @@ static int Open( vlc_object_t *p_this )
 
     p_access_data->psz_name = psz_name;
     p_access_data->psz_network = "";
-    if( config_GetInt( p_input, "ipv4" ) )
+    memset(p_access_data->psz_auth_string, 0, MAX_QUERY_SIZE);
+
+    var_Create( p_input, "ipv4", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_input, "ipv4", &val );
+    if( val.i_int )
     {
         p_access_data->psz_network = "ipv4";
     }
-    if( config_GetInt( p_input, "ipv6" ) )
+    var_Create( p_input, "ipv6", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_input, "ipv6", &val );
+    if( val.i_int )
     {
         p_access_data->psz_network = "ipv6";
     }
@@ -430,11 +513,39 @@ static int Open( vlc_object_t *p_this )
     }
 
     /* Parse psz_name syntax :
-     * //<hostname>[:<port>][/<path>] */
+     * //[user:password]@<hostname>[:<port>][/<path>] */
+
+
     while( *psz_parser == '/' )
     {
         psz_parser++;
     }
+    psz_auth_parser = psz_parser;
+
+    while ( *psz_auth_parser != '@' && *psz_auth_parser != '\0' )
+    {
+        psz_auth_parser++;
+    }
+    if ( *psz_auth_parser == '@' )
+    {
+        psz_user = psz_parser;
+        while ( *psz_parser != ':' && psz_parser < psz_auth_parser )
+        {
+            psz_parser++;
+        }
+        if ( psz_parser != psz_auth_parser )
+        {
+            *psz_parser = '\0';
+            psz_pwd = psz_parser + 1;
+        }
+        else
+        {
+            psz_pwd = "";
+        }
+        *psz_auth_parser = '\0';
+        psz_parser = psz_auth_parser + 1;
+    }
+
     psz_server_addr = psz_parser;
 
     while( *psz_parser && *psz_parser != ':' && *psz_parser != '/' )
@@ -495,8 +606,34 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
+    /* Handle autehtification */
+
+   if ( !psz_user )
+    {
+        var_Create( p_input, "http-user", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+        var_Get( p_input, "http-user", &val );
+        psz_user = val.psz_string;
+
+        var_Create( p_input, "http-pwd", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+        var_Get( p_input, "http-pwd", &val );
+        psz_pwd = val.psz_string;
+    }
+
+    if ( *psz_user )
+    {
+        char psz_user_pwd[MAX_QUERY_SIZE];
+        msg_Dbg( p_input, "authenticating, user=%s, password=%s",
+                                           psz_user, psz_pwd );
+        snprintf( psz_user_pwd, MAX_QUERY_SIZE, "%s:%s", psz_user, psz_pwd );
+        snprintf( p_access_data->psz_auth_string, MAX_QUERY_SIZE,
+                  "Authorization: Basic %s\r\n",
+                  str_base64_encode( psz_user_pwd, p_input ) );
+    }
+
     /* Check proxy config variable */
-    psz_proxy_orig = config_GetPsz( p_input, "http-proxy" );
+    var_Create( p_input, "http-proxy", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    var_Get( p_input, "http-proxy", &val );
+    psz_proxy_orig = val.psz_string;
     if( psz_proxy_orig == NULL )
     {
         /* Check proxy environment variable */
@@ -581,6 +718,7 @@ static int Open( vlc_object_t *p_this )
         p_access_data->socket_desc.psz_server_addr = psz_proxy;
         p_access_data->socket_desc.i_server_port = i_proxy_port;
         p_access_data->socket_desc.i_type = NETWORK_TCP;
+        p_access_data->socket_desc.i_ttl           = 0;
 
         snprintf( p_access_data->psz_buffer, MAX_QUERY_SIZE,
                   "GET http://%s:%d/%s HTTP/1.0\r\n",
@@ -592,6 +730,7 @@ static int Open( vlc_object_t *p_this )
         p_access_data->socket_desc.i_type = NETWORK_TCP;
         p_access_data->socket_desc.psz_server_addr = psz_server_addr;
         p_access_data->socket_desc.i_server_port = i_server_port;
+        p_access_data->socket_desc.i_ttl           = 0;
 
         snprintf( p_access_data->psz_buffer, MAX_QUERY_SIZE,
                   "GET /%s HTTP/1.1\r\nHost: %s\r\n",
@@ -637,7 +776,10 @@ static int Open( vlc_object_t *p_this )
     }
 
     /* Update default_pts to a suitable value for http access */
-    p_input->i_pts_delay = config_GetInt( p_input, "http-caching" ) * 1000;
+
+    var_Create( p_input, "http-caching", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_input, "http-caching", &val );
+    p_input->i_pts_delay = val.i_int * 1000;
 
     return VLC_SUCCESS;
 }
