@@ -2,7 +2,7 @@
  * alsa.c : alsa plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: alsa.c,v 1.34 2003/08/03 23:11:21 gbazin Exp $
+ * $Id: alsa.c,v 1.36 2003/10/25 00:49:13 sam Exp $
  *
  * Authors: Henri Fallon <henri@videolan.org> - Original Author
  *          Jeffrey Baker <jwbaker@acm.org> - Port to ALSA 1.0 API
@@ -70,7 +70,7 @@ struct aout_sys_t
 /* These values are in frames.
    To convert them to a number of bytes you have to multiply them by the
    number of channel(s) (eg. 2 for stereo) and the size of a sample (eg.
-   2 for s16). */
+   2 for int16_t). */
 #define ALSA_DEFAULT_PERIOD_SIZE        1024
 #define ALSA_DEFAULT_BUFFER_SIZE        ( ALSA_DEFAULT_PERIOD_SIZE << 8 )
 #define ALSA_SPDIF_PERIOD_SIZE          A52_FRAME_NB
@@ -726,15 +726,20 @@ static void ALSAFill( aout_instance_t * p_aout )
                 msleep( p_sys->i_period_time >> 1 );
                 return;
             }
+
+            /* Underrun, try to recover as quickly as possible */
+            next_date = mdate();
         }
+        else
+        {
+            /* Here the device should be either in the RUNNING state.
+             * p_status is valid. */
 
-        /* Here the device should be either in the RUNNING state either in
-           the PREPARE state. p_status is valid. */
-
-        snd_pcm_status_get_tstamp( p_status, &ts_next );
-        next_date = (mtime_t)ts_next.tv_sec * 1000000 + ts_next.tv_usec;
-        next_date += (mtime_t)snd_pcm_status_get_delay(p_status)
-                     * 1000000 / p_aout->output.output.i_rate;
+            snd_pcm_status_get_tstamp( p_status, &ts_next );
+            next_date = (mtime_t)ts_next.tv_sec * 1000000 + ts_next.tv_usec;
+            next_date += (mtime_t)snd_pcm_status_get_delay(p_status)
+                * 1000000 / p_aout->output.output.i_rate;
+        }
 
         p_buffer = aout_OutputNextBuffer( p_aout, next_date,
                         (p_aout->output.output.i_format ==
