@@ -2,15 +2,16 @@
  * parse.c: SPU parser
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: parse.c,v 1.13 2003/07/29 22:25:40 gbazin Exp $
+ * $Id: parse.c,v 1.16 2003/11/24 00:39:01 fenrir Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
+ *          Laurent Aimar <fenrir@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -24,9 +25,6 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>                                      /* malloc(), free() */
-#include <string.h>                                    /* memcpy(), memset() */
-
 #include <vlc/vlc.h>
 #include <vlc/vout.h>
 #include <vlc/decoder.h>
@@ -36,8 +34,8 @@
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
-static int  ParseControlSeq  ( spudec_thread_t *, subpicture_t * );
-static int  ParseRLE         ( spudec_thread_t *, subpicture_t *, uint8_t * );
+static int  ParseControlSeq  ( decoder_t *, subpicture_t * );
+static int  ParseRLE         ( decoder_t *, subpicture_t * );
 
 static void DestroySPU       ( subpicture_t * );
 
@@ -61,62 +59,20 @@ static inline unsigned int AddNibble( unsigned int i_code,
     }
 }
 
-/*****************************************************************************
- * SyncPacket: get in sync with the stream
- *****************************************************************************
- * This function makes a few sanity checks and returns 0 if it looks like we
- * are at the beginning of a subpicture packet.
- *****************************************************************************/
-int E_(SyncPacket)( spudec_thread_t *p_spudec )
-{
-    /* Re-align the buffer on an 8-bit boundary */
-    RealignBits( &p_spudec->bit_stream );
-
-    /* The total SPU packet size, often bigger than a PS packet */
-    p_spudec->i_spu_size = GetBits( &p_spudec->bit_stream, 16 );
-
-    /* The RLE stuff size (remove 4 because we just read 32 bits) */
-    p_spudec->i_rle_size = ShowBits( &p_spudec->bit_stream, 16 ) - 4;
-
-    /* If the values we got are a bit strange, skip packet */
-    if( !p_spudec->i_spu_size
-         || ( p_spudec->i_rle_size >= p_spudec->i_spu_size ) )
-    {
-        return VLC_EGENERIC;
-    }
-
-    RemoveBits( &p_spudec->bit_stream, 16 );
-
-    return VLC_SUCCESS;
-}
-
 /*****************************************************************************
  * ParsePacket: parse an SPU packet and send it to the video output
  *****************************************************************************
  * This function parses the SPU packet and, if valid, sends it to the
  * video output.
  *****************************************************************************/
-void E_(ParsePacket)( spudec_thread_t *p_spudec )
+void E_(ParsePacket)( decoder_t *p_dec)
 {
-    subpicture_t * p_spu;
-    uint8_t      * p_src;
-    unsigned int   i_offset;
-    mtime_t        i_pts;
+    decoder_sys_t *p_sys = p_dec->p_sys;
 
-    msg_Dbg( p_spudec->p_fifo, "trying to gather a 0x%.2x long subtitle",
-                               p_spudec->i_spu_size );
-
-    /* We cannot display a subpicture with no date */
-    NextPTS( &p_spudec->bit_stream, &i_pts, NULL );
-    if( i_pts == 0 )
-    {
-        msg_Warn( p_spudec->p_fifo, "subtitle without a date" );
-        return;
-    }
+    subpicture_t  *p_spu;
 
     /* Allocate the subpicture internal data. */
-    p_spu = vout_CreateSubPicture( p_spudec->p_vout, MEMORY_SUBPICTURE );
-
+    p_spu = vout_CreateSubPicture( p_sys->p_vout, MEMORY_SUBPICTURE );
     if( p_spu == NULL )
     {
         return;
@@ -126,17 +82,10 @@ void E_(ParsePacket)( spudec_thread_t *p_spudec )
      * expand the RLE stuff so that we won't need to read nibbles later
      * on. This will speed things up a lot. Plus, we'll only need to do
      * this stupid interlacing stuff once. */
-    p_spu->p_sys = malloc( sizeof( subpicture_sys_t )
-                            + p_spudec->i_rle_size * 4 );
-
-    if( p_spu->p_sys == NULL )
-    {
-        vout_DestroySubPicture( p_spudec->p_vout, p_spu );
-        return;
-    }
+    p_spu->p_sys = malloc( sizeof( subpicture_sys_t ) + 4*p_sys->i_rle_size );
 
     /* Fill the p_spu structure */
-    vlc_mutex_init( p_spudec->p_fifo, &p_spu->p_sys->lock );
+    vlc_mutex_init( p_dec, &p_spu->p_sys->lock );
 
     p_spu->pf_render = E_(RenderSPU);
     p_spu->pf_destroy = DestroySPU;
@@ -151,10 +100,10 @@ void E_(ParsePacket)( spudec_thread_t *p_spudec )
     p_spu->p_sys->b_crop = VLC_FALSE;
 
     /* Get display time now. If we do it later, we may miss the PTS. */
-    p_spu->p_sys->i_pts = i_pts;
+    p_spu->p_sys->i_pts = p_sys->i_pts;
 
     /* Attach to our input thread */
-    p_spu->p_sys->p_input = vlc_object_find( p_spudec->p_fifo,
+    p_spu->p_sys->p_input = vlc_object_find( p_dec,
                                              VLC_OBJECT_INPUT, FIND_PARENT );
     if( p_spu->p_sys->p_input )
     {
@@ -173,70 +122,30 @@ void E_(ParsePacket)( spudec_thread_t *p_spudec )
         }
     }
 
-    /* Allocate the temporary buffer we will parse */
-    p_src = malloc( p_spudec->i_rle_size );
-
-    if( p_src == NULL )
-    {
-        msg_Err( p_spudec->p_fifo, "out of memory" );
-        vout_DestroySubPicture( p_spudec->p_vout, p_spu );
-        return;
-    }
-
-    /* Get RLE data */
-    for( i_offset = 0; i_offset < p_spudec->i_rle_size;
-         i_offset += SPU_CHUNK_SIZE )
-    {
-        GetChunk( &p_spudec->bit_stream, p_src + i_offset,
-                  ( i_offset + SPU_CHUNK_SIZE < p_spudec->i_rle_size ) ?
-                  SPU_CHUNK_SIZE : p_spudec->i_rle_size - i_offset );
-
-        /* Abort subtitle parsing if we were requested to stop */
-        if( p_spudec->p_fifo->b_die )
-        {
-            free( p_src );
-            vout_DestroySubPicture( p_spudec->p_vout, p_spu );
-            return;
-        }
-    }
-
-#if 0
-    /* Dump the subtitle info */
-    intf_WarnHexDump( 5, p_spu->p_sys->p_data, p_spudec->i_rle_size );
-#endif
-
     /* Getting the control part */
-    if( ParseControlSeq( p_spudec, p_spu ) )
+    if( ParseControlSeq( p_dec, p_spu ) )
     {
         /* There was a parse error, delete the subpicture */
-        free( p_src );
-        vout_DestroySubPicture( p_spudec->p_vout, p_spu );
+        vout_DestroySubPicture( p_sys->p_vout, p_spu );
         return;
     }
 
-    /* At this point, no more GetBit() command is needed, so we have all
-     * the data we need to tell whether the subtitle is valid. Thus we
-     * try to display it and we ignore b_die. */
-
-    if( ParseRLE( p_spudec, p_spu, p_src ) )
+     /* We try to display it */
+    if( ParseRLE( p_dec, p_spu ) )
     {
         /* There was a parse error, delete the subpicture */
-        free( p_src );
-        vout_DestroySubPicture( p_spudec->p_vout, p_spu );
+        vout_DestroySubPicture( p_sys->p_vout, p_spu );
         return;
     }
 
-    msg_Dbg( p_spudec->p_fifo, "total size: 0x%x, RLE offsets: 0x%x 0x%x",
-             p_spudec->i_spu_size,
+    msg_Dbg( p_dec, "total size: 0x%x, RLE offsets: 0x%x 0x%x",
+             p_sys->i_spu_size,
              p_spu->p_sys->pi_offset[0], p_spu->p_sys->pi_offset[1] );
 
     /* SPU is finished - we can ask the video output to display it */
-    vout_DisplaySubPicture( p_spudec->p_vout, p_spu );
+    vout_DisplaySubPicture( p_sys->p_vout, p_spu );
 
     /* TODO: do stuff! */
-
-    /* Clean up */
-    free( p_src );
 }
 
 /*****************************************************************************
@@ -246,11 +155,12 @@ void E_(ParsePacket)( spudec_thread_t *p_spudec )
  * information, coordinates, and so on. For more information on the
  * subtitles format, see http://sam.zoy.org/doc/dvd/subtitles/index.html
  *****************************************************************************/
-static int ParseControlSeq( spudec_thread_t *p_spudec,
-                                  subpicture_t * p_spu )
+static int ParseControlSeq( decoder_t *p_dec, subpicture_t * p_spu )
 {
+    decoder_sys_t *p_sys = p_dec->p_sys;
+
     /* Our current index in the SPU packet */
-    unsigned int i_index = p_spudec->i_rle_size + 4;
+    unsigned int i_index = p_sys->i_rle_size + 4;
 
     /* The next start-of-control-sequence index and the previous one */
     unsigned int i_next_seq = 0, i_cur_seq = 0;
@@ -267,25 +177,35 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
 
     do
     {
+        if( (int)i_index >= p_sys->i_spu_size + 1 )
+        {
+            /* sanity
+             * XXX only on test by loop as p_sys->buffer is bigger than needed
+             * to avoid checking at each access
+             */
+            break;
+        }
+
         /* If we just read a command sequence, read the next one;
          * otherwise, go on with the commands of the current sequence. */
         if( i_command == SPU_CMD_END )
         {
             /* Get the control sequence date */
-            date = (mtime_t)GetBits( &p_spudec->bit_stream, 16 ) * 11000
+            date = (mtime_t)GetWBE( &p_sys->buffer[i_index] ) * 11000;
+            /* FIXME How to access i_rate
                     * p_spudec->bit_stream.p_pes->i_rate / DEFAULT_RATE;
- 
+            */
+
             /* Next offset */
             i_cur_seq = i_index;
-            i_next_seq = GetBits( &p_spudec->bit_stream, 16 );
- 
+            i_next_seq = GetWBE( &p_sys->buffer[i_index+2] );
+
             /* Skip what we just read */
             i_index += 4;
         }
- 
-        i_command = GetBits( &p_spudec->bit_stream, 8 );
-        i_index++;
- 
+
+        i_command = p_sys->buffer[i_index++];
+
         switch( i_command )
         {
         case SPU_CMD_FORCE_DISPLAY: /* 00 (force displaying) */
@@ -305,17 +225,20 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
         case SPU_CMD_SET_PALETTE:
 
             /* 03xxxx (palette) */
-            if( p_spudec->p_fifo->p_demux_data
-                 && *(int*)p_spudec->p_fifo->p_demux_data == 0xBeeF )
+            if( p_dec->fmt_in.subs.spu.palette[0] == 0xBeeF )
             {
-                uint32_t i_color;
+                unsigned int idx[4];
 
                 p_spu->p_sys->b_palette = VLC_TRUE;
+
+                idx[0] = (p_sys->buffer[i_index+0]>>4)&0x0f;
+                idx[1] = (p_sys->buffer[i_index+0])&0x0f;
+                idx[2] = (p_sys->buffer[i_index+1]>>4)&0x0f;
+                idx[3] = (p_sys->buffer[i_index+1])&0x0f;
+
                 for( i = 0; i < 4 ; i++ )
                 {
-                    i_color = ((uint32_t*)((char*)p_spudec->p_fifo->
-                                p_demux_data + sizeof(int)))[
-                                  GetBits(&p_spudec->bit_stream, 4) ];
+                    uint32_t i_color = p_dec->fmt_in.subs.spu.palette[1+idx[i]];
 
                     /* FIXME: this job should be done sooner */
                     p_spu->p_sys->pi_yuv[3-i][0] = (i_color>>16) & 0xff;
@@ -323,19 +246,15 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
                     p_spu->p_sys->pi_yuv[3-i][2] = (i_color>>8) & 0xff;
                 }
             }
-            else
-            {
-                RemoveBits( &p_spudec->bit_stream, 16 );
-            }
             i_index += 2;
 
             break;
 
         case SPU_CMD_SET_ALPHACHANNEL: /* 04xxxx (alpha channel) */
-            pi_alpha[3] = GetBits( &p_spudec->bit_stream, 4 );
-            pi_alpha[2] = GetBits( &p_spudec->bit_stream, 4 );
-            pi_alpha[1] = GetBits( &p_spudec->bit_stream, 4 );
-            pi_alpha[0] = GetBits( &p_spudec->bit_stream, 4 );
+            pi_alpha[3] = (p_sys->buffer[i_index+0]>>4)&0x0f;
+            pi_alpha[2] = (p_sys->buffer[i_index+0])&0x0f;
+            pi_alpha[1] = (p_sys->buffer[i_index+1]>>4)&0x0f;
+            pi_alpha[0] = (p_sys->buffer[i_index+1])&0x0f;
 
             /* Ignore blank alpha palette. Sometimes spurious blank
              * alpha palettes are present - dunno why. */
@@ -348,31 +267,29 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
             }
             else
             {
-                msg_Warn( p_spudec->p_fifo, "ignoring blank alpha palette" );
+                msg_Warn( p_dec, "ignoring blank alpha palette" );
             }
 
             i_index += 2;
             break;
 
         case SPU_CMD_SET_COORDINATES: /* 05xxxyyyxxxyyy (coordinates) */
-            p_spu->i_x = GetBits( &p_spudec->bit_stream, 12 );
-            p_spu->i_width = GetBits( &p_spudec->bit_stream, 12 )
-                              - p_spu->i_x + 1;
+            p_spu->i_x = (p_sys->buffer[i_index+0]<<4)|
+                         ((p_sys->buffer[i_index+1]>>4)&0x0f);
+            p_spu->i_width = (((p_sys->buffer[i_index+1]&0x0f)<<8)|
+                              p_sys->buffer[i_index+2]) - p_spu->i_x + 1;
 
-            p_spu->i_y = GetBits( &p_spudec->bit_stream, 12 );
-            p_spu->i_height = GetBits( &p_spudec->bit_stream, 12 )
-                               - p_spu->i_y + 1;
+            p_spu->i_y = (p_sys->buffer[i_index+3]<<4)|
+                         ((p_sys->buffer[i_index+4]>>4)&0x0f);
+            p_spu->i_height = (((p_sys->buffer[i_index+4]&0x0f)<<8)|
+                              p_sys->buffer[i_index+5]) - p_spu->i_y + 1;
 
             i_index += 6;
             break;
 
         case SPU_CMD_SET_OFFSETS: /* 06xxxxyyyy (byte offsets) */
-            p_spu->p_sys->pi_offset[0] =
-                GetBits( &p_spudec->bit_stream, 16 ) - 4;
-
-            p_spu->p_sys->pi_offset[1] =
-                GetBits( &p_spudec->bit_stream, 16 ) - 4;
-
+            p_spu->p_sys->pi_offset[0] = GetWBE(&p_sys->buffer[i_index+0]) - 4;
+            p_spu->p_sys->pi_offset[1] = GetWBE(&p_sys->buffer[i_index+2]) - 4;
             i_index += 4;
             break;
 
@@ -380,13 +297,12 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
             break;
 
         default: /* xx (unknown command) */
-            msg_Err( p_spudec->p_fifo, "unknown command 0x%.2x",
-                                       i_command );
+            msg_Err( p_dec, "unknown command 0x%.2x", i_command );
             return VLC_EGENERIC;
         }
 
         /* We need to check for quit commands here */
-        if( p_spudec->p_fifo->b_die )
+        if( p_dec->b_die )
         {
             return VLC_EGENERIC;
         }
@@ -396,55 +312,41 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
     /* Check that the next sequence index matches the current one */
     if( i_next_seq != i_cur_seq )
     {
-        msg_Err( p_spudec->p_fifo, "index mismatch (0x%.4x != 0x%.4x)",
-                                   i_next_seq, i_cur_seq );
+        msg_Err( p_dec, "index mismatch (0x%.4x != 0x%.4x)",
+                 i_next_seq, i_cur_seq );
         return VLC_EGENERIC;
     }
 
-    if( i_index > p_spudec->i_spu_size )
+    if( (int)i_index > p_sys->i_spu_size )
     {
-        msg_Err( p_spudec->p_fifo, "uh-oh, we went too far (0x%.4x > 0x%.4x)",
-                                   i_index, p_spudec->i_spu_size );
+        msg_Err( p_dec, "uh-oh, we went too far (0x%.4x > 0x%.4x)",
+                 i_index, p_sys->i_spu_size );
         return VLC_EGENERIC;
     }
 
     if( !p_spu->i_start )
     {
-        msg_Err( p_spudec->p_fifo, "no `start display' command" );
+        msg_Err( p_dec, "no `start display' command" );
     }
 
     if( p_spu->i_stop <= p_spu->i_start && !p_spu->b_ephemer )
     {
         /* This subtitle will live for 5 seconds or until the next subtitle */
-        p_spu->i_stop = p_spu->i_start + (mtime_t)500 * 11000
+        p_spu->i_stop = p_spu->i_start + (mtime_t)500 * 11000;
+        /* FIXME how to access i_rate ?
                         * p_spudec->bit_stream.p_pes->i_rate / DEFAULT_RATE;
+        */
         p_spu->b_ephemer = VLC_TRUE;
     }
 
     /* Get rid of padding bytes */
-    switch( p_spudec->i_spu_size - i_index )
+    if( p_sys->i_spu_size > (int)i_index + 1 )
     {
-        /* Zero or one padding byte, quite usual */
-        case 1:
-            RemoveBits( &p_spudec->bit_stream, 8 );
-            i_index++;
-        case 0:
-            break;
-
-        /* More than one padding byte - this is very strange, but
+        /* Zero or one padding byte, are quite usual
+         * More than one padding byte - this is very strange, but
          * we can deal with it */
-        default:
-            msg_Warn( p_spudec->p_fifo,
-                      "%i padding bytes, we usually get 0 or 1 of them",
-                      p_spudec->i_spu_size - i_index );
-
-            while( i_index < p_spudec->i_spu_size )
-            {
-                RemoveBits( &p_spudec->bit_stream, 8 );
-                i_index++;
-            }
-
-            break;
+        msg_Warn( p_dec, "%i padding bytes, we usually get 0 or 1 of them",
+                  p_sys->i_spu_size - i_index );
     }
 
     /* Successfully parsed ! */
@@ -458,9 +360,11 @@ static int ParseControlSeq( spudec_thread_t *p_spudec,
  * convenient structure for later decoding. For more information on the
  * subtitles format, see http://sam.zoy.org/doc/dvd/subtitles/index.html
  *****************************************************************************/
-static int ParseRLE( spudec_thread_t *p_spudec,
-                     subpicture_t * p_spu, uint8_t * p_src )
+static int ParseRLE( decoder_t *p_dec, subpicture_t * p_spu )
 {
+    decoder_sys_t *p_sys = p_dec->p_sys;
+    uint8_t       *p_src = &p_sys->buffer[4];
+
     unsigned int i_code;
 
     unsigned int i_width = p_spu->i_width;
@@ -519,7 +423,7 @@ static int ParseRLE( spudec_thread_t *p_spudec,
                             else
                             {
                                 /* We have a boo boo ! */
-                                msg_Err( p_spudec->p_fifo, "unknown RLE code "
+                                msg_Err( p_dec, "unknown RLE code "
                                          "0x%.4x", i_code );
                                 return VLC_EGENERIC;
                             }
@@ -530,7 +434,7 @@ static int ParseRLE( spudec_thread_t *p_spudec,
 
             if( ( (i_code >> 2) + i_x + i_y * i_width ) > i_height * i_width )
             {
-                msg_Err( p_spudec->p_fifo,
+                msg_Err( p_dec,
                          "out of bounds, %i at (%i,%i) is out of %ix%i",
                          i_code >> 2, i_x, i_y, i_width, i_height );
                 return VLC_EGENERIC;
@@ -580,8 +484,8 @@ static int ParseRLE( spudec_thread_t *p_spudec,
         /* Check that we didn't go too far */
         if( i_x > i_width )
         {
-            msg_Err( p_spudec->p_fifo, "i_x overflowed, %i > %i",
-                                       i_x, i_width );
+            msg_Err( p_dec, "i_x overflowed, %i > %i",
+                     i_x, i_width );
             return VLC_EGENERIC;
         }
 
@@ -598,9 +502,9 @@ static int ParseRLE( spudec_thread_t *p_spudec,
     /* We shouldn't get any padding bytes */
     if( i_y < i_height )
     {
-        msg_Err( p_spudec->p_fifo, "padding bytes found in RLE sequence" );
-        msg_Err( p_spudec->p_fifo, "send mail to <sam@zoy.org> if you "
-                                   "want to help debugging this" );
+        msg_Err( p_dec, "padding bytes found in RLE sequence" );
+        msg_Err( p_dec, "send mail to <sam@zoy.org> if you "
+                        "want to help debugging this" );
 
         /* Skip them just in case */
         while( i_y < i_height )
@@ -612,7 +516,7 @@ static int ParseRLE( spudec_thread_t *p_spudec,
         return VLC_EGENERIC;
     }
 
-    msg_Dbg( p_spudec->p_fifo, "valid subtitle, size: %ix%i, position: %i,%i",
+    msg_Dbg( p_dec, "valid subtitle, size: %ix%i, position: %i,%i",
              p_spu->i_width, p_spu->i_height, p_spu->i_x, p_spu->i_y );
 
 #if 0 /* cropping */
@@ -679,7 +583,7 @@ static int ParseRLE( spudec_thread_t *p_spudec,
             p_spu->p_sys->pi_yuv[i_shade][2] = 0x80;
         }
 
-        msg_Dbg( p_spudec->p_fifo,
+        msg_Dbg( p_dec,
                  "using custom palette (border %i, inner %i, shade %i)",
                  i_border, i_inner, i_shade );
     }
