@@ -2,7 +2,7 @@
  * MediaControlView.cpp: beos interface
  *****************************************************************************
  * Copyright (C) 1999, 2000, 2001 VideoLAN
- * $Id: MediaControlView.cpp,v 1.15 2003/02/03 17:18:48 stippi Exp $
+ * $Id: MediaControlView.cpp,v 1.21 2003/12/28 01:49:12 titer Exp $
  *
  * Authors: Tony Castley <tony@castley.net>
  *          Stephan Aßmus <stippi@yellowbites.com>
@@ -31,6 +31,10 @@
 /* VLC headers */
 #include <vlc/vlc.h>
 #include <vlc/intf.h>
+extern "C"
+{
+  #include <audio_output.h>
+}
 
 /* BeOS interface headers */
 #include "VlcWrapper.h"
@@ -61,7 +65,7 @@ const rgb_color kSeekRed = (rgb_color){ 255, 0, 0, 255 };
 const rgb_color kSeekRedLight = (rgb_color){ 255, 152, 152, 255 };
 const rgb_color kSeekRedShadow = (rgb_color){ 178, 0, 0, 255 };
 
-const char* kDisabledSeekMessage = "Drop files to play";
+#define DISABLED_SEEK_MESSAGE _("Drop files to play")
 
 enum
 {
@@ -86,8 +90,8 @@ MediaControlView::MediaControlView(BRect frame, intf_thread_t *p_interface)
 	BRect frame(0.0, 0.0, 10.0, 10.0);
 	
     // Seek Slider
-    fSeekSlider = new SeekSlider(frame, "seek slider", this,
-                                 0, SEEKSLIDER_RANGE - 1);
+    fSeekSlider = new SeekSlider( frame, "seek slider", this,
+                                  0, SEEKSLIDER_RANGE );
     fSeekSlider->SetValue(0);
     fSeekSlider->ResizeToPreferred();
     AddChild( fSeekSlider );
@@ -172,7 +176,7 @@ MediaControlView::MediaControlView(BRect frame, intf_thread_t *p_interface)
 										   kVolumeSliderBitmapHeight - 1.0),
 									 "volume slider", 1, AOUT_VOLUME_MAX,
 									 new BMessage(VOLUME_CHG));
-	fVolumeSlider->SetValue(AOUT_VOLUME_DEFAULT);
+	fVolumeSlider->SetValue( config_GetInt( p_intf, "volume" ) );
 	AddChild( fVolumeSlider );
 	
 	// Position Info View
@@ -279,6 +283,10 @@ MediaControlView::Pulse()
 	InterfaceWindow* window = dynamic_cast<InterfaceWindow*>(Window());
 	if (window && window->IsStopped())
 			fPlayPause->SetStopped();
+
+    unsigned short i_volume;
+    aout_VolumeGet( p_intf, (audio_volume_t*)&i_volume );
+    fVolumeSlider->SetValue( i_volume );
 }
 
 // SetProgress
@@ -300,14 +308,12 @@ MediaControlView::SetStatus(int status, int rate)
         case PLAYING_S:
         case FORWARD_S:
         case BACKWARD_S:
-        case START_S:
             fPlayPause->SetPlaying();
             break;
         case PAUSE_S:
             fPlayPause->SetPaused();
             break;
         case UNDEF_S:
-        case NOT_STARTED_S:
         default:
             fPlayPause->SetStopped();
             break;
@@ -695,7 +701,7 @@ SeekSlider::Draw(BRect updateRect)
 		SetHighColor(darkShadow);
 		SetLowColor(shadow);
 		// stripes
-		float width = floorf(StringWidth(kDisabledSeekMessage));
+		float width = floorf(StringWidth(DISABLED_SEEK_MESSAGE));
 		float textPos = r.left + r.Width() / 2.0 - width / 2.0;
 		pattern stripes = {{ 0xc7, 0x8f, 0x1f, 0x3e, 0x7c, 0xf8, 0xf1, 0xe3 }};
 		BRect stripesRect(r);
@@ -712,7 +718,7 @@ SeekSlider::Draw(BRect updateRect)
 		SetLowColor(darkShadow);
 		font_height fh;
 		GetFontHeight(&fh);
-		DrawString(kDisabledSeekMessage, BPoint(textPos, r.top + ceilf(fh.ascent) - 1.0));
+		DrawString(DISABLED_SEEK_MESSAGE, BPoint(textPos, r.top + ceilf(fh.ascent) - 1.0));
 	}
 }
 
@@ -763,7 +769,7 @@ SeekSlider::MouseUp(BPoint where)
 void
 SeekSlider::ResizeToPreferred()
 {
-	float width = 15.0 + StringWidth(kDisabledSeekMessage) + 15.0;
+	float width = 15.0 + StringWidth(DISABLED_SEEK_MESSAGE) + 15.0;
 	ResizeTo(width, 17.0);
 }
 
@@ -1254,9 +1260,9 @@ PositionInfoView::Draw( BRect updateRect )
 			float height = ( r.Height() - timeHeight ) / 3.0;
 			SetFont( &tinyFont );
 			SetHighColor( 0, 180, 0, 255 );
-			DrawString( "File", BPoint( r.left + 3.0, r.top + height ) );
-			DrawString( "Title", BPoint( r.left + 3.0, r.top + 2.0 * height ) );
-			DrawString( "Chapter", BPoint( r.left + 3.0, r.top + 3.0 * height ) );
+			DrawString( _("File"), BPoint( r.left + 3.0, r.top + height ) );
+			DrawString( _("Title"), BPoint( r.left + 3.0, r.top + 2.0 * height ) );
+			DrawString( _("Chapter"), BPoint( r.left + 3.0, r.top + 3.0 * height ) );
 			SetFont( &smallFont );
 			BString helper;
 			SetHighColor( 0, 255, 0, 255 );
