@@ -1,10 +1,10 @@
 /*****************************************************************************
  * gtk_callbacks.c : Callbacks for the Gtk+ plugin.
  *****************************************************************************
- * Copyright (C) 2000, 2001 VideoLAN
- * $Id: gtk_callbacks.c,v 1.13 2003/02/09 01:56:21 massiot Exp $
+ * Copyright (C) 2000, 2001, 2003 VideoLAN
+ * $Id: gtk_callbacks.c,v 1.16 2003/12/22 14:32:56 sam Exp $
  *
- * Authors: Samuel Hocevar <sam@zoy.org>
+ * Authors: Sam Hocevar <sam@zoy.org>
  *          Stphane Borel <stef@via.ecp.fr>
  *          Julien BLACHE <jb@technologeek.org>
  *
@@ -46,7 +46,17 @@
 
 #include "common.h"
 
-#include "netutils.h"
+#ifdef HAVE_CDDAX
+#define CDDA_MRL "cddax://"
+#else
+#define CDDA_MRL "cdda://"
+#endif
+
+#ifdef HAVE_VCDX
+#define VCD_MRL "vcdx://"
+#else
+#define VCD_MRL "vcdx://"
+#endif
 
 /*****************************************************************************
  * Useful function to retrieve p_intf
@@ -324,44 +334,6 @@ void GtkChapterNext( GtkButton * button, gpointer user_data )
     vlc_mutex_unlock( &p_intf->p_sys->p_input->stream.stream_lock );
 }
 
-/****************************************************************************
- * Network specific items
- ****************************************************************************/
-void GtkNetworkJoin( GtkEditable * editable, gpointer user_data )
-{
-    int     i_channel;
-
-    i_channel = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( editable ) );
-#if 0
-    msg_Dbg( "intf info: joining channel %d", i_channel );
-
-    network_ChannelJoin( i_channel );
-#endif
-}
-
-void GtkChannelGo( GtkButton * button, gpointer user_data )
-{
-    GtkWidget *     window;
-    GtkWidget *     spin;
-    int             i_channel;
-
-    intf_thread_t *p_intf = GtkGetIntf( button );
-
-    window = gtk_widget_get_toplevel( GTK_WIDGET (button) );
-    spin = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( window ),
-                       "network_channel_spinbutton" ) );
-
-    i_channel = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( spin ) );
-    msg_Dbg( p_intf, "joining channel %d", i_channel );
-
-    vlc_mutex_lock( &p_intf->change_lock );
-    network_ChannelJoin( p_intf, i_channel );
-    vlc_mutex_unlock( &p_intf->change_lock );
-
-/*    input_SetStatus( p_intf->p_sys->p_input, INPUT_STATUS_PLAY ); */
-}
-
-
 /****************************************************************************
  * About box
  ****************************************************************************/
@@ -487,31 +459,45 @@ gboolean GtkDiscEject ( GtkWidget *widget, gpointer user_data )
 
     if( psz_current != NULL )
     {
-        if( !strncmp(psz_current, "dvd:", 4) )
+        if( !strncmp(psz_current, "dvd://", 4) )
         {
-            switch( psz_current[4] )
+            switch( psz_current[strlen("dvd://")] )
             {
             case '\0':
             case '@':
                 psz_device = config_GetPsz( p_intf, "dvd" );
                 break;
             default:
-                /* Omit the first 4 characters */
-                psz_device = strdup( psz_current + 4 );
+                /* Omit the first MRL-selector characters */
+                psz_device = strdup( psz_current + strlen("dvd://" ) );
                 break;
             }
         }
-        else if( !strncmp(psz_current, "vcd:", 4) )
+        else if( !strncmp(psz_current, "vcd:", strlen("vcd:")) )
         {
-            switch( psz_current[4] )
+            switch( psz_current[strlen("vcd:")] )
             {
             case '\0':
             case '@':
-                psz_device = config_GetPsz( p_intf, "vcd" );
+                psz_device = config_GetPsz( p_intf, VCD_MRL );
                 break;
             default:
-                /* Omit the first 4 characters */
-                psz_device = strdup( psz_current + 4 );
+                /* Omit the beginning MRL-selector characters */
+                psz_device = strdup( psz_current + strlen(VCD_MRL) );
+                break;
+            }
+        }
+        else if( !strncmp(psz_current, CDDA_MRL, strlen(CDDA_MRL) ) )
+        {
+            switch( psz_current[strlen(CDDA_MRL)] )
+            {
+            case '\0':
+            case '@':
+                psz_device = config_GetPsz( p_intf, "cd-audio" );
+                break;
+            default:
+                /* Omit the beginning MRL-selector characters */
+                psz_device = strdup( psz_current + strlen(CDDA_MRL) );
                 break;
             }
         }
@@ -650,8 +636,6 @@ GtkMenubarDeinterlace                  ( GtkMenuItem     *menuitem,
 
     if( p_intf )
         msg_Dbg( p_intf, "GtkMenubarDeinterlace" );
-
-    
 }
 
 
