@@ -2,7 +2,7 @@
  * gtk_playlist.c : Interface for the playlist dialog
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: playlist.c,v 1.4 2003/07/23 01:13:47 gbazin Exp $
+ * $Id: playlist.c,v 1.8 2003/12/22 14:32:56 sam Exp $
  *
  * Authors: Pierre Baillet <oct@zoy.org>
  *          Stphane Borel <stef@via.ecp.fr>
@@ -694,8 +694,18 @@ void GtkRebuildCList( GtkCList * p_clist, playlist_t * p_playlist )
     vlc_mutex_lock( &p_playlist->object_lock );
     for( i_dummy = p_playlist->i_size ; i_dummy-- ; )
     {
+        char psz_duration[MSTRTIME_MAX_SIZE];
+        mtime_t dur = p_playlist->pp_items[i_dummy]->i_duration;
+        if ( dur != -1 )
+        {
+            secstotimestr( psz_duration, dur/1000000 );
+        }
+        else
+        {
+            memcpy( psz_duration ,"no info",sizeof("no info" ));
+        }
         ppsz_text[0] = p_playlist->pp_items[i_dummy]->psz_name;
-        ppsz_text[1] = "no info";
+        ppsz_text[1] = strdup( psz_duration );
         gtk_clist_insert( p_clist, 0, ppsz_text );
     }
     vlc_mutex_unlock( &p_playlist->object_lock );
