@@ -2,11 +2,11 @@
  * intf.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: intf.m,v 1.93 2003/07/27 23:05:41 hartman Exp $
+ * $Id: intf.m,v 1.110 2003/12/22 17:08:22 bigben Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
- *          Derk-Jan Hartman <thedj@users.sourceforge.net>
+ *          Derk-Jan Hartman <hartman at videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <stdlib.h>                                      /* malloc(), free() */
 #include <sys/param.h>                                    /* for MAXPATHLEN */
 #include <string.h>
+#include <vlc_keys.h>
 
 #include "intf.h"
 #include "vout.h"
@@ -70,9 +71,9 @@ int E_(OpenIntf) ( vlc_object_t *p_this )
     p_intf->pf_run = Run;
     
     [[VLCApplication sharedApplication] autorelease];
-    [NSApp initIntlSupport];
     [NSApp setIntf: p_intf];
 
+
     [NSBundle loadNibNamed: @"MainMenu" owner: NSApp];
 
     return( 0 );
@@ -111,18 +112,6 @@ static void Run( intf_thread_t *p_intf )
  *****************************************************************************/
 @implementation VLCApplication
 
-- (id)init
-{
-    /* default encoding: ISO-8859-1 */
-    i_encoding = NSUTF8StringEncoding;
-
-    return( [super init] );
-}
-
-- (void)initIntlSupport
-{
-}
-
 - (NSString *)localizedString:(char *)psz
 {
     NSString * o_str = nil;
@@ -141,13 +130,13 @@ static void Run( intf_thread_t *p_intf )
 
 - (char *)delocalizeString:(NSString *)id
 {
-    NSData * o_data = [id dataUsingEncoding: i_encoding
+    NSData * o_data = [id dataUsingEncoding: NSUTF8StringEncoding
                           allowLossyConversion: NO];
     char * psz_string;
 
     if ( o_data == nil )
     {
-        o_data = [id dataUsingEncoding: i_encoding
+        o_data = [id dataUsingEncoding: NSUTF8StringEncoding
                      allowLossyConversion: YES];
         psz_string = malloc( [o_data length] + 1 ); 
         [o_data getBytes: psz_string];
@@ -165,11 +154,6 @@ static void Run( intf_thread_t *p_intf )
     return psz_string;
 }
 
-- (NSStringEncoding)getEncoding
-{
-    return i_encoding;
-}
-
 /* i_width is in pixels */
 - (NSString *)wrapString: (NSString *)o_in_string toWidth: (int) i_width
 {
@@ -307,6 +291,83 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     return VLC_SUCCESS;
 }
 
+static struct
+{
+    unichar i_nskey;
+    unsigned int i_vlckey;
+} nskeys_to_vlckeys[] = 
+{
+    { NSUpArrowFunctionKey, KEY_UP },
+    { NSDownArrowFunctionKey, KEY_DOWN },
+    { NSLeftArrowFunctionKey, KEY_LEFT },
+    { NSRightArrowFunctionKey, KEY_RIGHT },
+    { NSF1FunctionKey, KEY_F1 },
+    { NSF2FunctionKey, KEY_F2 },
+    { NSF3FunctionKey, KEY_F3 },
+    { NSF4FunctionKey, KEY_F4 },
+    { NSF5FunctionKey, KEY_F5 },
+    { NSF6FunctionKey, KEY_F6 },
+    { NSF7FunctionKey, KEY_F7 },
+    { NSF8FunctionKey, KEY_F8 },
+    { NSF9FunctionKey, KEY_F9 },
+    { NSF10FunctionKey, KEY_F10 },
+    { NSF11FunctionKey, KEY_F11 },
+    { NSF12FunctionKey, KEY_F12 },
+    { NSHomeFunctionKey, KEY_HOME },
+    { NSEndFunctionKey, KEY_END },
+    { NSPageUpFunctionKey, KEY_PAGEUP },
+    { NSPageDownFunctionKey, KEY_PAGEDOWN },
+    { NSTabCharacter, KEY_TAB },
+    { NSCarriageReturnCharacter, KEY_ENTER },
+    { NSEnterCharacter, KEY_ENTER },
+    { NSBackspaceCharacter, KEY_BACKSPACE },
+    { (unichar) ' ', KEY_SPACE },
+    { (unichar) 0x1b, KEY_ESC },
+    {0,0}
+};
+
+unichar VLCKeyToCocoa( unsigned int i_key )
+{
+    unsigned int i;
+    
+    for( i = 0; nskeys_to_vlckeys[i].i_vlckey != 0; i++ )
+    {
+        if( nskeys_to_vlckeys[i].i_vlckey == (i_key & ~KEY_MODIFIER) )
+        {
+            return nskeys_to_vlckeys[i].i_nskey;
+        }
+    }
+    return (unichar)(i_key & ~KEY_MODIFIER);
+}
+
+unsigned int CocoaKeyToVLC( unichar i_key )
+{
+    unsigned int i;
+    
+    for( i = 0; nskeys_to_vlckeys[i].i_nskey != 0; i++ )
+    {
+        if( nskeys_to_vlckeys[i].i_nskey == i_key )
+        {
+            return nskeys_to_vlckeys[i].i_vlckey;
+        }
+    }
+    return (unsigned int)i_key;
+}
+
+unsigned int VLCModifiersToCocoa( unsigned int i_key )
+{
+    unsigned int new = 0;
+    if( i_key & KEY_MODIFIER_COMMAND )
+        new |= NSCommandKeyMask;
+    if( i_key & KEY_MODIFIER_ALT )
+        new |= NSAlternateKeyMask;
+    if( i_key & KEY_MODIFIER_SHIFT )
+        new |= NSShiftKeyMask;
+    if( i_key & KEY_MODIFIER_CTRL )
+        new |= NSControlKeyMask;
+    return new;
+}
+
 /*****************************************************************************
  * VLCMain implementation 
  *****************************************************************************/
@@ -314,25 +375,84 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
 - (void)awakeFromNib
 {
-    [o_window setTitle: _NS("VLC - Controller")];
+    unsigned int i_key = 0;
+    intf_thread_t * p_intf = [NSApp getIntf];
+    vlc_value_t val;
+
+    [self initStrings];
     [o_window setExcludedFromWindowsMenu: TRUE];
+    [o_msgs_panel setExcludedFromWindowsMenu: TRUE];
+    [o_msgs_panel setDelegate: self];
+    
+    i_key = config_GetInt( p_intf, "key-quit" );
+    [o_mi_quit setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_quit setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-play-pause" );
+    [o_mi_play setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_play setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-stop" );
+    [o_mi_stop setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_stop setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-faster" );
+    [o_mi_faster setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_faster setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-slower" );
+    [o_mi_slower setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_slower setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-prev" );
+    [o_mi_previous setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_previous setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-next" );
+    [o_mi_next setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_next setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-jump+10sec" );
+    [o_mi_fwd setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_fwd setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-jump-10sec" );
+    [o_mi_bwd setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_bwd setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-vol-up" );
+    [o_mi_vol_up setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_vol_up setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-vol-down" );
+    [o_mi_vol_down setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_vol_down setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-vol-mute" );
+    [o_mi_mute setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_mute setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+    i_key = config_GetInt( p_intf, "key-fullscreen" );
+    [o_mi_fullscreen setKeyEquivalent: [NSString stringWithFormat:@"%C", VLCKeyToCocoa( i_key )]];
+    [o_mi_fullscreen setKeyEquivalentModifierMask: VLCModifiersToCocoa(i_key)];
+
+    var_Create (p_intf, "fullscreen", VLC_VAR_BOOL );
+    var_Change (p_intf, "fullscreen", VLC_VAR_INHERITVALUE, &val, NULL );
+    [o_btn_fullscreen setState: val.b_bool];
+
+    var_Create(p_intf,"intf-change",VLC_VAR_BOOL );
+
+    [self setSubmenusEnabled: FALSE];
+    [self manageVolumeSlider];
+
+}
+
+- (void)initStrings
+{
+    [o_window setTitle: _NS("VLC - Controller")];
+    [o_scrollfield setStringValue: _NS("VLC Media Player")];
 
     /* button controls */
-    [o_btn_playlist setToolTip: _NS("Playlist")];
     [o_btn_prev setToolTip: _NS("Previous")];
-    [o_btn_slower setToolTip: _NS("Slower")];
+    [o_btn_rewind setToolTip: _NS("Rewind")];
     [o_btn_play setToolTip: _NS("Play")];
     [o_btn_stop setToolTip: _NS("Stop")];
-    [o_btn_faster setToolTip: _NS("Faster")];
+    [o_btn_ff setToolTip: _NS("Fast Forward")];
     [o_btn_next setToolTip: _NS("Next")];
-    [o_btn_prefs setToolTip: _NS("Preferences")];
+    [o_btn_fullscreen setToolTip: _NS("Fullscreen")];
     [o_volumeslider setToolTip: _NS("Volume")];
     [o_timeslider setToolTip: _NS("Position")];
 
     /* messages panel */ 
-    [o_msgs_panel setDelegate: self];
     [o_msgs_panel setTitle: _NS("Messages")];
-    [o_msgs_panel setExcludedFromWindowsMenu: TRUE];
     [o_msgs_btn_crashlog setTitle: _NS("Open CrashLog")];
 
     /* main menu */
@@ -344,8 +464,8 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     [o_mi_quit setTitle: _NS("Quit VLC")];
 
     [o_mu_file setTitle: _ANS("1:File")];
-    [o_mi_open_generic setTitle: _NS("Open...")];
-    [o_mi_open_file setTitle: _NS("Open File...")];
+    [o_mi_open_generic setTitle: _NS("Open File...")];
+    [o_mi_open_file setTitle: _NS("Quick Open File...")];
     [o_mi_open_disc setTitle: _NS("Open Disc...")];
     [o_mi_open_net setTitle: _NS("Open Network...")];
     [o_mi_open_recent setTitle: _NS("Open Recent")];
@@ -365,7 +485,9 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     [o_mi_slower setTitle: _NS("Slower")];
     [o_mi_previous setTitle: _NS("Previous")];
     [o_mi_next setTitle: _NS("Next")];
-    [o_mi_loop setTitle: _NS("Loop")];
+    [o_mi_random setTitle: _NS("Shuffle")];
+    [o_mi_repeat setTitle: _NS("Repeat Item")];
+    [o_mi_loop setTitle: _NS("Repeat Playlist")];
     [o_mi_fwd setTitle: _NS("Step Forward")];
     [o_mi_bwd setTitle: _NS("Step Backward")];
     [o_mi_program setTitle: _NS("Program")];
@@ -385,6 +507,8 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     [o_mu_channels setTitle: _NS("Audio channels")];
     [o_mi_device setTitle: _NS("Audio device")];
     [o_mu_device setTitle: _NS("Audio device")];
+    [o_mi_visual setTitle: _NS("Visualizations")];
+    [o_mu_visual setTitle: _NS("Visualizations")];
     
     [o_mu_video setTitle: _NS("Video")];
     [o_mi_half_window setTitle: _NS("Half Size")];
@@ -433,9 +557,6 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     [o_err_btn_dismiss setTitle: _NS("Dismiss")];
 
     [o_info_window setTitle: _NS("Info")];
-
-    [self setSubmenusEnabled: FALSE];
-    [self manageVolumeSlider];
 }
 
 - (void)applicationWillFinishLaunching:(NSNotification *)o_notification
@@ -446,7 +567,9 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     o_msg_arr = [[NSMutableArray arrayWithCapacity: 200] retain];
 
     o_img_play = [[NSImage imageNamed: @"play"] retain];
+    o_img_play_pressed = [[NSImage imageNamed: @"play_blue"] retain];
     o_img_pause = [[NSImage imageNamed: @"pause"] retain];
+    o_img_pause_pressed = [[NSImage imageNamed: @"pause_blue"] retain];
 
     [p_intf->p_sys->o_sendport setDelegate: self];
     [[NSRunLoop currentRunLoop] 
@@ -481,6 +604,15 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     return nil;
 }
 
+- (id)getPlaylist
+{
+    if ( o_playlist )
+    {
+        return o_playlist;
+    }
+    return nil;
+}
+
 - (void)manage
 {
     NSDate * o_sleep_date;
@@ -492,7 +624,7 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     while( !p_intf->b_die )
     {
         playlist_t * p_playlist;
-
+        vlc_value_t val;
         vlc_mutex_lock( &p_intf->change_lock );
 
         p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, 
@@ -501,12 +633,46 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
         if( p_playlist != NULL )
         {
             var_AddCallback( p_playlist, "intf-change", PlaylistChanged, self );
-            vlc_mutex_lock( &p_playlist->object_lock );
+#define p_input p_playlist->p_input
+        
+            if( p_input )
+            {
+                if( !p_input->b_die )
+                {
+                    vlc_value_t val;
+
+                    /* New input or stream map change */
+                    if( p_input->stream.b_changed )
+                    {
+                        msg_Dbg( p_intf, "stream has changed, refreshing interface" );
+                        p_intf->p_sys->b_playing = TRUE;
+                        p_intf->p_sys->b_current_title_update = 1;
+                        p_input->stream.b_changed = 0;
+                        p_intf->p_sys->b_intf_update = TRUE;
+                    }
+
+                    if( var_Get( (vlc_object_t *)p_input, "intf-change", &val )
+                        >= 0 && val.b_bool )
+                    {
+                        p_intf->p_sys->b_input_update = TRUE;
+                    }
+                }
+            }
+            else if( p_intf->p_sys->b_playing && !p_intf->b_die )
+            {
+                p_intf->p_sys->b_playing = FALSE;
+            }
             
-            [self manage: p_playlist];
-            
-            vlc_mutex_unlock( &p_playlist->object_lock );
+#undef p_input
             vlc_object_release( p_playlist );
+
+            if( var_Get( p_intf, "intf-change", &val )
+                        >= 0 && val.b_bool )
+            {
+                p_intf->p_sys->b_fullscreen_update = TRUE;
+            }
+            val.b_bool = VLC_FALSE;
+            var_Set( p_intf,"intf-change",val);
         }
 
         vlc_mutex_unlock( &p_intf->change_lock );
@@ -516,77 +682,9 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     }
 
     [self terminate];
-
     [o_pool release];
 }
 
-- (void)manage:(playlist_t *)p_playlist
-{
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-#define p_input p_playlist->p_input
-
-    if( p_input )
-    {
-        vlc_mutex_lock( &p_input->stream.stream_lock );
-
-        if( !p_input->b_die )
-        {
-            /* New input or stream map change */
-            if( p_input->stream.b_changed )
-            {
-                p_intf->p_sys->b_playing = TRUE;
-                [self manageMode: p_playlist];
-            }
-
-            vlc_value_t val;
-
-            if( var_Get( (vlc_object_t *)p_input, "intf-change", &val )
-                >= 0 && val.b_bool )
-            {
-                p_intf->p_sys->b_input_update = TRUE;
-            }
-        }
-        vlc_mutex_unlock( &p_input->stream.stream_lock );
-    }
-    else if( p_intf->p_sys->b_playing && !p_intf->b_die )
-    {
-        p_intf->p_sys->b_playing = FALSE;
-        [self manageMode: p_playlist];
-    }
-
-#undef p_input
-}
-
-- (void)manageMode:(playlist_t *)p_playlist
-{
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    if( p_playlist->p_input != NULL )
-    {
-        p_intf->p_sys->b_current_title_update = 1;
-        p_playlist->p_input->stream.b_changed = 0;
-        
-        msg_Dbg( p_intf, "stream has changed, refreshing interface" );
-    }
-    else
-    {
-        vout_thread_t * p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                                          FIND_ANYWHERE );
-        if( p_vout != NULL )
-        {
-            vlc_object_detach( p_vout );
-            vlc_object_release( p_vout );
-
-            vlc_mutex_unlock( &p_playlist->object_lock );
-            vout_Destroy( p_vout );
-            vlc_mutex_lock( &p_playlist->object_lock );
-        }
-    }
-
-    p_intf->p_sys->b_intf_update = VLC_TRUE;
-}
-
 - (void)manageIntf:(NSTimer *)o_timer
 {
     intf_thread_t * p_intf = [NSApp getIntf];
@@ -613,13 +711,18 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
     if( p_intf->p_sys->b_current_title_update )
     {
-        vout_thread_t   *p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                              FIND_ANYWHERE );
+        vout_thread_t *p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                                FIND_ANYWHERE );
+
+        vlc_mutex_lock( &p_playlist->object_lock );
+        [o_scrollfield setStringValue: [NSString stringWithUTF8String: 
+            p_playlist->pp_items[p_playlist->i_index]->psz_name]]; 
+        vlc_mutex_unlock( &p_playlist->object_lock );
+
         if( p_vout != NULL )
         {
-            /* We need to lock the vout here to make sure it does not disappear */
             id o_vout_wnd;
-            NSEnumerator * o_enum = [[NSApp windows] objectEnumerator];
+            NSEnumerator * o_enum = [[NSApp orderedWindows] objectEnumerator];
             
             while( ( o_vout_wnd = [o_enum nextObject] ) )
             {
@@ -640,11 +743,6 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
 #define p_input p_playlist->p_input
 
-    if( p_input != NULL )
-    {
-        vlc_mutex_lock( &p_input->stream.stream_lock );
-    }
-
     if( p_intf->p_sys->b_intf_update )
     {
         vlc_bool_t b_input = VLC_FALSE;
@@ -657,31 +755,23 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
         if( ( b_input = ( p_input != NULL ) ) )
         {
+            vlc_mutex_lock( &p_input->stream.stream_lock );
+
             /* seekable streams */
             b_seekable = p_input->stream.b_seekable;
 
-            /* control buttons for free pace streams */
+            /* check wether slow/fast motion is possible*/
             b_control = p_input->stream.b_pace_control; 
 
             /* chapters & titles */
             b_chapters = p_input->stream.i_area_nb > 1; 
-
-            /* play status */
-            p_intf->p_sys->b_play_status = 
-                p_input->stream.control.i_status != PAUSE_S;
+ 
+            vlc_mutex_unlock( &p_input->stream.stream_lock );
         }
-        else
-        {
-            /* play status */
-            p_intf->p_sys->b_play_status = FALSE;
-            [self setSubmenusEnabled: FALSE];
-        }
-
-        [self playStatusUpdated: p_intf->p_sys->b_play_status];
 
         [o_btn_stop setEnabled: b_input];
-        [o_btn_faster setEnabled: b_control];
-        [o_btn_slower setEnabled: b_control];
+        [o_btn_ff setEnabled: b_seekable];
+        [o_btn_rewind setEnabled: b_seekable];
         [o_btn_prev setEnabled: (b_plmul || b_chapters)];
         [o_btn_next setEnabled: (b_plmul || b_chapters)];
 
@@ -690,76 +780,83 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
         [o_timefield setStringValue: @"0:00:00"];
 
         [self manageVolumeSlider];
-
         p_intf->p_sys->b_intf_update = VLC_FALSE;
     }
 
-#define p_area p_input->stream.p_selected_area
+    if (p_intf->p_sys->b_fullscreen_update )
+    {
+        vout_thread_t * p_vout;
+        vlc_value_t val;
+        if (var_Change (p_intf, "fullscreen", VLC_VAR_INHERITVALUE, &val, NULL)>=0 &&  val.b_bool )
+        {
+            [o_btn_fullscreen setState:VLC_TRUE];
+        }
+        else
+        {
+            [o_btn_fullscreen setState:VLC_FALSE];
+        }
+        p_vout = vlc_object_find(p_intf,VLC_OBJECT_VOUT,FIND_ANYWHERE);
+        if (p_vout != NULL)
+        {
+            [o_btn_fullscreen setEnabled: VLC_TRUE];
+            vlc_object_release(p_vout);
+        }
+        else
+        {
+            [o_btn_fullscreen setEnabled: VLC_FALSE];
+        }
+        p_intf->p_sys->b_fullscreen_update = VLC_FALSE;
+    }
 
     if( p_intf->p_sys->b_playing && p_input != NULL )
     {
-        vlc_bool_t b_field_update = TRUE;
-
-        if( !p_input->b_die && ( p_intf->p_sys->b_play_status !=
-            ( p_input->stream.control.i_status != PAUSE_S ) ) ) 
-        {
-            p_intf->p_sys->b_play_status =
-                !p_intf->p_sys->b_play_status;
-
-            [self playStatusUpdated: p_intf->p_sys->b_play_status]; 
-        }
+        vlc_value_t time;
+        NSString * o_time;
+        mtime_t i_seconds;
 
         if( p_input->stream.b_seekable )
         {
-            if( f_slider == f_slider_old )
+            vlc_value_t pos;
+            float f_updated;
+            
+            var_Get( p_input, "position", &pos );
+            f_updated = 10000. * pos.f_float;
+
+            if( f_slider != f_updated )
             {
-                float f_updated = ( 10000. * p_area->i_tell ) /
-                                           p_area->i_size;
-
-                if( f_slider != f_updated )
-                {
-                    [o_timeslider setFloatValue: f_updated];
-                }
-            }
-            else
-            {
-                off_t i_seek = ( f_slider * p_area->i_size ) / 10000;
-
-                /* release the lock to be able to seek */
-                vlc_mutex_unlock( &p_input->stream.stream_lock );
-                vlc_mutex_unlock( &p_playlist->object_lock );
-                playlist_Play( p_playlist );
-                input_Seek( p_input, i_seek, INPUT_SEEK_SET );
-                vlc_mutex_lock( &p_playlist->object_lock );
-                vlc_mutex_lock( &p_input->stream.stream_lock );
-
-                /* update the old value */
-                f_slider_old = f_slider; 
-
-                b_field_update = VLC_FALSE;
+                [o_timeslider setFloatValue: f_updated];
             }
         }
-
-        if( b_field_update )
-        {
-            NSString * o_time;
-            char psz_time[ OFFSETTOTIME_MAX_SIZE ];
-
-            input_OffsetToTime( p_input, psz_time, p_area->i_tell );
-
-            o_time = [NSString stringWithUTF8String: psz_time];
-            [o_timefield setStringValue: o_time];
-        }
-
-        /* disable screen saver */
-        UpdateSystemActivity( UsrActivity );
+            
+        var_Get( p_input, "time", &time );
+        i_seconds = time.i_time / 1000000;
+        
+        o_time = [NSString stringWithFormat: @"%d:%02d:%02d",
+                        (int) (i_seconds / (60 * 60)),
+                        (int) (i_seconds / 60 % 60),
+                        (int) (i_seconds % 60)];
+        [o_timefield setStringValue: o_time];
     }
-
-#undef p_area
-
-    if( p_input != NULL )
+    if( p_input )
     {
-        vlc_mutex_unlock( &p_input->stream.stream_lock );
+        vlc_value_t val;
+        var_Get( p_input, "state", &val );
+
+        if( val.i_int != PAUSE_S )
+        {
+            p_intf->p_sys->b_play_status = TRUE;
+        }
+        else
+        {
+            p_intf->p_sys->b_play_status = FALSE;
+        }
+        [self playStatusUpdated: p_intf->p_sys->b_play_status];
+    }
+    else
+    {
+        p_intf->p_sys->b_play_status = FALSE;
+        [self playStatusUpdated: p_intf->p_sys->b_play_status];
+        [self setSubmenusEnabled: FALSE];
     }
 
 #undef p_input
@@ -812,6 +909,9 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
             
                 [o_controls setupVarMenuItem: o_mi_device target: (vlc_object_t *)p_aout
                     var: "audio-device" selector: @selector(toggleVar:)];
+                    
+                [o_controls setupVarMenuItem: o_mi_visual target: (vlc_object_t *)p_aout
+                    var: "visual" selector: @selector(toggleVar:)];
                 vlc_object_release( (vlc_object_t *)p_aout );
             }
             
@@ -918,6 +1018,7 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     if( b_pause )
     {
         [o_btn_play setImage: o_img_pause];
+        [o_btn_play setAlternateImage: o_img_pause_pressed];
         [o_btn_play setToolTip: _NS("Pause")];
         [o_mi_play setTitle: _NS("Pause")];
         [o_dmi_play setTitle: _NS("Pause")];
@@ -925,6 +1026,7 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     else
     {
         [o_btn_play setImage: o_img_play];
+        [o_btn_play setAlternateImage: o_img_play_pressed];
         [o_btn_play setToolTip: _NS("Play")];
         [o_mi_play setTitle: _NS("Play")];
         [o_dmi_play setTitle: _NS("Play")];
@@ -937,6 +1039,7 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
     [o_mi_title setEnabled: b_enabled];
     [o_mi_chapter setEnabled: b_enabled];
     [o_mi_audiotrack setEnabled: b_enabled];
+    [o_mi_visual setEnabled: b_enabled];
     [o_mi_videotrack setEnabled: b_enabled];
     [o_mi_subtitle setEnabled: b_enabled];
     [o_mi_channels setEnabled: b_enabled];
@@ -960,15 +1063,14 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
 - (IBAction)timesliderUpdate:(id)sender
 {
+    intf_thread_t * p_intf;
+    input_thread_t * p_input;
     float f_updated;
 
     switch( [[NSApp currentEvent] type] )
     {
         case NSLeftMouseUp:
         case NSLeftMouseDown:
-            f_slider = [sender floatValue];
-            return;
-
         case NSLeftMouseDragged:
             f_updated = [sender floatValue];
             break;
@@ -977,70 +1079,79 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
             return;
     }
 
-    intf_thread_t * p_intf = [NSApp getIntf];
+    p_intf = [NSApp getIntf];
+    p_input = vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                            FIND_ANYWHERE );
 
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-
-    if( p_playlist == NULL )
+    if( p_input != NULL )
     {
-        return;
-    }
-
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input != NULL )
-    {
-        off_t i_tell;
+        vlc_value_t time;
+        mtime_t i_seconds;
         NSString * o_time;
-        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
 
-#define p_area p_playlist->p_input->stream.p_selected_area
-        vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
-        i_tell = f_updated / 10000. * p_area->i_size;
-        input_OffsetToTime( p_playlist->p_input, psz_time, i_tell );
-        vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
-#undef p_area
-
-        o_time = [NSString stringWithUTF8String: psz_time];
-        [o_timefield setStringValue: o_time]; 
+        if( p_input->stream.b_seekable )
+        {
+                vlc_value_t pos;
+                pos.f_float = f_updated / 10000.;
+                if( f_slider != f_updated )
+                {
+                    var_Set( p_input, "position", pos );
+                    [o_timeslider setFloatValue: f_updated];
+                }
+        }
+            
+        var_Get( p_input, "time", &time );
+        i_seconds = time.i_time / 1000000;
+        
+        o_time = [NSString stringWithFormat: @"%d:%02d:%02d",
+                        (int) (i_seconds / (60 * 60)),
+                        (int) (i_seconds / 60 % 60),
+                        (int) (i_seconds % 60)];
+        [o_timefield setStringValue: o_time];
+        vlc_object_release( p_input );
     }
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
 }
 
 - (void)terminate
 {
     NSEvent * o_event;
-    vout_thread_t * p_vout;
     playlist_t * p_playlist;
+    vout_thread_t * p_vout;
     intf_thread_t * p_intf = [NSApp getIntf];
 
-    /*
-     * Free playlists
-     */
-    msg_Dbg( p_intf, "removing all playlists" );
-    while( (p_playlist = vlc_object_find( p_intf->p_vlc, VLC_OBJECT_PLAYLIST,
-                                          FIND_CHILD )) )
+    /* Stop playback */
+    if( ( p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                        FIND_ANYWHERE ) ) )
     {
-        vlc_object_detach( p_playlist );
+        playlist_Stop( p_playlist );
         vlc_object_release( p_playlist );
-        playlist_Destroy( p_playlist );
     }
 
-    /*
-     * Free video outputs
-     */
-    msg_Dbg( p_intf, "removing all video outputs" );
-    while( (p_vout = vlc_object_find( p_intf->p_vlc, 
-                                      VLC_OBJECT_VOUT, FIND_CHILD )) )
+    /* FIXME - Wait here until all vouts are terminated because
+       libvlc's VLC_Stop destroys interfaces before vouts, which isn't
+       good on OS X. We definitly need a cleaner way to handle this,
+       but this may hopefully be good enough for now.
+         -- titer 2003/11/22 */
+    while( ( p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                       FIND_ANYWHERE ) ) )
     {
-        vlc_object_detach( p_vout );
         vlc_object_release( p_vout );
-        vout_Destroy( p_vout );
+        msleep( 100000 );
     }
+    msleep( 500000 );
 
+    if( o_img_pause_pressed != nil )
+    {
+        [o_img_pause_pressed release];
+        o_img_pause_pressed = nil;
+    }
+    
+    if( o_img_pause_pressed != nil )
+    {
+        [o_img_pause_pressed release];
+        o_img_pause_pressed = nil;
+    }
+    
     if( o_img_pause != nil )
     {
         [o_img_pause release];
@@ -1129,7 +1240,7 @@ int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
 
 - (IBAction)openWebsite:(id)sender
 {
-    NSURL * o_url = [NSURL URLWithString: @"http://www.videolan.org"];
+    NSURL * o_url = [NSURL URLWithString: @"http://www.videolan.org/"];
 
     [[NSWorkspace sharedWorkspace] openURL: o_url];
 }
