@@ -2,7 +2,7 @@
  * vout.m: MacOS X video output plugin
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: vout.m,v 1.50 2003/06/03 23:17:43 massiot Exp $
+ * $Id: vout.m,v 1.68 2003/12/22 17:08:22 bigben Exp $
  *
  * Authors: Colin Delacroix <colin@zoy.org>
  *          Florian G. Pflug <fgp@phlo.org>
@@ -33,6 +33,8 @@
 
 #include <QuickTime/QuickTime.h>
 
+#include <vlc_keys.h>
+
 #include "intf.h"
 #include "vout.h"
 
@@ -51,6 +53,7 @@ struct picture_sys_t
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
+
 static int  vout_Init      ( vout_thread_t * );
 static void vout_End       ( vout_thread_t * );
 static int  vout_Manage    ( vout_thread_t * );
@@ -131,8 +134,7 @@ int E_(OpenVideo) ( vlc_object_t *p_this )
     p_vout->p_sys->p_matrix = (MatrixRecordPtr)malloc( sizeof(MatrixRecord) );
     p_vout->p_sys->p_fullscreen_state = NULL;
 
-    p_vout->p_sys->b_mouse_pointer_visible = YES;
-    p_vout->p_sys->b_mouse_moved = YES;
+    p_vout->p_sys->b_mouse_moved = VLC_TRUE;
     p_vout->p_sys->i_time_mouse_last_moved = mdate();
 
     /* set window size */
@@ -367,47 +369,19 @@ static int vout_Manage( vout_thread_t *p_vout )
      * it has to deal with multiple monitors and therefore checks a lot */
     if( !p_vout->p_sys->b_mouse_moved && p_vout->b_fullscreen )
     {
-        if( mdate() - p_vout->p_sys->i_time_mouse_last_moved > 2000000 && 
-                   p_vout->p_sys->b_mouse_pointer_visible )
+        if( mdate() - p_vout->p_sys->i_time_mouse_last_moved > 3000000 )
         {
             VLCHideMouse( p_vout, YES );
         }
-        else if ( !p_vout->p_sys->b_mouse_pointer_visible )
-        {
-            vlc_bool_t b_playing = NO;
-            playlist_t * p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST,
-                                                                    FIND_ANYWHERE );
-
-            if ( p_playlist != nil )
-            {
-                vlc_mutex_lock( &p_playlist->object_lock );
-                if( p_playlist->p_input != NULL )
-                {
-                    vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
-                    b_playing = p_playlist->p_input->stream.control.i_status != PAUSE_S;
-                    vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
-                }
-                vlc_mutex_unlock( &p_playlist->object_lock );
-                vlc_object_release( p_playlist );
-            }
-            if ( !b_playing )
-            {
-                VLCHideMouse( p_vout, NO );
-            }
-        }
     }
     else if ( p_vout->p_sys->b_mouse_moved && p_vout->b_fullscreen )
     {
-        if( !p_vout->p_sys->b_mouse_pointer_visible )
-        {
-            VLCHideMouse( p_vout, NO );
-        }
-        else
-        {
-            p_vout->p_sys->b_mouse_moved = NO;
-        }
+        VLCHideMouse( p_vout, NO );
     }
     
+    /* disable screen saver */
+    UpdateSystemActivity( UsrActivity );
+    
     return( 0 );
 }
 
@@ -443,6 +417,8 @@ static void vout_Display( vout_thread_t *p_vout, picture_t *p_pic )
 static int CoSendRequest( vout_thread_t *p_vout, SEL sel )
 {
     int i_ret = 0;
+    vlc_value_t val;
+    intf_thread_t * p_intf;
 
     VLCVout * o_vlv = [[VLCVout alloc] init];
 
@@ -453,6 +429,16 @@ static int CoSendRequest( vout_thread_t *p_vout, SEL sel )
 
     [o_vlv release];
 
+    /*This makes this function dependant of the presence of a macosx 
+    interface. We do not check if this interface exists, since it has 
+    already been done before.*/
+
+    p_intf = [NSApp getIntf];
+
+    val.b_bool = VLC_TRUE;
+    var_Create(p_intf,"intf-change",VLC_VAR_BOOL);
+    var_Set(p_intf, "intf-change",val);
+
     return( i_ret );
 }
 
@@ -468,7 +454,7 @@ static int CoCreateWindow( vout_thread_t *p_vout )
         msg_Err( p_vout, "CoSendRequest (createWindow) failed" );
         return( 1 );
     }
-
+    
     return( 0 );
 }
 
@@ -479,10 +465,8 @@ static int CoCreateWindow( vout_thread_t *p_vout )
  *****************************************************************************/
 static int CoDestroyWindow( vout_thread_t *p_vout )
 {
-    if( !p_vout->p_sys->b_mouse_pointer_visible )
-    {
-        VLCHideMouse( p_vout, NO );
-    }
+
+    VLCHideMouse( p_vout, NO );
 
     if( CoSendRequest( p_vout, @selector(destroyWindow:) ) )
     {
@@ -510,8 +494,6 @@ static int CoToggleFullscreen( vout_thread_t *p_vout )
     
     p_vout->b_fullscreen = !p_vout->b_fullscreen;
 
-    config_PutInt( p_vout, "fullscreen", p_vout->b_fullscreen );
-
     if( CoCreateWindow( p_vout ) )
     {
         msg_Err( p_vout, "unable to create window" );
@@ -549,13 +531,11 @@ static void VLCHideMouse ( vout_thread_t *p_vout, BOOL b_hide )
     if ( b_hide && b_inside )
     {
         /* only hide if mouse over VLCView */
-        [NSCursor hide];
-        p_vout->p_sys->b_mouse_pointer_visible = 0;
+        [NSCursor setHiddenUntilMouseMoves: YES];
     }
     else if ( !b_hide )
     {
-        [NSCursor unhide];
-        p_vout->p_sys->b_mouse_pointer_visible = 1;
+        [NSCursor setHiddenUntilMouseMoves: NO];
     }
     p_vout->p_sys->b_mouse_moved = NO;
     p_vout->p_sys->i_time_mouse_last_moved = mdate();
@@ -612,6 +592,7 @@ static void QTScaleMatrix( vout_thread_t *p_vout )
     TranslateMatrix( p_vout->p_sys->p_matrix, 
                      Long2Fix(i_offset_x), 
                      Long2Fix(i_offset_y) );
+
 }
 
 /*****************************************************************************
@@ -820,20 +801,21 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 
 - (void)toggleFloatOnTop
 {
-    if( config_GetInt( p_vout, "macosx-float" ) )
+    if( config_GetInt( p_vout, "video-on-top" ) )
     {
-        config_PutInt( p_vout, "macosx-float", 0 );
+        config_PutInt( p_vout, "video-on-top", 0 );
         [p_vout->p_sys->o_window setLevel: NSNormalWindowLevel];
     }
     else
     {
-        config_PutInt( p_vout, "macosx-float", 1 );
+        config_PutInt( p_vout, "video-on-top", 1 );
         [p_vout->p_sys->o_window setLevel: NSStatusWindowLevel];
     }
 }
 
 - (void)toggleFullscreen
 {
+    config_PutInt(p_vout, "fullscreen", !p_vout->b_fullscreen);
     p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
 }
 
@@ -851,63 +833,51 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 {
     unichar key = 0;
     vlc_value_t val;
+    unsigned int i_pressed_modifiers = 0;
+    val.i_int = 0;
+    
+    i_pressed_modifiers = [o_event modifierFlags];
 
-    if( [[o_event characters] length] )
+    if( i_pressed_modifiers & NSShiftKeyMask )
+        val.i_int |= KEY_MODIFIER_SHIFT;
+    if( i_pressed_modifiers & NSControlKeyMask )
+        val.i_int |= KEY_MODIFIER_CTRL;
+    if( i_pressed_modifiers & NSAlternateKeyMask )
+        val.i_int |= KEY_MODIFIER_ALT;
+    if( i_pressed_modifiers & NSCommandKeyMask )
+        val.i_int |= KEY_MODIFIER_COMMAND;
+
+    key = [[o_event charactersIgnoringModifiers] characterAtIndex: 0];
+
+    if( key )
     {
-        key = [[o_event characters] characterAtIndex: 0];
+        /* Escape should always get you out of fullscreen */
+        if( key == (unichar) 0x1b )
+        {
+             if( [self isFullscreen] )
+             {
+                 [self toggleFullscreen];
+             }
+        }
+        else if ( key == ' ' )
+        {
+             playlist_t *p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST,
+                                                     FIND_ANYWHERE );
+             if ( p_playlist != NULL )
+             {
+                 playlist_Pause( p_playlist );
+                 vlc_object_release( p_playlist);
+             }
+        }
+        else
+        {
+            val.i_int |= CocoaKeyToVLC( key );
+            var_Set( p_vout->p_vlc, "key-pressed", val );
+        }
     }
-
-    switch( key )
+    else
     {
-        case 'f': case 'F':
-            [self toggleFullscreen];
-            break;
-
-        case (unichar)0x1b: /* escape */
-            if( [self isFullscreen] )
-            {
-                [self toggleFullscreen];
-            }
-            break;
-
-        case 'q': case 'Q':
-            p_vout->p_vlc->b_die = VLC_TRUE;
-            break;
-
-        case ' ':
-            input_SetStatus( p_vout, INPUT_STATUS_PAUSE );
-            break;
-
-        case (unichar)0xf700: /* arrow up */
-            val.psz_string = "UP";
-            var_Set( p_vout, "key-pressed", val );
-            break;
-
-        case (unichar)0xf701: /* arrow down */
-            val.psz_string = "DOWN";
-            var_Set( p_vout, "key-pressed", val );
-            break;
-
-        case (unichar)0xf702: /* arrow left */
-            val.psz_string = "LEFT";
-            var_Set( p_vout, "key-pressed", val );
-            break;
-
-        case (unichar)0xf703: /* arrow right */
-            val.psz_string = "RIGHT";
-            var_Set( p_vout, "key-pressed", val );
-            break;
-
-        case (unichar)0xd: /* return */
-        case (unichar)0x3: /* enter */
-            val.psz_string = "ENTER";
-            var_Set( p_vout, "key-pressed", val );
-            break;
-
-
-        default:
-            [super keyDown: o_event];
-            break;
+        [super keyDown: o_event];
     }
 }
 
@@ -924,7 +894,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 
     vlc_mutex_lock( &p_playlist->object_lock );
     o_title = [NSMutableString stringWithUTF8String: 
-        p_playlist->pp_items[p_playlist->i_index]->psz_name]; 
+        p_playlist->pp_items[p_playlist->i_index]->psz_uri]; 
     vlc_mutex_unlock( &p_playlist->object_lock );
 
     vlc_object_release( p_playlist );
@@ -1204,13 +1174,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
         p_vout->p_sys->i_time_mouse_last_moved = mdate();
         p_vout->p_sys->b_mouse_moved = YES;
     }
-    else if ( !b_inside && !p_vout->p_sys->b_mouse_pointer_visible )
-    {
-        /* people with multiple monitors need their mouse,
-         * even if VLCView in fullscreen. */
-        VLCHideMouse( p_vout, NO );
-    }
-    
+
     [super mouseMoved: o_event];
 }
 
@@ -1274,7 +1238,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
             backing: NSBackingStoreBuffered
             defer: NO screen: o_screen];
 
-        [p_vout->p_sys->o_window setLevel: NSPopUpMenuWindowLevel - 1];
+        //[p_vout->p_sys->o_window setLevel: NSPopUpMenuWindowLevel - 1];
         p_vout->p_sys->b_mouse_moved = YES;
         p_vout->p_sys->i_time_mouse_last_moved = mdate();
     }
@@ -1297,7 +1261,7 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 
         [p_vout->p_sys->o_window setAlphaValue: config_GetFloat( p_vout, "macosx-opaqueness" )];
         
-        if( config_GetInt( p_vout, "macosx-float" ) )
+        if( config_GetInt( p_vout, "video-on-top" ) )
         {
             [p_vout->p_sys->o_window setLevel: NSStatusWindowLevel];
         }
@@ -1315,10 +1279,12 @@ static void QTFreePicture( vout_thread_t *p_vout, picture_t *p_pic )
 
     [o_view lockFocus];
     p_vout->p_sys->p_qdport = [o_view qdPort];
+
     [o_view unlockFocus];
     
     [p_vout->p_sys->o_window updateTitle];
     [p_vout->p_sys->o_window makeKeyAndOrderFront: nil];
+
 }
 
 - (void)destroyWindow:(NSValue *)o_value
