@@ -2,7 +2,7 @@
  * event.h: Event class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: event.h,v 1.12 2003/06/22 12:46:49 asmax Exp $
+ * $Id: event.h,v 1.15 2003/10/17 18:17:28 ipkiss Exp $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -59,9 +59,10 @@ using namespace std;
 #define VLC_OPEN            (VLC_MESSAGE + 5)
 #define VLC_LOAD_SKIN       (VLC_MESSAGE + 6)
 #define VLC_DROP            (VLC_MESSAGE + 7)
+#define VLC_ON_TOP          (VLC_MESSAGE + 8)
+#define VLC_NET             (VLC_MESSAGE + 9)
 
 #define VLC_LOG_SHOW        (VLC_MESSAGE + 20)
-#define VLC_LOG_CLEAR       (VLC_MESSAGE + 22)
 #define VLC_PREFS_SHOW      (VLC_MESSAGE + 23)
 #define VLC_INFO_SHOW       (VLC_MESSAGE + 24)
 
@@ -95,9 +96,6 @@ using namespace std;
 #define VLC_PLAYLIST_ADD_FILE (VLC_MESSAGE + 301)
 #define VLC_TEST_ALL_CLOSED (VLC_MESSAGE + 600)
 
-// Network events
-#define VLC_NET_ADDUDP      (VLC_MESSAGE + 701)
-
 // Window event
 #define WINDOW_MOVE         (VLC_WINDOW + 1)
 #define WINDOW_OPEN         (VLC_WINDOW + 2)
