@@ -2,7 +2,7 @@
  * preferences.cpp : wxWindows plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: preferences.cpp,v 1.24 2003/07/24 16:07:10 gbazin Exp $
+ * $Id: preferences.cpp,v 1.44 2003/12/22 02:24:52 sam Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -30,18 +30,12 @@
 #include <stdio.h>
 
 #include <vlc/vlc.h>
+#include <vlc/intf.h>
 
-#ifdef WIN32                                                 /* mingw32 hack */
-#undef Yield
-#undef CreateDialog
-#endif
+#include <vlc_help.h>
 
-/* Let vlc take care of the i18n stuff */
-#define WXINTL_NO_GETTEXT_MACRO
-
-#include <wx/wxprec.h>
-#include <wx/wx.h>
-#include <wx/window.h>
+#include "wxwindows.h"
+#include "preferences_widgets.h"
 #include <wx/notebook.h>
 #include <wx/textctrl.h>
 #include <wx/combobox.h>
@@ -51,18 +45,14 @@
 #include <wx/clntdata.h>
 #include <wx/dynarray.h>
 
-#include <vlc/intf.h>
-
-#if defined MODULE_NAME_IS_skins
-#   include "../skins/src/skin_common.h"
-#endif
-
-#include "wxwindows.h"
-
 #ifndef wxRB_SINGLE
 #   define wxRB_SINGLE 0
 #endif
 
+#define GENERAL_ID 1242
+#define PLUGIN_ID 1243
+#define CAPABILITY_ID 1244
+
 /*****************************************************************************
  * Classes declarations.
  *****************************************************************************/
@@ -82,6 +72,8 @@ public:
 private:
     /* Event handlers (these functions should _not_ be virtual) */
     void OnSelectTreeItem( wxTreeEvent& event );
+    void OnAdvanced( wxCommandEvent& event );
+
     ConfigTreeData *FindModuleConfig( ConfigTreeData *config_data );
 
     DECLARE_EVENT_TABLE()
@@ -90,37 +82,14 @@ private:
     PrefsDialog *p_prefs_dialog;
     wxBoxSizer *p_sizer;
     wxWindow *p_parent;
+    vlc_bool_t b_advanced;
 
     wxTreeItemId root_item;
+    wxTreeItemId general_item;
     wxTreeItemId plugins_item;
 };
 
-struct ConfigData
-{
-    ConfigData( wxPanel *_panel, int _i_conf_type,
-                vlc_bool_t _b_advanced, char *psz_name )
-    { panel = _panel; b_advanced = _b_advanced; b_config_list = VLC_FALSE;
-      i_config_type = _i_conf_type; option_name = wxU(psz_name); }
-
-    vlc_bool_t b_advanced;
-    int i_config_type;
-    vlc_bool_t b_config_list;
-
-    union control
-    {
-        wxComboBox *combobox;
-        wxRadioButton *radio;
-        wxSpinCtrl *spinctrl;
-        wxCheckBox *checkbox;
-        wxTextCtrl *textctrl;
-
-    } control;
-
-    wxPanel *panel;
-    wxString option_name;
-};
-
-WX_DEFINE_ARRAY(ConfigData *, ArrayOfConfigData);
+WX_DEFINE_ARRAY(ConfigControl *, ArrayOfConfigControls);
 
 class PrefsPanel : public wxPanel
 {
@@ -128,15 +97,13 @@ public:
 
     PrefsPanel() { }
     PrefsPanel( wxWindow *parent, intf_thread_t *_p_intf,
-                PrefsDialog *_p_prefs_dialog, int i_object_id, char * );
+                PrefsDialog *, int i_object_id, char *, char * );
     virtual ~PrefsPanel() {}
 
     void ApplyChanges();
+    void SwitchAdvanced( vlc_bool_t );
 
 private:
-    void OnAdvanced( wxCommandEvent& WXUNUSED(event) );
-    DECLARE_EVENT_TABLE()
-
     intf_thread_t *p_intf;
     PrefsDialog *p_prefs_dialog;
 
@@ -145,15 +112,18 @@ private:
     wxBoxSizer *config_sizer;
     wxScrolledWindow *config_window;
 
-    ArrayOfConfigData config_array;
+    ArrayOfConfigControls config_array;
 };
 
 class ConfigTreeData : public wxTreeItemData
 {
 public:
 
-    ConfigTreeData() { b_submodule = 0; panel = NULL; psz_section = NULL; }
-    virtual ~ConfigTreeData() { if( panel ) delete panel; }
+    ConfigTreeData() { b_submodule = 0; panel = NULL; psz_section = NULL;
+                       psz_help = NULL; }
+    virtual ~ConfigTreeData() { if( panel ) delete panel;
+                                if( psz_section) free(psz_section);
+                                if( psz_help) free(psz_help); }
 
     vlc_bool_t b_submodule;
 
@@ -161,22 +131,7 @@ public:
     wxBoxSizer *sizer;
     int i_object_id;
     char *psz_section;
-};
-
-class ConfigEvtHandler : public wxEvtHandler
-{
-public:
-    ConfigEvtHandler( intf_thread_t *p_intf, PrefsDialog *p_prefs_dialog );
-    virtual ~ConfigEvtHandler();
-
-    void ConfigEvtHandler::OnCommandEvent( wxCommandEvent& event );
-
-private:
-
-    DECLARE_EVENT_TABLE()
-
-    intf_thread_t *p_intf;
-    PrefsDialog *p_prefs_dialog;
+    char *psz_help;
 };
 
 /*****************************************************************************
@@ -198,6 +153,7 @@ BEGIN_EVENT_TABLE(PrefsDialog, wxFrame)
     EVT_BUTTON(wxID_CANCEL, PrefsDialog::OnCancel)
     EVT_BUTTON(wxID_SAVE, PrefsDialog::OnSave)
     EVT_BUTTON(ResetAll_Event, PrefsDialog::OnResetAll)
+    EVT_CHECKBOX(Advanced_Event, PrefsDialog::OnAdvanced)
 
     /* Don't destroy the window when the user closes it */
     EVT_CLOSE(PrefsDialog::OnCancel)
@@ -211,19 +167,7 @@ enum
 
 BEGIN_EVENT_TABLE(PrefsTreeCtrl, wxTreeCtrl)
     EVT_TREE_SEL_CHANGED(PrefsTree_Ctrl, PrefsTreeCtrl::OnSelectTreeItem)
-END_EVENT_TABLE()
-
-BEGIN_EVENT_TABLE(PrefsPanel, wxPanel)
-    /* Button events */
-    EVT_CHECKBOX(Advanced_Event, PrefsPanel::OnAdvanced)
-
-END_EVENT_TABLE()
-
-BEGIN_EVENT_TABLE(ConfigEvtHandler, wxEvtHandler)
-    EVT_BUTTON(-1, ConfigEvtHandler::OnCommandEvent)
-    EVT_TEXT(-1, ConfigEvtHandler::OnCommandEvent)
-    EVT_RADIOBOX(-1, ConfigEvtHandler::OnCommandEvent)
-    EVT_SPINCTRL(-1, ConfigEvtHandler::OnCommandEvent)
+    EVT_COMMAND(Advanced_Event, wxEVT_USER_FIRST, PrefsTreeCtrl::OnAdvanced)
 END_EVENT_TABLE()
 
 /*****************************************************************************
@@ -231,7 +175,7 @@ END_EVENT_TABLE()
  *****************************************************************************/
 PrefsDialog::PrefsDialog( intf_thread_t *_p_intf, wxWindow *p_parent)
   :  wxFrame( p_parent, -1, wxU(_("Preferences")), wxDefaultPosition,
-              wxSize(650,450), wxDEFAULT_FRAME_STYLE )
+              wxSize(700,450), wxDEFAULT_FRAME_STYLE )
 {
     /* Initializations */
     p_intf = _p_intf;
@@ -258,12 +202,27 @@ PrefsDialog::PrefsDialog( intf_thread_t *_p_intf, wxWindow *p_parent)
     wxButton *reset_button = new wxButton( panel, ResetAll_Event,
                                            wxU(_("Reset All")) );
 
+    wxPanel *dummy_panel = new wxPanel( this, -1 );
+    wxCheckBox *advanced_checkbox =
+        new wxCheckBox( panel, Advanced_Event, wxU(_("Advanced options")) );
+
+    if( config_GetInt( p_intf, "advanced" ) )
+    {
+        advanced_checkbox->SetValue(TRUE);
+        wxCommandEvent dummy_event;
+        dummy_event.SetInt(TRUE);
+        OnAdvanced( dummy_event );
+    }
+
     /* Place everything in sizers */
     wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
     button_sizer->Add( ok_button, 0, wxALL, 5 );
     button_sizer->Add( cancel_button, 0, wxALL, 5 );
     button_sizer->Add( save_button, 0, wxALL, 5 );
     button_sizer->Add( reset_button, 0, wxALL, 5 );
+    button_sizer->Add( dummy_panel, 1, wxALL, 5 );
+    button_sizer->Add( advanced_checkbox, 0, wxALL | wxALIGN_RIGHT |
+                       wxALIGN_CENTER_VERTICAL, 0 );
     button_sizer->Layout();
 
     wxBoxSizer *main_sizer = new wxBoxSizer( wxVERTICAL );
@@ -271,7 +230,7 @@ PrefsDialog::PrefsDialog( intf_thread_t *_p_intf, wxWindow *p_parent)
     panel_sizer->Add( controls_sizer, 1, wxEXPAND | wxALL, 5 );
     panel_sizer->Add( static_line, 0, wxEXPAND | wxALL, 5 );
     panel_sizer->Add( button_sizer, 0, wxALIGN_LEFT | wxALIGN_BOTTOM |
-                      wxALL, 5 );
+                      wxALL | wxEXPAND, 5 );
     panel_sizer->Layout();
     panel->SetSizer( panel_sizer );
     main_sizer->Add( panel, 1, wxEXPAND, 0 );
@@ -308,6 +267,7 @@ void PrefsDialog::OnSave( wxCommandEvent& WXUNUSED(event) )
 {
     prefs_tree->ApplyChanges();
     config_SaveConfigFile( p_intf, NULL );
+    this->Hide();
 }
 
 void PrefsDialog::OnResetAll( wxCommandEvent& WXUNUSED(event) )
@@ -326,6 +286,51 @@ void PrefsDialog::OnResetAll( wxCommandEvent& WXUNUSED(event) )
     }
 }
 
+void PrefsDialog::OnAdvanced( wxCommandEvent& event )
+{
+    wxCommandEvent newevent( wxEVT_USER_FIRST, Advanced_Event );
+    newevent.SetInt( event.GetInt() );
+
+    prefs_tree->AddPendingEvent( newevent );
+}
+
+/*****************************************************************************
+ * GetCapabilityHelp: Display the help for one capability.
+ *****************************************************************************/
+static char * GetCapabilityHelp( char *psz_capability, int i_type)
+{
+    if( psz_capability == NULL) return "";
+
+    if( !strcasecmp(psz_capability,"access") )
+        return i_type == 1 ? ACCESS_TITLE : ACCESS_HELP;
+    if( !strcasecmp(psz_capability,"audio filter") )
+        return i_type == 1 ? AUDIO_FILTER_TITLE : AUDIO_FILTER_HELP;
+    if( !strcasecmp(psz_capability,"audio output") )
+        return i_type == 1 ? AOUT_TITLE : AOUT_HELP;
+    if( !strcasecmp(psz_capability,"audio encoder") )
+        return i_type == 1 ? AOUT_ENC_TITLE : AOUT_ENC_HELP;
+    if( !strcasecmp(psz_capability,"chroma") )
+        return i_type == 1 ? CHROMA_TITLE : CHROMA_HELP;
+    if( !strcasecmp(psz_capability,"decoder") )
+        return i_type == 1 ? DECODER_TITLE : DECODER_HELP;
+    if( !strcasecmp(psz_capability,"demux") )
+        return i_type == 1 ? DEMUX_TITLE : DEMUX_HELP;
+    if( !strcasecmp(psz_capability,"interface") )
+        return i_type == 1 ? INTERFACE_TITLE : INTERFACE_HELP;
+    if( !strcasecmp(psz_capability,"sout access") )
+        return i_type == 1 ? SOUT_TITLE : SOUT_HELP;
+    if( !strcasecmp(psz_capability,"subtitle demux") )
+        return i_type == 1 ? SUBTITLE_DEMUX_TITLE : SUBTITLE_DEMUX_HELP;
+    if( !strcasecmp(psz_capability,"text renderer") )
+        return i_type == 1 ? TEXT_TITLE : TEXT_HELP;
+    if( !strcasecmp(psz_capability,"video output") )
+        return i_type == 1 ? VOUT__TITLE : VOUT_HELP;
+    if( !strcasecmp(psz_capability,"video filter") )
+        return i_type == 1 ? VIDEO_FILTER_TITLE : VIDEO_FILTER_HELP;
+
+    return "";
+}
+
 /*****************************************************************************
  * PrefsTreeCtrl class definition.
  *****************************************************************************/
@@ -347,6 +352,7 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
     p_prefs_dialog = _p_prefs_dialog;
     p_sizer = _p_sizer;
     p_parent = _p_parent;
+    b_advanced = VLC_FALSE;
 
     root_item = AddRoot( wxT("") );
 
@@ -357,6 +363,13 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
     /*
      * Build a tree of the main options
      */
+    ConfigTreeData *config_data = new ConfigTreeData;
+    config_data->i_object_id = GENERAL_ID;
+    config_data->psz_help = wraptext( GENERAL_HELP, 72 , ISUTF8 );
+    config_data->psz_section = strdup( GENERAL_TITLE );
+    general_item = AppendItem( root_item, wxU(_("General settings")),
+                                -1, -1, config_data );
+
     for( i_index = 0; i_index < p_list->i_count; i_index++ )
     {
         p_module = (module_t *)p_list->p_values[i_index].p_object;
@@ -378,24 +391,38 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
             case CONFIG_HINT_CATEGORY:
                 ConfigTreeData *config_data = new ConfigTreeData;
                 config_data->psz_section = strdup(p_item->psz_text);
+                if( p_item->psz_longtext )
+                {
+                    config_data->psz_help =
+                        wraptext( p_item->psz_longtext, 72 , ISUTF8 );
+                }
+                else
+                {
+                    config_data->psz_help = NULL;
+                }
                 config_data->i_object_id = p_module->i_object_id;
 
                 /* Add the category to the tree */
-                AppendItem( root_item, wxU(p_item->psz_text),
+                AppendItem( general_item, wxU(p_item->psz_text),
                             -1, -1, config_data );
                 break;
             }
         }
         while( p_item->i_type != CONFIG_HINT_END && p_item++ );
 
-        SortChildren( root_item );
+        SortChildren( general_item );
     }
 
 
     /*
      * Build a tree of all the plugins
      */
-    plugins_item = AppendItem( root_item, wxU(_("Plugins")) );
+    config_data = new ConfigTreeData;
+    config_data->i_object_id = PLUGIN_ID;
+    config_data->psz_help = wraptext( PLUGIN_HELP, 72, ISUTF8 );
+    config_data->psz_section = strdup( PLUGIN_TITLE );
+    plugins_item = AppendItem( root_item, wxU(_("Plugins")),
+                        -1,-1,config_data );
 
     for( i_index = 0; i_index < p_list->i_count; i_index++ )
     {
@@ -440,8 +467,17 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
             p_module->psz_capability && *p_module->psz_capability )
         {
             /* We didn't find it, add it */
+            ConfigTreeData *config_data = new ConfigTreeData;
+            config_data->psz_section =
+                wraptext( GetCapabilityHelp( p_module->psz_capability , 1 ),
+                          72, ISUTF8 );
+            config_data->psz_help =
+                wraptext( GetCapabilityHelp( p_module->psz_capability , 2 ),
+                          72, ISUTF8 );
+            config_data->i_object_id = CAPABILITY_ID;
             capability_item = AppendItem( plugins_item,
-                                          wxU(p_module->psz_capability) );
+                                          wxU(p_module->psz_capability),
+                                          -1,-1,config_data );
         }
 
         /* Add the plugin to the tree */
@@ -450,6 +486,7 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
         config_data->i_object_id = p_module->b_submodule ?
             ((module_t *)p_module->p_parent)->i_object_id :
             p_module->i_object_id;
+        config_data->psz_help = NULL;
         AppendItem( capability_item, wxU(p_module->psz_object_name), -1, -1,
                     config_data );
     }
@@ -476,6 +513,8 @@ PrefsTreeCtrl::PrefsTreeCtrl( wxWindow *_p_parent, intf_thread_t *_p_intf,
 #ifndef WIN32 /* Workaround a bug in win32 implementation */
     SelectItem( GetFirstChild( root_item, cookie ) );
 #endif
+
+    Expand( general_item );
 }
 
 PrefsTreeCtrl::~PrefsTreeCtrl()
@@ -488,9 +527,9 @@ void PrefsTreeCtrl::ApplyChanges()
     ConfigTreeData *config_data;
 
     /* Apply changes to the main module */
-    wxTreeItemId item = GetFirstChild( root_item, cookie );
+    wxTreeItemId item = GetFirstChild( general_item, cookie );
     for( size_t i_child_index = 0;
-         i_child_index < GetChildrenCount( root_item, FALSE );
+         i_child_index < GetChildrenCount( general_item, FALSE );
          i_child_index++ )
     {
         config_data = (ConfigTreeData *)GetItemData( item );
@@ -499,7 +538,7 @@ void PrefsTreeCtrl::ApplyChanges()
             config_data->panel->ApplyChanges();
         }
 
-        item = GetNextChild( root_item, cookie );
+        item = GetNextChild( general_item, cookie );
     }
 
     /* Apply changes to the plugins */
@@ -540,9 +579,9 @@ void PrefsTreeCtrl::CleanChanges()
     }
 
     /* Clean changes for the main module */
-    wxTreeItemId item = GetFirstChild( root_item, cookie );
+    wxTreeItemId item = GetFirstChild( general_item, cookie );
     for( size_t i_child_index = 0;
-         i_child_index < GetChildrenCount( root_item, FALSE );
+         i_child_index < GetChildrenCount( general_item, FALSE );
          i_child_index++ )
     {
         config_data = (ConfigTreeData *)GetItemData( item );
@@ -552,7 +591,7 @@ void PrefsTreeCtrl::CleanChanges()
             config_data->panel = NULL;
         }
 
-        item = GetNextChild( root_item, cookie );
+        item = GetNextChild( general_item, cookie );
     }
 
     /* Clean changes for the plugins */
@@ -649,39 +688,55 @@ void PrefsTreeCtrl::OnSelectTreeItem( wxTreeEvent& event )
             config_data->panel =
                 new PrefsPanel( p_parent, p_intf, p_prefs_dialog,
                                 config_data->i_object_id,
-                                config_data->psz_section );
+                                config_data->psz_section,
+                                config_data->psz_help );
+            config_data->panel->SwitchAdvanced( b_advanced );
         }
         else
         {
+            config_data->panel->SwitchAdvanced( b_advanced );
             config_data->panel->Show();
         }
 
-        p_sizer->Add( config_data->panel, 2, wxEXPAND | wxALL, 0 );
+        p_sizer->Add( config_data->panel, 3, wxEXPAND | wxALL, 0 );
         p_sizer->Layout();
     }
 }
 
+void PrefsTreeCtrl::OnAdvanced( wxCommandEvent& event )
+{
+    b_advanced = event.GetInt();
+
+    ConfigTreeData *config_data = !GetSelection() ? NULL :
+        FindModuleConfig( (ConfigTreeData *)GetItemData( GetSelection() ) );
+    if( config_data  )
+    {
+        config_data->panel->Hide();
+        p_sizer->Remove( config_data->panel );
+    }
+
+    if( GetSelection() )
+    {
+        wxTreeEvent event;
+        OnSelectTreeItem( event );
+    }
+}
+
 /*****************************************************************************
  * PrefsPanel class definition.
  *****************************************************************************/
 PrefsPanel::PrefsPanel( wxWindow* parent, intf_thread_t *_p_intf,
                         PrefsDialog *_p_prefs_dialog,
-                        int i_object_id, char *psz_section )
+                        int i_object_id, char *psz_section, char *psz_help )
   :  wxPanel( parent, -1, wxDefaultPosition, wxDefaultSize )
 {
     module_config_t *p_item;
-    vlc_list_t      *p_list;
-    module_t        *p_parser;
 
     wxStaticText *label;
-    wxComboBox *combo;
-    wxSpinCtrl *spin;
-    wxCheckBox *checkbox;
-    wxTextCtrl *textctrl;
-    wxButton *button;
+    wxStaticText *help;
     wxArrayString array;
 
-    vlc_bool_t b_has_advanced = VLC_FALSE;
+    module_t *p_module = NULL;
 
     /* Initializations */
     p_intf = _p_intf;
@@ -692,369 +747,168 @@ PrefsPanel::PrefsPanel( wxWindow* parent, intf_thread_t *_p_intf,
 
     wxBoxSizer *sizer = new wxBoxSizer( wxVERTICAL );
 
-    /* Get a pointer to the module */
-    module_t *p_module = (module_t *)vlc_object_get( p_intf, i_object_id );
-    if( p_module->i_object_type != VLC_OBJECT_MODULE )
-    {
-        /* 0OOoo something went really bad */
-        return;
-    }
 
-    /* Enumerate config options and add corresponding config boxes
-     * (submodules don't have config options, they are stored in the
-     *  parent module) */
-    if( p_module->b_submodule )
-        p_item = ((module_t *)p_module->p_parent)->p_config;
+    if( i_object_id == PLUGIN_ID || i_object_id == GENERAL_ID ||
+        i_object_id == CAPABILITY_ID )
+    {
+        label = new wxStaticText( this, -1,wxU(_( psz_section )));
+        wxFont heading_font = label->GetFont();
+        heading_font.SetPointSize( heading_font.GetPointSize() + 5 );
+        label->SetFont( heading_font );
+        sizer->Add( label, 0, wxEXPAND | wxLEFT, 10 );
+        sizer->Add( new wxStaticLine( this, 0 ), 0,
+                    wxEXPAND | wxLEFT | wxRIGHT, 2 );
+
+        help = new wxStaticText( this, -1, wxU(_( psz_help ) ) );
+        sizer->Add( help ,0 ,wxEXPAND | wxALL, 5 );
+
+        config_sizer = NULL; config_window = NULL;
+    }
     else
-        p_item = p_module->p_config;
-
-    /* Find the category if it has been specified */
-    if( psz_section && p_item->i_type == CONFIG_HINT_CATEGORY )
     {
-        while( !p_item->i_type == CONFIG_HINT_CATEGORY ||
-               strcmp( psz_section, p_item->psz_text ) )
+        /* Get a pointer to the module */
+        p_module = (module_t *)vlc_object_get( p_intf,  i_object_id );
+        if( p_module->i_object_type != VLC_OBJECT_MODULE )
         {
-            if( p_item->i_type == CONFIG_HINT_END )
+            /* 0OOoo something went really bad */
+            return;
+        }
+
+        /* Enumerate config options and add corresponding config boxes
+         * (submodules don't have config options, they are stored in the
+         *  parent module) */
+        if( p_module->b_submodule )
+            p_item = ((module_t *)p_module->p_parent)->p_config;
+        else
+            p_item = p_module->p_config;
+
+        /* Find the category if it has been specified */
+        if( psz_section && p_item->i_type == CONFIG_HINT_CATEGORY )
+        {
+            while( !p_item->i_type == CONFIG_HINT_CATEGORY ||
+                   strcmp( psz_section, p_item->psz_text ) )
+            {
+                if( p_item->i_type == CONFIG_HINT_END )
+                    break;
+                p_item++;
+            }
+        }
+
+        /* Add a head title to the panel */
+        label = new wxStaticText( this, -1,
+                                  wxU(_(psz_section ? p_item->psz_text :
+                                  p_module->psz_longname )));
+        wxFont heading_font = label->GetFont();
+        heading_font.SetPointSize( heading_font.GetPointSize() + 5 );
+        label->SetFont( heading_font );
+        sizer->Add( label, 0, wxEXPAND | wxLEFT, 10 );
+        sizer->Add( new wxStaticLine( this, 0 ), 0,
+                    wxEXPAND | wxLEFT | wxRIGHT, 2 );
+
+        /* Now put all the config options into a scrolled window */
+        config_sizer = new wxBoxSizer( wxVERTICAL );
+        config_window = new wxScrolledWindow( this, -1, wxDefaultPosition,
+            wxDefaultSize, wxSTATIC_BORDER | wxHSCROLL | wxVSCROLL );
+        config_window->SetAutoLayout( TRUE );
+        config_window->SetScrollRate( 5, 5 );
+
+        if( p_item ) do
+        {
+            /* If a category has been specified, check we finished the job */
+            if( psz_section && p_item->i_type == CONFIG_HINT_CATEGORY &&
+                strcmp( psz_section, p_item->psz_text ) )
                 break;
-            p_item++;
+
+            ConfigControl *control =
+                CreateConfigControl( VLC_OBJECT(p_intf),
+                                     p_item, config_window );
+
+            /* Don't add items that were not recognized */
+            if( control == NULL ) continue;
+
+            /* Add the config data to our array so we can keep a trace of it */
+            config_array.Add( control );
+
+            config_sizer->Add( control, 0, wxEXPAND | wxALL, 2 );
         }
-    }
+        while( p_item->i_type != CONFIG_HINT_END && p_item++ );
 
-    /* Add a head title to the panel */
-    wxStaticBox *static_box = new wxStaticBox( this, -1, wxT("") );
-    wxStaticBoxSizer *box_sizer = new wxStaticBoxSizer( static_box,
-                                                        wxHORIZONTAL );
-    label = new wxStaticText( this, -1,
-                              wxU(psz_section ? p_item->psz_text :
-                              p_module->psz_longname) );
+        config_sizer->Layout();
+        config_window->SetSizer( config_sizer );
+        sizer->Add( config_window, 1, wxEXPAND | wxALL, 5 );
 
-    box_sizer->Add( label, 1, wxALL, 5 );
-    sizer->Add( box_sizer, 0, wxEXPAND | wxALL, 5 );
-
-    /* Now put all the config options into a scrolled window */
-    config_sizer = new wxBoxSizer( wxVERTICAL );
-    config_window = new wxScrolledWindow( this, -1, wxDefaultPosition,
-        wxDefaultSize, wxSTATIC_BORDER | wxHSCROLL | wxVSCROLL );
-    config_window->SetAutoLayout( TRUE );
-    config_window->SetScrollRate( 5, 5 );
-
-    if( p_item ) do
-    {
-        /* If a category has been specified, check we finished the job */
-        if( psz_section && p_item->i_type == CONFIG_HINT_CATEGORY &&
-            strcmp( psz_section, p_item->psz_text ) )
-            break;
-
-        /* put each config option in a separate panel so we can hide advanced
-         * options easily */
-        wxPanel *panel = new wxPanel( config_window, -1, wxDefaultPosition,
-                                      wxDefaultSize );
-        wxBoxSizer *panel_sizer = new wxBoxSizer( wxHORIZONTAL );
-        ConfigData *config_data =
-            new ConfigData( panel, p_item->i_type,
-                            p_item->b_advanced, p_item->psz_name );
-
-        switch( p_item->i_type )
+        /* And at last put a useful help string if available */
+        if( psz_help && *psz_help )
         {
-        case CONFIG_ITEM_MODULE:
-            label = new wxStaticText(panel, -1, wxU(p_item->psz_text));
-            combo = new wxComboBox( panel, -1, wxU(p_item->psz_value),
-                                    wxDefaultPosition, wxDefaultSize,
-                                    0, NULL, wxCB_READONLY | wxCB_SORT );
-
-            /* build a list of available modules */
-            p_list = vlc_list_find( p_intf, VLC_OBJECT_MODULE, FIND_ANYWHERE );
-            combo->Append( wxU(_("Default")), (void *)NULL );
-            combo->SetSelection( 0 );
-            for( int i_index = 0; i_index < p_list->i_count; i_index++ )
-            {
-                p_parser = (module_t *)p_list->p_values[i_index].p_object ;
-
-                if( !strcmp( p_parser->psz_capability,
-                             p_item->psz_type ) )
-                {
-                    combo->Append( wxU(p_parser->psz_longname),
-                                   p_parser->psz_object_name );
-                    if( p_item->psz_value &&
-                        !strcmp(p_item->psz_value, p_parser->psz_object_name) )
-                        combo->SetValue( wxU(p_parser->psz_longname) );
-                }
-            }
-            vlc_list_release( p_list );
-
-            combo->SetToolTip( wxU(p_item->psz_longtext) );
-            config_data->control.combobox = combo;
-            panel_sizer->Add( label, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-            panel_sizer->Add( combo, 1, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-            if( p_item->b_advanced ) b_has_advanced = VLC_TRUE;
-            break;
-
-        case CONFIG_ITEM_STRING:
-        case CONFIG_ITEM_FILE:
-        case CONFIG_ITEM_DIRECTORY:
-            label = new wxStaticText(panel, -1, wxU(p_item->psz_text));
-            panel_sizer->Add( label, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-
-            if( !p_item->ppsz_list )
-            {
-                textctrl = new wxTextCtrl( panel, -1, wxU(p_item->psz_value),
-                                           wxDefaultPosition, wxDefaultSize,
-                                           wxTE_PROCESS_ENTER);
-                textctrl->SetToolTip( wxU(p_item->psz_longtext) );
-                config_data->control.textctrl = textctrl;
-                panel_sizer->Add( textctrl, 1, wxALL, 5 );
-            }
-            else
-            {
-                combo = new wxComboBox( panel, -1, wxU(p_item->psz_value),
-                                        wxDefaultPosition, wxDefaultSize,
-                                        0, NULL, wxCB_READONLY | wxCB_SORT );
-
-                /* build a list of available options */
-                for( int i_index = 0; p_item->ppsz_list[i_index]; i_index++ )
-                {
-                    combo->Append( wxU(p_item->ppsz_list[i_index]) );
-                    if( p_item->psz_value && !strcmp( p_item->psz_value,
-                                 p_item->ppsz_list[i_index] ) )
-                        combo->SetSelection( i_index );
-                }
-
-                if( p_item->psz_value )
-                    combo->SetValue( wxU(p_item->psz_value) );
-                combo->SetToolTip( wxU(p_item->psz_longtext) );
-                config_data->control.combobox = combo;
-                config_data->b_config_list = VLC_TRUE;
-                panel_sizer->Add( combo, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
-            }
-
-            if( p_item->i_type == CONFIG_ITEM_FILE )
-            {
-                button = new wxButton( panel, -1, wxU(_("Browse...")) );
-                panel_sizer->Add( button, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
-                button->SetClientData((void *)config_data);
-            }
-            if( p_item->b_advanced ) b_has_advanced = VLC_TRUE;
-            break;
-
-        case CONFIG_ITEM_INTEGER:
-            label = new wxStaticText(panel, -1, wxU(p_item->psz_text));
-            spin = new wxSpinCtrl( panel, -1,
-                                   wxString::Format(wxT("%d"),p_item->i_value),
-                                   wxDefaultPosition, wxDefaultSize,
-                                   wxSP_ARROW_KEYS,
-                                   -16000, 16000, p_item->i_value);
-            spin->SetToolTip( wxU(p_item->psz_longtext) );
-            config_data->control.spinctrl = spin;
-            panel_sizer->Add( label, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-            panel_sizer->Add( spin, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-
-            spin->SetClientData((void *)config_data);
-            if( p_item->b_advanced ) b_has_advanced = VLC_TRUE;
-            break;
-
-        case CONFIG_ITEM_FLOAT:
-            label = new wxStaticText(panel, -1, wxU(p_item->psz_text));
-            textctrl = new wxTextCtrl( panel, -1,
-                             wxString::Format(wxT("%f"),p_item->f_value),
-                             wxDefaultPosition, wxDefaultSize,
-                             wxTE_PROCESS_ENTER );
-            textctrl->SetToolTip( wxU(p_item->psz_longtext) );
-            config_data->control.textctrl = textctrl;
-            panel_sizer->Add( label, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
-            panel_sizer->Add( textctrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
-            if( p_item->b_advanced ) b_has_advanced = VLC_TRUE;
-            break;
-
-        case CONFIG_ITEM_BOOL:
-            checkbox = new wxCheckBox( panel, -1, wxU(p_item->psz_text) );
-            if( p_item->i_value ) checkbox->SetValue(TRUE);
-            checkbox->SetToolTip( wxU(p_item->psz_longtext) );
-            config_data->control.checkbox = checkbox;
-            panel_sizer->Add( checkbox, 0, wxALL, 5 );
-            if( p_item->b_advanced ) b_has_advanced = VLC_TRUE;
-            break;
-
-        default:
-            delete panel; panel = NULL;
-            delete panel_sizer;
-            delete config_data;
-            break;
+            sizer->Add( new wxStaticLine( this, 0 ), 0,
+                        wxEXPAND | wxLEFT | wxRIGHT, 2 );
+            help = new wxStaticText( this, -1, wxU(_(psz_help)),
+                                     wxDefaultPosition, wxDefaultSize,
+                                     wxALIGN_LEFT,
+                                     wxT("") );
+            sizer->Add( help ,0 ,wxEXPAND | wxALL, 5 );
         }
 
-        /* Don't add items that were not recognized */
-        if( panel == NULL ) continue;
-
-        panel_sizer->Layout();
-        panel->SetSizerAndFit( panel_sizer );
-
-        /* Add the config data to our array so we can keep a trace of it */
-        config_array.Add( config_data );
-
-        config_sizer->Add( panel, 0, wxEXPAND | wxALL, 2 );
     }
-    while( p_item->i_type != CONFIG_HINT_END && p_item++ );
-
-    /* Display a nice message if no configuration options are available */
-    if( !config_array.GetCount() )
-    {
-        config_sizer->Add( new wxStaticText( config_window, -1,
-                           wxU(_("No configuration options available")) ), 1,
-                           wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER, 2 );
-    }
-
-    config_sizer->Layout();
-    config_window->SetSizer( config_sizer );
-    sizer->Add( config_window, 1, wxEXPAND | wxALL, 5 );
-
-    /* Intercept all menu events in our custom event handler */
-    config_window->PushEventHandler(
-        new ConfigEvtHandler( p_intf, p_prefs_dialog ) );
-
-    /* Update panel */
-    wxCommandEvent dummy_event;
-    b_advanced = !config_GetInt( p_intf, "advanced" );
-    OnAdvanced( dummy_event );
-
-    /* Create advanced checkbox if needed */
-    if( config_array.GetCount() && b_has_advanced )
-    {
-        wxCheckBox *advanced_checkbox =
-           new wxCheckBox( this, Advanced_Event, wxU(_("Advanced options")) );
-
-        if( b_advanced ) advanced_checkbox->SetValue(TRUE);
-        sizer->Add( advanced_checkbox, 0, wxALL|wxALIGN_RIGHT, 0 );
-    }
-
     sizer->Layout();
     SetSizer( sizer );
 }
 
 void PrefsPanel::ApplyChanges()
 {
+    vlc_value_t val;
+
     for( size_t i = 0; i < config_array.GetCount(); i++ )
     {
-        ConfigData *config_data = config_array.Item(i);
+        ConfigControl *control = config_array.Item(i);
 
-        switch( config_data->i_config_type )
+        switch( control->GetType() )
         {
-        case CONFIG_ITEM_MODULE:
-            config_PutPsz( p_intf, config_data->option_name.mb_str(), (char *)
-                           config_data->control.combobox->GetClientData(
-                           config_data->control.combobox->GetSelection() ) );
-            break;
         case CONFIG_ITEM_STRING:
         case CONFIG_ITEM_FILE:
         case CONFIG_ITEM_DIRECTORY:
-            if( !config_data->b_config_list )
-                config_PutPsz( p_intf, config_data->option_name.mb_str(),
-                               config_data->control.textctrl->GetValue().mb_str() );
-            else
-                config_PutPsz( p_intf, config_data->option_name.mb_str(),
-                               config_data->control.combobox->GetValue().mb_str() );
-            break;
-        case CONFIG_ITEM_BOOL:
-            config_PutInt( p_intf, config_data->option_name.mb_str(),
-                           config_data->control.checkbox->IsChecked() );
+        case CONFIG_ITEM_MODULE:
+            config_PutPsz( p_intf, control->GetName().mb_str(),
+                           control->GetPszValue().mb_str() );
             break;
+        case CONFIG_ITEM_KEY:
+            /* So you don't need to restart to have the changes take effect */
+            val.i_int = control->GetIntValue();
+            var_Set( p_intf->p_vlc, control->GetName().mb_str(), val );
         case CONFIG_ITEM_INTEGER:
-            config_PutInt( p_intf, config_data->option_name.mb_str(),
-                           config_data->control.spinctrl->GetValue() );
+        case CONFIG_ITEM_BOOL:
+            config_PutInt( p_intf, control->GetName().mb_str(),
+                           control->GetIntValue() );
             break;
         case CONFIG_ITEM_FLOAT:
-            {
-                float f_value;
-                if( (wxSscanf(config_data->control.textctrl->GetValue(),
-                              wxT("%f"), &f_value) == 1) )
-                    config_PutFloat( p_intf, config_data->option_name.mb_str(),
-                                     f_value );
-            }
+            config_PutFloat( p_intf, control->GetName().mb_str(),
+                             control->GetFloatValue() );
             break;
         }
     }
 }
 
-void PrefsPanel::OnAdvanced( wxCommandEvent& WXUNUSED(event) )
+void PrefsPanel::SwitchAdvanced( vlc_bool_t b_new_advanced )
 {
-    b_advanced = !b_advanced;
+    if( b_advanced == b_new_advanced ) return;
 
-    for( size_t i = 0; i < config_array.GetCount(); i++ )
+    if( config_sizer && config_window )
     {
-        ConfigData *config_data = config_array.Item(i);
-        if( config_data->b_advanced )
+        b_advanced = b_new_advanced;
+
+        for( size_t i = 0; i < config_array.GetCount(); i++ )
         {
-            config_data->panel->Show( b_advanced );
-            config_sizer->Show( config_data->panel, b_advanced );
+            ConfigControl *control = config_array.Item(i);
+            if( control->IsAdvanced() )
+            {
+                control->Show( b_advanced );
+                config_sizer->Show( control, b_advanced );
+            }
         }
-    }
 
-    config_sizer->Layout();
-    config_window->FitInside();
-    config_window->Refresh();
-}
-
-/*****************************************************************************
- * A small helper class which intercepts all events
- *****************************************************************************/
-ConfigEvtHandler::ConfigEvtHandler( intf_thread_t *_p_intf,
-                                    PrefsDialog *_p_prefs_dialog )
-{
-    /* Initializations */
-    p_intf = _p_intf;
-    p_prefs_dialog = _p_prefs_dialog;
-}
-
-ConfigEvtHandler::~ConfigEvtHandler()
-{
-}
-
-void ConfigEvtHandler::OnCommandEvent( wxCommandEvent& event )
-{
-    if( !event.GetEventObject() )
-    {
-        event.Skip();
-        return;
+        config_sizer->Layout();
+        config_window->FitInside();
+        config_window->Refresh();
     }
-
-    ConfigData *config_data = (ConfigData *)
-        ((wxEvtHandler *)event.GetEventObject())->GetClientData();
-
-    if( !config_data )
-    {
-        event.Skip();
-        return;
-    }
-
-    if( config_data->i_config_type == CONFIG_ITEM_FILE )
-    {
-        wxFileDialog dialog( p_prefs_dialog, wxU(_("Open file")),
-                             wxT(""), wxT(""), wxT("*.*"),
-#if defined( __WXMSW__ )
-                             wxOPEN );
-#else
-                             wxOPEN | wxSAVE );
-#endif
-
-        if( dialog.ShowModal() == wxID_OK )
-        {
-            config_data->control.textctrl->SetValue( dialog.GetPath() );      
-        }
-    }
-
-    switch( config_data->i_config_type )
-    {
-    case CONFIG_ITEM_MODULE:
-        break;
-    case CONFIG_ITEM_STRING:
-        break;
-    case CONFIG_ITEM_FILE:
-        break;
-    case CONFIG_ITEM_INTEGER:
-        break;
-    case CONFIG_ITEM_FLOAT:
-        break;
-    case CONFIG_ITEM_BOOL:
-        break;
-    }
-
-    event.Skip();
+    return;
 }
