@@ -1,8 +1,8 @@
 /*****************************************************************************
  * timer.cpp : wxWindows plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000-2001 VideoLAN
- * $Id: timer.cpp,v 1.24 2003/06/13 15:54:42 gbazin Exp $
+ * Copyright (C) 2000-2003 VideoLAN
+ * $Id: timer.cpp,v 1.36 2003/12/03 13:27:51 rocky Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -31,28 +31,16 @@
 
 #include <vlc/vlc.h>
 #include <vlc/aout.h>
-
-#ifdef WIN32                                                 /* mingw32 hack */
-#undef Yield
-#undef CreateDialog
-#endif
-
-/* Let vlc take care of the i18n stuff */
-#define WXINTL_NO_GETTEXT_MACRO
-
-#include <wx/wxprec.h>
-#include <wx/wx.h>
-#include <wx/timer.h>
-
 #include <vlc/intf.h>
 
 #include "wxwindows.h"
+#include <wx/timer.h>
 
-void DisplayStreamDate( wxControl *, intf_thread_t *, int );
+//void DisplayStreamDate( wxControl *, intf_thread_t *, int );
 
 /* Callback prototype */
-int PopupMenuCB( vlc_object_t *p_this, const char *psz_variable,
-                 vlc_value_t old_val, vlc_value_t new_val, void *param );
+static int PopupMenuCB( vlc_object_t *p_this, const char *psz_variable,
+                        vlc_value_t old_val, vlc_value_t new_val, void *param );
 
 /*****************************************************************************
  * Constructor.
@@ -79,6 +67,15 @@ Timer::Timer( intf_thread_t *_p_intf, Interface *_p_main_interface )
 
 Timer::~Timer()
 {
+    /* Unregister callback */
+    playlist_t *p_playlist =
+        (playlist_t *)vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                       FIND_ANYWHERE );
+    if( p_playlist != NULL )
+    {
+        var_DelCallback( p_playlist, "intf-popupmenu", PopupMenuCB, p_intf );
+        vlc_object_release( p_playlist );
+    }
 }
 
 /*****************************************************************************
@@ -97,35 +94,12 @@ void Timer::Notify()
 
     vlc_mutex_lock( &p_intf->change_lock );
 
-    /* If the "display popup" flag has changed */
-    if( p_intf->p_sys->b_popup_change )
-    {
-        wxPoint mousepos = wxGetMousePosition();
-
-        wxMouseEvent event = wxMouseEvent( wxEVT_RIGHT_UP );
-        event.m_x = p_main_interface->ScreenToClient(mousepos).x;
-        event.m_y = p_main_interface->ScreenToClient(mousepos).y;
-
-        p_main_interface->AddPendingEvent(event);
-
-        p_intf->p_sys->b_popup_change = VLC_FALSE;
-    }
-
-    /* Update the log window */
-    p_intf->p_sys->p_messages_window->UpdateLog();
-
-    /* Update the playlist */
-    p_intf->p_sys->p_playlist_window->UpdatePlaylist();
-
-    /* Update the fileinfo windows */
-    p_intf->p_sys->p_fileinfo_window->UpdateFileInfo();
-
     /* Update the input */
     if( p_intf->p_sys->p_input == NULL )
     {
-        p_intf->p_sys->p_input = (input_thread_t *)vlc_object_find( p_intf,
-                                                       VLC_OBJECT_INPUT,
-                                                       FIND_ANYWHERE );
+        p_intf->p_sys->p_input =
+            (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                               FIND_ANYWHERE );
 
         /* Show slider */
         if( p_intf->p_sys->p_input )
@@ -148,17 +122,15 @@ void Timer::Notify()
             /* Take care of the volume */
             audio_volume_t i_volume;
             aout_VolumeGet( p_intf, &i_volume );
-            p_main_interface->volctrl->SetValue( i_volume * 200 /
+            p_main_interface->volctrl->SetValue( i_volume * 200 * 2 /
                                                  AOUT_VOLUME_MAX );
             p_main_interface->volctrl->SetToolTip(
                 wxString::Format((wxString)wxU(_("Volume")) + wxT(" %d"),
                 i_volume * 200 / AOUT_VOLUME_MAX ) );
 
+            /* control buttons for free pace streams */
+            b_pace_control = p_intf->p_sys->p_input->stream.b_pace_control;
         }
-
-        /* control buttons for free pace streams */
-        b_pace_control = p_intf->p_sys->p_input->stream.b_pace_control;
-
     }
     else if( p_intf->p_sys->p_input->b_dead )
     {
@@ -208,25 +180,30 @@ void Timer::Notify()
             /* Manage the slider */
             if( p_input->stream.b_seekable && p_intf->p_sys->b_playing )
             {
-                stream_position_t position;
-
                 /* Update the slider if the user isn't dragging it. */
                 if( p_intf->p_sys->b_slider_free )
                 {
+                    vlc_value_t pos;
+                    char psz_time[ MSTRTIME_MAX_SIZE ];
+                    vlc_value_t time;
+                    mtime_t i_seconds;
+
                     /* Update the value */
-                    vlc_mutex_unlock( &p_input->stream.stream_lock );
-                    input_Tell( p_input, &position );
-                    vlc_mutex_lock( &p_input->stream.stream_lock );
-                    if( position.i_size )
+                    var_Get( p_input, "position", &pos );
+                    if( pos.f_float >= 0.0 )
                     {
                         p_intf->p_sys->i_slider_pos =
-                        ( SLIDER_MAX_POS * position.i_tell ) / position.i_size;
+                            (int)(SLIDER_MAX_POS * pos.f_float);
 
                         p_main_interface->slider->SetValue(
                             p_intf->p_sys->i_slider_pos );
 
-                        DisplayStreamDate( p_main_interface->slider_box,p_intf,
-                                           p_intf->p_sys->i_slider_pos );
+                        var_Get( p_intf->p_sys->p_input, "time", &time );
+                        i_seconds = time.i_time / 1000000;
+
+                        secstotimestr ( psz_time, i_seconds );
+
+                        p_main_interface->slider_box->SetLabel( wxU(psz_time) );
                     }
                 }
             }
@@ -276,39 +253,21 @@ void Timer::Notify()
     vlc_mutex_unlock( &p_intf->change_lock );
 }
 
-/*****************************************************************************
- * DisplayStreamDate: display stream date
- *****************************************************************************
- * This function displays the current date related to the position in
- * the stream. It is called whenever the slider changes its value.
- * The lock has to be taken before you call the function.
- *****************************************************************************/
-void DisplayStreamDate( wxControl *p_slider_frame, intf_thread_t * p_intf ,
-                        int i_pos )
-{
-    if( p_intf->p_sys->p_input )
-    {
-#define p_area p_intf->p_sys->p_input->stream.p_selected_area
-        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
-
-        p_slider_frame->SetLabel(
-            wxU(input_OffsetToTime( p_intf->p_sys->p_input,
-                    psz_time, p_area->i_size * i_pos / SLIDER_MAX_POS )) );
-#undef p_area
-     }
-}
-
 /*****************************************************************************
  * PopupMenuCB: callback triggered by the intf-popupmenu playlist variable.
  *  We don't show the menu directly here because we don't want the
  *  caller to block for a too long time.
  *****************************************************************************/
-int PopupMenuCB( vlc_object_t *p_this, const char *psz_variable,
-                 vlc_value_t old_val, vlc_value_t new_val, void *param )
+static int PopupMenuCB( vlc_object_t *p_this, const char *psz_variable,
+                        vlc_value_t old_val, vlc_value_t new_val, void *param )
 {
     intf_thread_t *p_intf = (intf_thread_t *)param;
 
-    p_intf->p_sys->b_popup_change = VLC_TRUE;
+    if( p_intf->p_sys->pf_show_dialog )
+    {
+        p_intf->p_sys->pf_show_dialog( p_intf, INTF_DIALOG_POPUPMENU,
+                                       new_val.b_bool, 0 );
+    }
 
     return VLC_SUCCESS;
 }
