@@ -1,8 +1,8 @@
 /*****************************************************************************
- * mp4.c
+ * mp4.c: mp4/mov muxer
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2003 VideoLAN
- * $Id: mp4.c,v 1.3 2003/08/01 20:06:43 gbazin Exp $
+ * $Id: mp4.c,v 1.7 2003/11/21 15:32:08 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -73,7 +73,7 @@ typedef struct
 
 typedef struct
 {
-    sout_format_t *p_fmt;
+    es_format_t   *p_fmt;
     int           i_track_id;
 
     /* index */
@@ -92,6 +92,8 @@ struct sout_mux_sys_t
     uint64_t i_mdat_pos;
     uint64_t i_pos;
 
+    mtime_t  i_start_dts;
+
     int          i_nb_streams;
     mp4_stream_t **pp_streams;
 };
@@ -117,6 +119,7 @@ static void bo_add_64be ( bo_t *, uint64_t );
 static void bo_add_fourcc(bo_t *, char * );
 static void bo_add_bo   ( bo_t *, bo_t * );
 static void bo_add_mem  ( bo_t *, int , uint8_t * );
+static void bo_add_descr ( bo_t *, uint8_t , uint32_t );
 
 static void bo_fix_32be ( bo_t *, int , uint32_t );
 
@@ -147,6 +150,7 @@ static int Open( vlc_object_t *p_this )
     p_sys->pp_streams   = NULL;
     p_sys->i_mdat_pos   = 0;
     p_sys->b_mov        = p_mux->psz_mux && !strcmp( p_mux->psz_mux, "mov" );
+    p_sys->i_start_dts  = 0;
 
     msg_Info( p_mux, "Open" );
 
@@ -169,6 +173,10 @@ static int Open( vlc_object_t *p_this )
         p_sys->i_mdat_pos = p_sys->i_pos;
 
         box_send( p_mux, box );
+
+        /* FIXME FIXME
+         * Quicktime actually doesn't like the 64 bits extensions !!! */
+        p_sys->b_mov = VLC_TRUE;
     }
 
     /* Now add mdat header */
@@ -182,15 +190,16 @@ static int Open( vlc_object_t *p_this )
     return VLC_SUCCESS;
 }
 
-static uint32_t GetDescriptorLength24b( int i_length )
+static int GetDescrLength( int i_size )
 {
-    uint32_t    i_l1, i_l2, i_l3;
-
-    i_l1 = i_length&0x7f;
-    i_l2 = ( i_length >> 7 )&0x7f;
-    i_l3 = ( i_length >> 14 )&0x7f;
-
-    return( 0x808000 | ( i_l3 << 16 ) | ( i_l2 << 8 ) | i_l1 );
+    if( i_size < 0x00000080 )
+        return 2 + i_size;
+    else if( i_size < 0x00004000 )
+        return 3 + i_size;
+    else if( i_size < 0x00200000 )
+        return 4 + i_size;
+    else
+        return 5 + i_size;
 }
 
 static bo_t *GetESDS( mp4_stream_t *p_stream )
@@ -200,9 +209,10 @@ static bo_t *GetESDS( mp4_stream_t *p_stream )
     int  i_object_type_indication;
     int  i_decoder_specific_info_size;
 
-    if( p_stream->p_fmt->i_extra_data > 0 )
+    if( p_stream->p_fmt->i_extra > 0 )
     {
-        i_decoder_specific_info_size = p_stream->p_fmt->i_extra_data + 4;
+        i_decoder_specific_info_size =
+            GetDescrLength( p_stream->p_fmt->i_extra );
     }
     else
     {
@@ -211,17 +221,17 @@ static bo_t *GetESDS( mp4_stream_t *p_stream )
 
     esds = box_full_new( "esds", 0, 0 );
 
-    bo_add_8   ( esds, 0x03 );      // ES_DescrTag
-    bo_add_24be( esds,
-                 GetDescriptorLength24b( 25 + i_decoder_specific_info_size ) );
+    /* ES_Descr */
+    bo_add_descr( esds, 0x03, 3 +
+                  GetDescrLength( 13 + i_decoder_specific_info_size ) +
+                  GetDescrLength( 1 ) );
     bo_add_16be( esds, p_stream->i_track_id );
     bo_add_8   ( esds, 0x1f );      // flags=0|streamPriority=0x1f
 
-    bo_add_8   ( esds, 0x04 );      // DecoderConfigDescrTag
-    bo_add_24be( esds,
-                 GetDescriptorLength24b( 13 + i_decoder_specific_info_size ) );
+    /* DecoderConfigDescr */
+    bo_add_descr( esds, 0x04, 13 + i_decoder_specific_info_size );
 
-    switch( p_stream->p_fmt->i_fourcc )
+    switch( p_stream->p_fmt->i_codec )
     {
         case VLC_FOURCC( 'm', 'p', '4', 'v' ):
             i_object_type_indication = 0x20;
@@ -236,7 +246,7 @@ static bo_t *GetESDS( mp4_stream_t *p_stream )
             break;
         case VLC_FOURCC( 'm', 'p', 'g', 'a' ):
             i_object_type_indication =
-                p_stream->p_fmt->i_sample_rate < 32000 ? 0x69 : 0x6b;
+                p_stream->p_fmt->audio.i_rate < 32000 ? 0x69 : 0x6b;
             break;
         default:
             i_object_type_indication = 0x00;
@@ -250,23 +260,22 @@ static bo_t *GetESDS( mp4_stream_t *p_stream )
     bo_add_32be( esds, 0x7fffffff );        // maxBitrate
     bo_add_32be( esds, 0 );                 // avgBitrate
 
-    if( p_stream->p_fmt->i_extra_data > 0 )
+    if( p_stream->p_fmt->i_extra > 0 )
     {
         int i;
-        bo_add_8   ( esds, 0x05 );  // DecoderSpecificInfo
-        bo_add_24be( esds,
-                     GetDescriptorLength24b( p_stream->p_fmt->i_extra_data ) );
 
-        for( i = 0; i < p_stream->p_fmt->i_extra_data; i++ )
+        /* DecoderSpecificInfo */
+        bo_add_descr( esds, 0x05, p_stream->p_fmt->i_extra );
+
+        for( i = 0; i < p_stream->p_fmt->i_extra; i++ )
         {
-            bo_add_8( esds, p_stream->p_fmt->p_extra_data[i] );
+            bo_add_8( esds, ((uint8_t*)p_stream->p_fmt->p_extra)[i] );
         }
     }
 
     /* SL_Descr mandatory */
-    bo_add_8   ( esds, 0x06 );  // SLConfigDescriptorTag
-    bo_add_24be( esds, GetDescriptorLength24b( 1 ) );
-    bo_add_8   ( esds, 0x02 );  // sl_predefined
+    bo_add_descr( esds, 0x06, 1 );
+    bo_add_8    ( esds, 0x02 );  // sl_predefined
 
     box_fix( esds );
 
@@ -418,7 +427,7 @@ static void Close( vlc_object_t * p_this )
         }
         if( p_stream->p_fmt->i_cat == AUDIO_ES )
         {
-            i_timescale = p_stream->p_fmt->i_sample_rate;
+            i_timescale = p_stream->p_fmt->audio.i_rate;
         }
         else
         {
@@ -471,8 +480,8 @@ static void Close( vlc_object_t * p_this )
         }
         else
         {
-            bo_add_32be( tkhd, p_stream->p_fmt->i_width  << 16 );     // width (presentation)
-            bo_add_32be( tkhd, p_stream->p_fmt->i_height << 16 );     // height(presentation)
+            bo_add_32be( tkhd, p_stream->p_fmt->video.i_width  << 16 );     // width (presentation)
+            bo_add_32be( tkhd, p_stream->p_fmt->video.i_height << 16 );     // height(presentation)
         }
         box_fix( tkhd );
         box_gather( trak, tkhd );
@@ -480,7 +489,7 @@ static void Close( vlc_object_t * p_this )
         /* *** add /moov/trak/mdia *** */
         mdia = box_new( "mdia" );
 
-        /* */
+        /* media header */
         if( p_sys->b_mov )
         {
             mdhd = box_full_new( "mdhd", 0, 0 );
@@ -505,9 +514,10 @@ static void Close( vlc_object_t * p_this )
         box_fix( mdhd );
         box_gather( mdia, mdhd );
 
-        /* */
+        /* handler reference */
         hdlr = box_full_new( "hdlr", 0, 0 );
-        bo_add_32be( hdlr, 0 );         // predefined
+
+        bo_add_fourcc( hdlr, "mhlr" );         // media handler
         if( p_stream->p_fmt->i_cat == AUDIO_ES )
         {
             bo_add_fourcc( hdlr, "soun" );
@@ -516,11 +526,19 @@ static void Close( vlc_object_t * p_this )
         {
             bo_add_fourcc( hdlr, "vide" );
         }
-        for( i = 0; i < 3; i++ )
+
+        bo_add_32be( hdlr, 0 );         // reserved
+        bo_add_32be( hdlr, 0 );         // reserved
+        bo_add_32be( hdlr, 0 );         // reserved
+
+        if( p_stream->p_fmt->i_cat == AUDIO_ES )
         {
-            bo_add_32be( hdlr, 0 );     // reserved
+            bo_add_mem( hdlr, 13, "SoundHandler" );
+        }
+        else
+        {
+            bo_add_mem( hdlr, 13, "VideoHandler" );
         }
-        bo_add_mem( hdlr, 2, "?" );
 
         box_fix( hdlr );
         box_gather( mdia, hdlr );
@@ -581,7 +599,7 @@ static void Close( vlc_object_t * p_this )
             int  i;
             vlc_bool_t b_mpeg4_hdr;
 
-            switch( p_stream->p_fmt->i_fourcc )
+            switch( p_stream->p_fmt->i_codec )
             {
                 case VLC_FOURCC( 'm', 'p', '4', 'a' ):
                     memcpy( fcc, "mp4a", 4 );
@@ -597,7 +615,7 @@ static void Close( vlc_object_t * p_this )
                     break;
 
                 default:
-                    memcpy( fcc, (char*)&p_stream->p_fmt->i_fourcc, 4 );
+                    memcpy( fcc, (char*)&p_stream->p_fmt->i_codec, 4 );
                     b_mpeg4_hdr = VLC_FALSE;
                     break;
             }
@@ -608,13 +626,16 @@ static void Close( vlc_object_t * p_this )
                 bo_add_8( soun, 0 );        // reserved;
             }
             bo_add_16be( soun, 1 );         // data-reference-index
-            bo_add_32be( soun, 0 );         // reserved;
-            bo_add_32be( soun, 0 );         // reserved;
-            bo_add_16be( soun, p_stream->p_fmt->i_channels );   // channel-count
+
+            /* SoundDescription */
+            bo_add_16be( soun, 0 );         // version;
+            bo_add_16be( soun, 0 );         // revision level (0)
+            bo_add_32be( soun, 0 );         // vendor
+            bo_add_16be( soun, p_stream->p_fmt->audio.i_channels );   // channel-count
             bo_add_16be( soun, 16);         // FIXME sample size
-            bo_add_16be( soun, 0 );         // predefined
-            bo_add_16be( soun, 0 );         // reserved
-            bo_add_16be( soun, p_stream->p_fmt->i_sample_rate ); // sampleratehi
+            bo_add_16be( soun, -2 );        // compression id
+            bo_add_16be( soun, 0 );         // packet size (0)
+            bo_add_16be( soun, p_stream->p_fmt->audio.i_rate ); // sampleratehi
             bo_add_16be( soun, 0 );                              // sampleratelo
 
             /* add an ES Descriptor */
@@ -638,7 +659,7 @@ static void Close( vlc_object_t * p_this )
             int  i;
             vlc_bool_t b_mpeg4_hdr;
 
-            switch( p_stream->p_fmt->i_fourcc )
+            switch( p_stream->p_fmt->i_codec )
             {
                 case VLC_FOURCC( 'm', 'p', '4', 'v' ):
                 case VLC_FOURCC( 'm', 'p', 'g', 'v' ):
@@ -652,7 +673,7 @@ static void Close( vlc_object_t * p_this )
                     break;
 
                 default:
-                    memcpy( fcc, (char*)&p_stream->p_fmt->i_fourcc, 4 );
+                    memcpy( fcc, (char*)&p_stream->p_fmt->i_codec, 4 );
                     b_mpeg4_hdr = VLC_FALSE;
                     break;
             }
@@ -671,18 +692,21 @@ static void Close( vlc_object_t * p_this )
                 bo_add_32be( vide, 0 );     // predefined;
             }
 
-            bo_add_16be( vide, p_stream->p_fmt->i_width );  // i_width
-            bo_add_16be( vide, p_stream->p_fmt->i_height ); // i_height
+            bo_add_16be( vide, p_stream->p_fmt->video.i_width );  // i_width
+            bo_add_16be( vide, p_stream->p_fmt->video.i_height ); // i_height
 
             bo_add_32be( vide, 0x00480000 );                // h 72dpi
             bo_add_32be( vide, 0x00480000 );                // v 72dpi
 
-            bo_add_32be( vide, 0 );         // reserved
-            bo_add_16be( vide, 1 );         // predefined
+            bo_add_32be( vide, 0 );         // data size, always 0
+            bo_add_16be( vide, 1 );         // frames count per sample
+
+            // compressor name;
             for( i = 0; i < 32; i++ )
             {
-                bo_add_8( vide, 0 );        // compressor name;
+                bo_add_8( vide, 0 );
             }
+
             bo_add_16be( vide, 0x18 );      // depth
             bo_add_16be( vide, 0xffff );    // predefined
 
@@ -724,8 +748,10 @@ static void Close( vlc_object_t * p_this )
             i_chunk_count++;
         }
 
-//        if( p_sys->i_pos >= 0xffffffff )
+        /* chunk offset table */
+        if( p_sys->i_pos >= (((uint64_t)0x1) << 32) )
         {
+            /* 64 bits version */
             bo_t *co64;
             bo_t *stsc;
 
@@ -747,7 +773,9 @@ static void Close( vlc_object_t * p_this )
 
                 while( i < p_stream->i_entry_count )
                 {
-                    if( i + 1 < p_stream->i_entry_count && p_stream->entry[i].i_pos + p_stream->entry[i].i_size != p_stream->entry[i + 1].i_pos )
+                    if( i + 1 < p_stream->i_entry_count &&
+                        p_stream->entry[i].i_pos + p_stream->entry[i].i_size
+                        != p_stream->entry[i + 1].i_pos )
                     {
                         i++;
                         break;
@@ -767,10 +795,52 @@ static void Close( vlc_object_t * p_this )
             box_fix( stsc );
             box_gather( stbl, stsc );
         }
-//        else
-//        {
-//              FIXME implement it
-//        }
+        else
+        {
+            /* 32 bits version */
+            bo_t *stco;
+            bo_t *stsc;
+
+            unsigned int  i_chunk;
+
+            msg_Dbg( p_mux, "creating %d chunk (stco)", i_chunk_count );
+
+            stco = box_full_new( "stco", 0, 0 );
+            bo_add_32be( stco, i_chunk_count );
+
+            stsc = box_full_new( "stsc", 0, 0 );
+            bo_add_32be( stsc, i_chunk_count );     // entry-count
+            for( i_chunk = 0, i = 0; i < p_stream->i_entry_count; i_chunk++ )
+            {
+                int i_first;
+                bo_add_32be( stco, p_stream->entry[i].i_pos );
+
+                i_first = i;
+
+                while( i < p_stream->i_entry_count )
+                {
+                    if( i + 1 < p_stream->i_entry_count &&
+                        p_stream->entry[i].i_pos + p_stream->entry[i].i_size
+                        != p_stream->entry[i + 1].i_pos )
+                    {
+                        i++;
+                        break;
+                    }
+
+                    i++;
+                }
+                bo_add_32be( stsc, 1 + i_chunk );   // first-chunk
+                bo_add_32be( stsc, i - i_first ) ;  // samples-per-chunk
+                bo_add_32be( stsc, 1 );             // sample-descr-index
+            }
+            /* append stco to stbl */
+            box_fix( stco );
+            box_gather( stbl, stco );
+
+            /* append stsc to stbl */
+            box_fix( stsc );
+            box_gather( stbl, stsc );
+        }
 
 
         /* add stts */
@@ -846,9 +916,9 @@ static void Close( vlc_object_t * p_this )
 
         p_stream = p_sys->pp_streams[i_trak];
 
-        if( p_stream->p_fmt->p_extra_data )
+        if( p_stream->p_fmt->p_extra )
         {
-            free( p_stream->p_fmt->p_extra_data );
+            free( p_stream->p_fmt->p_extra );
         }
         free( p_stream->p_fmt );
         free( p_stream->entry );
@@ -875,7 +945,7 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     sout_mux_sys_t  *p_sys = p_mux->p_sys;
     mp4_stream_t    *p_stream;
 
-    switch( p_input->p_fmt->i_fourcc )
+    switch( p_input->p_fmt->i_codec )
     {
         case VLC_FOURCC( 'm', 'p', '4', 'a' ):
         case VLC_FOURCC( 'm', 'p', '4', 'v' ):
@@ -886,20 +956,20 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             break;
         default:
             msg_Err( p_mux, "unsupported codec %4.4s in mp4",
-                     (char*)&p_input->p_fmt->i_fourcc );
+                     (char*)&p_input->p_fmt->i_codec );
             return VLC_EGENERIC;
     }
 
     p_stream                = malloc( sizeof( mp4_stream_t ) );
-    p_stream->p_fmt         = malloc( sizeof( sout_format_t ) );
-    memcpy( p_stream->p_fmt, p_input->p_fmt, sizeof( sout_format_t ) );
-    if( p_stream->p_fmt->i_extra_data )
+    p_stream->p_fmt         = malloc( sizeof( es_format_t ) );
+    memcpy( p_stream->p_fmt, p_input->p_fmt, sizeof( es_format_t ) );
+    if( p_stream->p_fmt->i_extra )
     {
-        p_stream->p_fmt->p_extra_data =
-            malloc( p_stream->p_fmt->i_extra_data );
-        memcpy( p_stream->p_fmt->p_extra_data,
-                p_input->p_fmt->p_extra_data,
-                p_input->p_fmt->i_extra_data );
+        p_stream->p_fmt->p_extra =
+            malloc( p_stream->p_fmt->i_extra );
+        memcpy( p_stream->p_fmt->p_extra,
+                p_input->p_fmt->p_extra,
+                p_input->p_fmt->i_extra );
     }
     p_stream->i_track_id    = p_sys->i_nb_streams + 1;
     p_stream->i_entry_count = 0;
@@ -980,6 +1050,8 @@ static int Mux( sout_mux_t *p_mux )
             return( VLC_SUCCESS );
         }
 
+        if( !p_sys->i_start_dts )
+            p_sys->i_start_dts = i_dts;
 
         p_input  = p_mux->pp_inputs[i_stream];
         p_stream = (mp4_stream_t*)p_input->p_sys;
@@ -995,6 +1067,16 @@ static int Mux( sout_mux_t *p_mux )
         p_stream->entry[p_stream->i_entry_count].i_length=
             __MAX( p_data->i_length, 0 );
 
+        if( p_stream->i_entry_count == 0 )
+        {
+            /* Here is another bad hack.
+             * To make sure audio/video are in sync, we report a corrected
+             * length for the 1st sample. */
+            p_stream->entry[p_stream->i_entry_count].i_length =
+                __MAX( p_data->i_length, 0 ) +
+                p_data->i_pts - p_sys->i_start_dts;
+        }
+
         p_stream->i_entry_count++;
         if( p_stream->i_entry_count >= p_stream->i_entry_max )
         {
@@ -1102,6 +1184,47 @@ static void bo_add_mem( bo_t *p_bo, int i_size, uint8_t *p_mem )
         bo_add_8( p_bo, p_mem[i] );
     }
 }
+
+static void bo_add_descr( bo_t *p_bo, uint8_t tag, uint32_t i_size )
+{
+    uint32_t i_length;
+    uint8_t  vals[4];
+
+    i_length = i_size;
+    vals[3] = (unsigned char)(i_length & 0x7f);
+    i_length >>= 7;
+    vals[2] = (unsigned char)((i_length & 0x7f) | 0x80); 
+    i_length >>= 7;
+    vals[1] = (unsigned char)((i_length & 0x7f) | 0x80); 
+    i_length >>= 7;
+    vals[0] = (unsigned char)((i_length & 0x7f) | 0x80);
+
+    bo_add_8( p_bo, tag );
+
+    if( i_size < 0x00000080 )
+    {
+        bo_add_8( p_bo, vals[3] );
+    }
+    else if( i_size < 0x00004000 )
+    {
+        bo_add_8( p_bo, vals[2] );
+        bo_add_8( p_bo, vals[3] );
+    }
+    else if( i_size < 0x00200000 )
+    {
+        bo_add_8( p_bo, vals[1] );
+        bo_add_8( p_bo, vals[2] );
+        bo_add_8( p_bo, vals[3] );
+    }
+    else if( i_size < 0x10000000 )
+    {
+        bo_add_8( p_bo, vals[0] );
+        bo_add_8( p_bo, vals[1] );
+        bo_add_8( p_bo, vals[2] );
+        bo_add_8( p_bo, vals[3] );
+    }
+}
+
 static void bo_add_bo( bo_t *p_bo, bo_t *p_bo2 )
 {
     int i;
