@@ -2,7 +2,7 @@
  * clone.c : Clone video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2003 VideoLAN
- * $Id: clone.c,v 1.9 2003/03/30 18:14:38 gbazin Exp $
+ * $Id: clone.c,v 1.11 2003/10/15 22:49:48 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -52,11 +52,11 @@ static int  SendEvents( vlc_object_t *, char const *,
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define COUNT_TEXT N_("number of clones")
+#define COUNT_TEXT N_("Number of clones")
 #define COUNT_LONGTEXT N_("Select the number of video windows in which to "\
     "clone the video")
 
-#define VOUTLIST_TEXT N_("list of vout modules")
+#define VOUTLIST_TEXT N_("List of vout modules")
 #define VOUTLIST_LONGTEXT N_("Select the specific vout modules that you want to activate")
 
 vlc_module_begin();
@@ -243,6 +243,8 @@ static int Init( vout_thread_t *p_vout )
     if( psz_default_vout ) free( psz_default_vout );
     ALLOCATE_DIRECTBUFFERS( VOUT_MAX_PICTURES );
 
+    ADD_PARENT_CALLBACKS( SendEventsToChild );
+
     return VLC_SUCCESS;
 }
 
@@ -272,6 +274,8 @@ static void Destroy( vlc_object_t *p_this )
 
     RemoveAllVout( p_vout );
 
+    DEL_PARENT_CALLBACKS( SendEventsToChild );
+
     free( p_vout->p_sys->pp_vout );
     free( p_vout->p_sys );
 }
@@ -367,3 +371,22 @@ static int SendEvents( vlc_object_t *p_this, char const *psz_var,
 
     return VLC_SUCCESS;
 }
+
+/*****************************************************************************
+ * SendEventsToChild: forward events to the child/children vout
+ *****************************************************************************/
+static int SendEventsToChild( vlc_object_t *p_this, char const *psz_var,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    int i_vout;
+
+    for( i_vout = 0; i_vout < p_vout->p_sys->i_clones; i_vout++ )
+    {
+        var_Set( p_vout->p_sys->pp_vout[ i_vout ], psz_var, newval );
+
+        if( !strcmp( psz_var, "fullscreen" ) ) break;
+    }
+
+    return VLC_SUCCESS;
+}
