@@ -2,9 +2,9 @@
  * deinterlace.c : deinterlacer plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: deinterlace.c,v 1.11 2003/03/30 18:14:38 gbazin Exp $
+ * $Id: deinterlace.c,v 1.18 2003/12/22 14:32:56 sam Exp $
  *
- * Authors: Samuel Hocevar <sam@zoy.org>
+ * Author: Sam Hocevar <sam@zoy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,15 +72,18 @@ static int FilterCallback ( vlc_object_t *, char const *,
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define MODE_TEXT N_("deinterlace mode")
-#define MODE_LONGTEXT N_("you can choose the default deinterlace mode")
+#define MODE_TEXT N_("Deinterlace mode")
+#define MODE_LONGTEXT N_("You can choose the default deinterlace mode")
 
-static char *mode_list[] = { "discard", "blend", "mean", "bob", "linear", NULL };
+static char *mode_list[] = { "discard", "blend", "mean", "bob", "linear" };
+static char *mode_list_text[] = { N_("discard"), N_("Blend"), N_("Mean"),
+                                  N_("Bob"), N_("Linear") };
 
 vlc_module_begin();
     add_category_hint( N_("Deinterlace"), NULL, VLC_FALSE );
-    add_string_from_list( "deinterlace-mode", "discard", mode_list, NULL,
-                          MODE_TEXT, MODE_LONGTEXT, VLC_FALSE );
+    add_string( "deinterlace-mode", "discard", NULL, MODE_TEXT,
+                MODE_LONGTEXT, VLC_FALSE );
+        change_string_list( mode_list, mode_list_text, 0 );
     set_description( _("video deinterlacing filter") );
     set_capability( "video filter", 0 );
     add_shortcut( "deinterlace" );
@@ -136,20 +139,18 @@ static int Create( vlc_object_t *p_this )
     vlc_mutex_init( p_vout, &p_vout->p_sys->filter_lock );
 
     /* Look what method was requested */
-    val.psz_string = config_GetPsz( p_vout, "deinterlace-mode" );
-
     var_Create( p_vout, "deinterlace-mode", VLC_VAR_STRING );
+    var_Change( p_vout, "deinterlace-mode", VLC_VAR_INHERITVALUE, &val, NULL );
 
     if( val.psz_string == NULL )
     {
-        msg_Err( p_vout, "configuration variable %s empty",
-                         "deinterlace-mode" );
+        msg_Err( p_vout, "configuration variable deinterlace-mode empty" );
         msg_Err( p_vout, "no deinterlace mode provided, using \"discard\"" );
 
         val.psz_string = strdup( "discard" );
     }
 
-    var_Set( p_vout, "deinterlace-mode", val );
+    msg_Dbg( p_vout, "using %s deinterlace mode", val.psz_string );
 
     SetFilterMethod( p_vout, val.psz_string );
 
@@ -246,6 +247,8 @@ static int Init( vout_thread_t *p_vout )
 
     ADD_CALLBACKS( p_vout->p_sys->p_vout, SendEvents );
 
+    ADD_PARENT_CALLBACKS( SendEventsToChild );
+
     return VLC_SUCCESS;
 }
 
@@ -327,6 +330,8 @@ static void Destroy( vlc_object_t *p_this )
     vlc_object_detach( p_vout->p_sys->p_vout );
     vout_Destroy( p_vout->p_sys->p_vout );
 
+    DEL_PARENT_CALLBACKS( SendEventsToChild );
+
     free( p_vout->p_sys );
 }
 
@@ -371,7 +376,7 @@ static void Render ( vout_thread_t *p_vout, picture_t *p_pic )
                 vlc_mutex_unlock( &p_vout->p_sys->filter_lock );
                 return;
             }
-	    msleep( VOUT_OUTMEM_SLEEP );
+            msleep( VOUT_OUTMEM_SLEEP );
         }
 
         /* 20ms is a bit arbitrary, but it's only for the first image we get */
@@ -513,36 +518,101 @@ static void RenderBob( vout_thread_t *p_vout,
         p_out_end = p_out + p_outpic->p[i_plane].i_pitch
                              * p_outpic->p[i_plane].i_lines;
 
-        /* For BOTTOM field we need to add the first line */
-        if( i_field == 1 )
+        switch( p_vout->render.i_chroma )
         {
-            p_vout->p_vlc->pf_memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
-            p_in += p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
-        }
+            case VLC_FOURCC('I','4','2','0'):
+            case VLC_FOURCC('I','Y','U','V'):
+            case VLC_FOURCC('Y','V','1','2'):
+                /* For BOTTOM field we need to add the first line */
+                if( i_field == 1 )
+                {
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
+                    p_in += p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
+                }
 
-        p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
+                p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
 
-        for( ; p_out < p_out_end ; )
-        {
-            p_vout->p_vlc->pf_memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
+                for( ; p_out < p_out_end ; )
+                {
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
 
-            p_out += p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
 
-            p_vout->p_vlc->pf_memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
 
-            p_in += 2 * p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
-        }
+                    p_in += 2 * p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
+                }
 
-        p_vout->p_vlc->pf_memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
+                p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                          p_pic->p[i_plane].i_pitch );
 
-        /* For TOP field we need to add the last line */
-        if( i_field == 0 )
-        {
-            p_in += p_pic->p[i_plane].i_pitch;
-            p_out += p_pic->p[i_plane].i_pitch;
-            p_vout->p_vlc->pf_memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
+                /* For TOP field we need to add the last line */
+                if( i_field == 0 )
+                {
+                    p_in += p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
+                }
+                break;
+
+            case VLC_FOURCC('I','4','2','2'):
+                /* For BOTTOM field we need to add the first line */
+                if( i_field == 1 )
+                {
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
+                    p_in += p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
+                }
+
+                p_out_end -= 2 * p_outpic->p[i_plane].i_pitch;
+
+                if( i_plane == Y_PLANE )
+                {
+                    for( ; p_out < p_out_end ; )
+                    {
+                        p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                                  p_pic->p[i_plane].i_pitch );
+
+                        p_out += p_pic->p[i_plane].i_pitch;
+
+                        p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                                  p_pic->p[i_plane].i_pitch );
+
+                        p_in += 2 * p_pic->p[i_plane].i_pitch;
+                        p_out += p_pic->p[i_plane].i_pitch;
+                    }
+                }
+                else
+                {
+                    for( ; p_out < p_out_end ; )
+                    {
+                        p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                                  p_pic->p[i_plane].i_pitch );
+
+                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_in += 2 * p_pic->p[i_plane].i_pitch;
+                    }
+                }
+
+                p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                          p_pic->p[i_plane].i_pitch );
+
+                /* For TOP field we need to add the last line */
+                if( i_field == 0 )
+                {
+                    p_in += p_pic->p[i_plane].i_pitch;
+                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                              p_pic->p[i_plane].i_pitch );
+                }
+                break;
         }
     }
 }
@@ -648,19 +718,58 @@ static void RenderBlend( vout_thread_t *p_vout,
         p_out_end = p_out + p_outpic->p[i_plane].i_pitch
                              * p_outpic->p[i_plane].i_lines;
 
-        /* First line: simple copy */
-        p_vout->p_vlc->pf_memcpy( p_out, p_in,
-                                  p_pic->p[i_plane].i_pitch );
-        p_out += p_pic->p[i_plane].i_pitch;
-
-        /* Remaining lines: mean value */
-        for( ; p_out < p_out_end ; )
+        switch( p_vout->render.i_chroma )
         {
-            Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
-                   p_pic->p[i_plane].i_pitch );
+            case VLC_FOURCC('I','4','2','0'):
+            case VLC_FOURCC('I','Y','U','V'):
+            case VLC_FOURCC('Y','V','1','2'):
+                /* First line: simple copy */
+                p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                          p_pic->p[i_plane].i_pitch );
+                p_out += p_pic->p[i_plane].i_pitch;
 
-            p_out += p_pic->p[i_plane].i_pitch;
-            p_in += p_pic->p[i_plane].i_pitch;
+                /* Remaining lines: mean value */
+                for( ; p_out < p_out_end ; )
+                {
+                   Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
+                          p_pic->p[i_plane].i_pitch );
+
+                    p_out += p_pic->p[i_plane].i_pitch;
+                    p_in += p_pic->p[i_plane].i_pitch;
+                }
+                break;
+
+            case VLC_FOURCC('I','4','2','2'):
+                /* First line: simple copy */
+                p_vout->p_vlc->pf_memcpy( p_out, p_in,
+                                          p_pic->p[i_plane].i_pitch );
+                p_out += p_pic->p[i_plane].i_pitch;
+
+                /* Remaining lines: mean value */
+                if( i_plane == Y_PLANE )
+                {
+                    for( ; p_out < p_out_end ; )
+                    {
+                        Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
+                               p_pic->p[i_plane].i_pitch );
+
+                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_in += p_pic->p[i_plane].i_pitch;
+                    }
+                }
+
+                else
+                {
+                    for( ; p_out < p_out_end ; )
+                    {
+                        Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
+                               p_pic->p[i_plane].i_pitch );
+
+                        p_out += p_pic->p[i_plane].i_pitch;
+                        p_in += 2*p_pic->p[i_plane].i_pitch;
+                    }
+                }
+                break;
         }
     }
 }
@@ -728,6 +837,8 @@ static int FilterCallback( vlc_object_t *p_this, char const *psz_cmd,
     vout_thread_t * p_vout = (vout_thread_t *)p_this;
     int i_old_mode = p_vout->p_sys->i_mode;
 
+    msg_Dbg( p_vout, "using %s deinterlace mode", newval.psz_string );
+
     vlc_mutex_lock( &p_vout->p_sys->filter_lock );
 
     SetFilterMethod( p_vout, newval.psz_string );
@@ -796,3 +907,14 @@ static int FilterCallback( vlc_object_t *p_this, char const *psz_cmd,
     vlc_mutex_unlock( &p_vout->p_sys->filter_lock );
     return VLC_SUCCESS;
 }
+
+/*****************************************************************************
+ * SendEventsToChild: forward events to the child/children vout
+ *****************************************************************************/
+static int SendEventsToChild( vlc_object_t *p_this, char const *psz_var,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    var_Set( p_vout->p_sys->p_vout, psz_var, newval );
+    return VLC_SUCCESS;
+}
