@@ -2,7 +2,7 @@
  * wall.c : Wall video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: wall.c,v 1.7 2003/02/20 01:52:46 sigmunau Exp $
+ * $Id: wall.c,v 1.11 2003/10/15 22:49:48 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -50,15 +50,15 @@ static int  SendEvents( vlc_object_t *, char const *,
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define COLS_TEXT N_("number of columns")
+#define COLS_TEXT N_("Number of columns")
 #define COLS_LONGTEXT N_("Select the number of horizontal videowindows in " \
     "which to split the video")
 
-#define ROWS_TEXT N_("number of rows")
+#define ROWS_TEXT N_("Number of rows")
 #define ROWS_LONGTEXT N_("Select the number of vertical videowindows in " \
     "which to split the video")
 
-#define ACTIVE_TEXT N_("active windows")
+#define ACTIVE_TEXT N_("Active windows")
 #define ACTIVE_LONGTEXT N_("comma separated list of active windows, " \
     "defaults to all")
 
@@ -67,7 +67,7 @@ vlc_module_begin();
     add_integer( "wall-cols", 3, NULL, COLS_TEXT, COLS_LONGTEXT, VLC_FALSE );
     add_integer( "wall-rows", 3, NULL, ROWS_TEXT, ROWS_LONGTEXT, VLC_FALSE );
     add_string( "wall-active", NULL, NULL, ACTIVE_TEXT, ACTIVE_LONGTEXT, VLC_FALSE );
-    set_description( _("image wall video module") );
+    set_description( _("wall video filter") );
     set_capability( "video filter", 0 );
     add_shortcut( "wall" );
     set_callbacks( Create, Destroy );
@@ -273,6 +273,8 @@ static int Init( vout_thread_t *p_vout )
 
     ALLOCATE_DIRECTBUFFERS( VOUT_MAX_PICTURES );
 
+    ADD_PARENT_CALLBACKS( SendEventsToChild );
+
     return VLC_SUCCESS;
 }
 
@@ -302,6 +304,8 @@ static void Destroy( vlc_object_t *p_this )
 
     RemoveAllVout( p_vout );
 
+    DEL_PARENT_CALLBACKS( SendEventsToChild );
+
     free( p_vout->p_sys->pp_vout );
     free( p_vout->p_sys );
 }
@@ -423,6 +427,8 @@ static void RemoveAllVout( vout_thread_t *p_vout )
              DEL_CALLBACKS(
                  p_vout->p_sys->pp_vout[ p_vout->p_sys->i_vout ].p_vout,
                  SendEvents );
+             vlc_object_detach(
+                 p_vout->p_sys->pp_vout[ p_vout->p_sys->i_vout ].p_vout );
              vout_Destroy(
                  p_vout->p_sys->pp_vout[ p_vout->p_sys->i_vout ].p_vout );
          }
@@ -472,3 +478,24 @@ static int SendEvents( vlc_object_t *p_this, char const *psz_var,
     return VLC_SUCCESS;
 }
 
+/*****************************************************************************
+ * SendEventsToChild: forward events to the child/children vout
+ *****************************************************************************/
+static int SendEventsToChild( vlc_object_t *p_this, char const *psz_var,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    int i_row, i_col, i_vout = 0;
+
+    for( i_row = 0; i_row < p_vout->p_sys->i_row; i_row++ )
+    {
+        for( i_col = 0; i_col < p_vout->p_sys->i_col; i_col++ )
+        {
+            var_Set( p_vout->p_sys->pp_vout[ i_vout ].p_vout, psz_var, newval);
+            if( !strcmp( psz_var, "fullscreen" ) ) break;
+            i_vout++;
+        }
+    }
+
+    return VLC_SUCCESS;
+}
