@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout.c: Windows DirectX video output display method
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: directx.c,v 1.21 2003/05/25 11:31:54 gbazin Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: directx.c,v 1.31 2004/01/02 22:17:57 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -45,6 +45,13 @@
 #include <windows.h>
 #include <ddraw.h>
 
+#include <multimon.h>
+#undef GetSystemMetrics
+
+#ifndef MONITOR_DEFAULTTONEAREST
+#   define MONITOR_DEFAULTTONEAREST 2
+#endif
+
 #include "vout.h"
 
 /*****************************************************************************
@@ -84,6 +91,8 @@ static void DirectXGetDDrawCaps   ( vout_thread_t *p_vout );
 static int  DirectXLockSurface    ( vout_thread_t *p_vout, picture_t *p_pic );
 static int  DirectXUnlockSurface  ( vout_thread_t *p_vout, picture_t *p_pic );
 
+static DWORD DirectXFindColorkey( vout_thread_t *p_vout, uint32_t i_color );
+
 /* Object variables callbacks */
 static int OnTopCallback( vlc_object_t *, char const *,
                           vlc_value_t, vlc_value_t, void * );
@@ -91,8 +100,6 @@ static int OnTopCallback( vlc_object_t *, char const *,
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define ON_TOP_TEXT N_("Always on top")
-#define ON_TOP_LONGTEXT N_("Place the directx window on top of other windows")
 #define HW_YUV_TEXT N_("Use hardware YUV->RGB conversions")
 #define HW_YUV_LONGTEXT N_( \
     "Try to use hardware acceleration for YUV->RGB conversions. " \
@@ -110,7 +117,6 @@ static int OnTopCallback( vlc_object_t *, char const *,
 
 vlc_module_begin();
     add_category_hint( N_("Video"), NULL, VLC_FALSE );
-    add_bool( "directx-on-top", 0, NULL, ON_TOP_TEXT, ON_TOP_LONGTEXT, VLC_FALSE );
     add_bool( "directx-hw-yuv", 1, NULL, HW_YUV_TEXT, HW_YUV_LONGTEXT, VLC_TRUE );
     add_bool( "directx-use-sysmem", 0, NULL, SYSMEM_TEXT, SYSMEM_LONGTEXT, VLC_TRUE );
     add_bool( "directx-3buffering", 1, NULL, TRIPLEBUF_TEXT, TRIPLEBUF_LONGTEXT, VLC_TRUE );
@@ -137,6 +143,7 @@ static int OpenVideo( vlc_object_t *p_this )
 {
     vout_thread_t * p_vout = (vout_thread_t *)p_this;
     vlc_value_t val, text;
+    HMODULE huser32;
 
     /* Allocate structure */
     p_vout->p_sys = malloc( sizeof( vout_sys_t ) );
@@ -157,16 +164,27 @@ static int OpenVideo( vlc_object_t *p_this )
     p_vout->p_sys->p_display = NULL;
     p_vout->p_sys->p_current_surface = NULL;
     p_vout->p_sys->p_clipper = NULL;
-    p_vout->p_sys->hbrush = NULL;
     p_vout->p_sys->hwnd = NULL;
     p_vout->p_sys->hparent = NULL;
     p_vout->p_sys->i_changes = 0;
-    p_vout->p_sys->b_caps_overlay_clipping = 0;
-    SetRectEmpty( &p_vout->p_sys->rect_display );
-    p_vout->p_sys->b_using_overlay = config_GetInt( p_vout, "overlay" );
-    p_vout->p_sys->b_use_sysmem = config_GetInt( p_vout, "directx-use-sysmem");
-    p_vout->p_sys->b_hw_yuv = config_GetInt( p_vout, "directx-hw-yuv" );
-    p_vout->p_sys->b_3buf_overlay = config_GetInt( p_vout, "directx-3buffering" );
+
+    /* Multimonitor stuff */
+    p_vout->p_sys->hmonitor = NULL;
+    p_vout->p_sys->p_display_driver = NULL;
+    p_vout->p_sys->MonitorFromWindow = NULL;
+    p_vout->p_sys->GetMonitorInfo = NULL;
+    if( (huser32 = GetModuleHandle( "USER32" ) ) )
+    {
+        p_vout->p_sys->MonitorFromWindow =
+            GetProcAddress( huser32, "MonitorFromWindow" );
+        p_vout->p_sys->GetMonitorInfo =
+            GetProcAddress( huser32, "GetMonitorInfoA" );
+    }
+
+    var_Create( p_vout, "overlay", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "directx-use-sysmem", VLC_VAR_BOOL|VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "directx-hw-yuv", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "directx-3buffering", VLC_VAR_BOOL|VLC_VAR_DOINHERIT );
 
     p_vout->p_sys->b_cursor_hidden = 0;
     p_vout->p_sys->i_lastmoved = mdate();
@@ -186,9 +204,8 @@ static int OpenVideo( vlc_object_t *p_this )
     p_vout->p_sys->p_event =
         vlc_object_create( p_vout, sizeof(event_thread_t) );
     p_vout->p_sys->p_event->p_vout = p_vout;
-    if( vlc_thread_create( p_vout->p_sys->p_event,
-                           "DirectX Events Thread", DirectXEventThread,
-                           0, 1 ) )
+    if( vlc_thread_create( p_vout->p_sys->p_event, "DirectX Events Thread",
+                           DirectXEventThread, 0, 1 ) )
     {
         msg_Err( p_vout, "cannot create DirectXEventThread" );
         vlc_object_destroy( p_vout->p_sys->p_event );
@@ -221,13 +238,12 @@ static int OpenVideo( vlc_object_t *p_this )
     }
 
     /* Add a variable to indicate if the window should be on top of others */
-    var_Create( p_vout, "directx-on-top", VLC_VAR_BOOL );
+    var_Create( p_vout, "video-on-top", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     text.psz_string = _("Always on top");
-    var_Change( p_vout, "directx-on-top", VLC_VAR_SETTEXT, &text, NULL );
-    val.b_bool = config_GetInt( p_vout, "directx-on-top" );
-    var_Set( p_vout, "directx-on-top", val );
+    var_Change( p_vout, "video-on-top", VLC_VAR_SETTEXT, &text, NULL );
+    var_Get( p_vout, "video-on-top", &val );
     p_vout->p_sys->b_on_top_change = val.b_bool; 
-    var_AddCallback( p_vout, "directx-on-top", OnTopCallback, NULL );
+    var_AddCallback( p_vout, "video-on-top", OnTopCallback, NULL );
 
     return VLC_SUCCESS;
 
@@ -245,6 +261,35 @@ static int OpenVideo( vlc_object_t *p_this )
 static int Init( vout_thread_t *p_vout )
 {
     int i_chroma_backup;
+    vlc_value_t val;
+
+    /* Get a few default parameters */
+    var_Get( p_vout, "overlay", &val );
+    p_vout->p_sys->b_using_overlay = val.b_bool;
+    var_Get( p_vout, "directx-use-sysmem", &val );
+    p_vout->p_sys->b_use_sysmem = val.b_bool;
+    var_Get( p_vout, "directx-hw-yuv", &val );
+    p_vout->p_sys->b_hw_yuv = val.b_bool;
+    var_Get( p_vout, "directx-3buffering", &val );
+    p_vout->p_sys->b_3buf_overlay = val.b_bool;
+
+    /* Initialise DirectDraw if not already done.
+     * We do this here because on multi-monitor systems we may have to
+     * re-create the directdraw surfaces. */
+    if( !p_vout->p_sys->p_ddobject &&
+        DirectXInitDDraw( p_vout ) != VLC_SUCCESS )
+    {
+        msg_Err( p_vout, "cannot initialize DirectDraw" );
+        return VLC_EGENERIC;
+    }
+
+    /* Create the directx display */
+    if( !p_vout->p_sys->p_display &&
+        DirectXCreateDisplay( p_vout ) != VLC_SUCCESS )
+    {
+        msg_Err( p_vout, "cannot initialize DirectDraw" );
+        return VLC_EGENERIC;
+    }
 
     /* Initialize the output structure.
      * Since DirectDraw can do rescaling for us, stick to the default
@@ -307,16 +352,8 @@ static int Init( vout_thread_t *p_vout )
     }
 
     /* Change the window title bar text */
-    if( p_vout->p_sys->hparent )
-        ; /* Do nothing */
-    else if( p_vout->p_sys->b_using_overlay )
-        SetWindowText( p_vout->p_sys->hwnd,
-                       VOUT_TITLE " (hardware YUV overlay DirectX output)" );
-    else if( p_vout->p_sys->b_hw_yuv )
-        SetWindowText( p_vout->p_sys->hwnd,
-                       VOUT_TITLE " (hardware YUV DirectX output)" );
-    else SetWindowText( p_vout->p_sys->hwnd,
-                        VOUT_TITLE " (software RGB DirectX output)" );
+    if( p_vout->p_sys->hparent ) ; /* Do nothing */
+    else PostMessage( p_vout->p_sys->hwnd, WM_VLC_CHANGE_TEXT, 0, 0 );
 
     return VLC_SUCCESS;
 }
@@ -330,6 +367,10 @@ static int Init( vout_thread_t *p_vout )
 static void End( vout_thread_t *p_vout )
 {
     FreePictureVec( p_vout, p_vout->p_picture, I_OUTPUTPICTURES );
+
+    DirectXCloseDisplay( p_vout );
+    DirectXCloseDDraw( p_vout );
+
     return;
 }
 
@@ -344,10 +385,7 @@ static void CloseVideo( vlc_object_t *p_this )
 
     msg_Dbg( p_vout, "CloseVideo" );
 
-    var_Destroy( p_vout, "directs-on-top" );
-
-    DirectXCloseDisplay( p_vout );
-    DirectXCloseDDraw( p_vout );
+    var_Destroy( p_vout, "video-on-top" );
 
     if( p_vout->p_sys->p_event )
     {
@@ -391,41 +429,32 @@ static int Manage( vout_thread_t *p_vout )
         DirectXUpdateRects( p_vout, VLC_FALSE );
     }
 
+    /*
+     * Position Change
+     */
+    if( p_vout->p_sys->i_changes & DX_POSITION_CHANGE )
+    {
+        if( p_vout->p_sys->b_using_overlay )
+            DirectXUpdateOverlay( p_vout );
+
+        /* Check if we are still on the same monitor */
+        if( p_vout->p_sys->MonitorFromWindow &&
+            p_vout->p_sys->hmonitor !=
+                p_vout->p_sys->MonitorFromWindow( p_vout->p_sys->hwnd,
+                                                  MONITOR_DEFAULTTONEAREST ) )
+        {
+            /* This will force the vout core to recreate the picture buffers */
+            p_vout->i_changes |= VOUT_PICTURE_BUFFERS_CHANGE;
+        }
+
+        p_vout->p_sys->i_changes &= ~DX_POSITION_CHANGE;
+    }
+
     /* We used to call the Win32 PeekMessage function here to read the window
      * messages. But since window can stay blocked into this function for a
      * long time (for example when you move your window on the screen), I
      * decided to isolate PeekMessage in another thread. */
 
-    /*
-     * Scale Change
-     */
-    if( p_vout->i_changes & VOUT_SCALE_CHANGE
-         || p_vout->p_sys->i_changes & VOUT_SCALE_CHANGE )
-    {
-        msg_Dbg( p_vout, "scale change" );
-        if( !p_vout->p_sys->b_using_overlay )
-            InvalidateRect( p_vout->p_sys->hwnd, NULL, TRUE );
-        else
-            DirectXUpdateOverlay( p_vout );
-        p_vout->i_changes &= ~VOUT_SCALE_CHANGE;
-        p_vout->p_sys->i_changes &= ~VOUT_SCALE_CHANGE;
-    }
-
-    /*
-     * Size Change
-     */
-    if( p_vout->i_changes & VOUT_SIZE_CHANGE
-        || p_vout->p_sys->i_changes & VOUT_SIZE_CHANGE )
-    {
-        msg_Dbg( p_vout, "size change" );
-        if( !p_vout->p_sys->b_using_overlay )
-            InvalidateRect( p_vout->p_sys->hwnd, NULL, TRUE );
-        else
-            DirectXUpdateOverlay( p_vout );
-        p_vout->i_changes &= ~VOUT_SIZE_CHANGE;
-        p_vout->p_sys->i_changes &= ~VOUT_SIZE_CHANGE;
-    }
-
     /*
      * Fullscreen change
      */
@@ -444,7 +473,7 @@ static int Manage( vout_thread_t *p_vout )
             /* Maximized window */
             window_placement.showCmd = SW_SHOWMAXIMIZED;
             /* Change window style, no borders and no title bar */
-            SetWindowLong( p_vout->p_sys->hwnd, GWL_STYLE, 0 );
+            SetWindowLong( p_vout->p_sys->hwnd, GWL_STYLE, WS_CLIPCHILDREN );
 
         }
         else
@@ -452,7 +481,7 @@ static int Manage( vout_thread_t *p_vout )
             /* Normal window */
             window_placement.showCmd = SW_SHOWNORMAL;
             /* Change window style, borders and title bar */
-            SetWindowLong( p_vout->p_sys->hwnd, GWL_STYLE,
+            SetWindowLong( p_vout->p_sys->hwnd, GWL_STYLE, WS_CLIPCHILDREN |
                            WS_OVERLAPPEDWINDOW | WS_SIZEBOX | WS_VISIBLE );
         }
 
@@ -488,7 +517,7 @@ static int Manage( vout_thread_t *p_vout )
         vlc_value_t val;
         HMENU hMenu = GetSystemMenu( p_vout->p_sys->hwnd, FALSE );
 
-        var_Get( p_vout, "directx-on-top", &val );
+        var_Get( p_vout, "video-on-top", &val );
  
         /* Set the window on top if necessary */
         if( val.b_bool && !( GetWindowLong( p_vout->p_sys->hwnd, GWL_EXSTYLE )
@@ -599,9 +628,32 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
     }
 }
 
-
 /* following functions are local */
 
+/*****************************************************************************
+ * DirectXEnumCallback: Device enumeration
+ *****************************************************************************
+ * This callback function is called by DirectDraw once for each 
+ * available DirectDraw device.
+ *****************************************************************************/
+BOOL WINAPI DirectXEnumCallback( GUID* p_guid, LPTSTR psz_desc,
+                                 LPTSTR psz_drivername, VOID* p_context,
+                                 HMONITOR hmon )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_context;
+    msg_Dbg( p_vout, "DirectXEnumCallback: %s, %s", psz_desc, psz_drivername );
+
+    if( hmon && hmon == p_vout->p_sys->hmonitor )
+    {
+        msg_Dbg( p_vout, "selecting %s, %s", psz_desc, psz_drivername );
+        p_vout->p_sys->p_display_driver = malloc( sizeof(GUID) );
+        if( p_vout->p_sys->p_display_driver )
+            memcpy( p_vout->p_sys->p_display_driver, p_guid, sizeof(GUID) );
+    }
+
+    return TRUE; /* Keep enumerating */
+}
+
 /*****************************************************************************
  * DirectXInitDDraw: Takes care of all the DirectDraw initialisations
  *****************************************************************************
@@ -609,13 +661,15 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
  *****************************************************************************/
 static int DirectXInitDDraw( vout_thread_t *p_vout )
 {
-    HRESULT    dxresult;
-    HRESULT    (WINAPI *OurDirectDrawCreate)(GUID *,LPDIRECTDRAW *,IUnknown *);
-    LPDIRECTDRAW  p_ddobject;
+    HRESULT dxresult;
+    HRESULT (WINAPI *OurDirectDrawCreate)(GUID *,LPDIRECTDRAW *,IUnknown *);
+    HRESULT (WINAPI *OurDirectDrawEnumerateEx)( LPDDENUMCALLBACKEXA, LPVOID,
+                                                DWORD );
+    LPDIRECTDRAW p_ddobject;
 
     msg_Dbg( p_vout, "DirectXInitDDraw" );
 
-    /* load direct draw DLL */
+    /* Load direct draw DLL */
     p_vout->p_sys->hddraw_dll = LoadLibrary("DDRAW.DLL");
     if( p_vout->p_sys->hddraw_dll == NULL )
     {
@@ -625,14 +679,30 @@ static int DirectXInitDDraw( vout_thread_t *p_vout )
 
     OurDirectDrawCreate =
       (void *)GetProcAddress(p_vout->p_sys->hddraw_dll, "DirectDrawCreate");
-    if ( OurDirectDrawCreate == NULL )
+    if( OurDirectDrawCreate == NULL )
     {
         msg_Err( p_vout, "DirectXInitDDraw failed GetProcAddress" );
         goto error;
     }
 
+    OurDirectDrawEnumerateEx =
+      (void *)GetProcAddress( p_vout->p_sys->hddraw_dll,
+                              "DirectDrawEnumerateExA" );
+
+    if( OurDirectDrawEnumerateEx && p_vout->p_sys->MonitorFromWindow )
+    {
+        p_vout->p_sys->hmonitor =
+            p_vout->p_sys->MonitorFromWindow( p_vout->p_sys->hwnd,
+                                              MONITOR_DEFAULTTONEAREST );
+
+        /* Enumerate displays */
+        OurDirectDrawEnumerateEx( DirectXEnumCallback, p_vout, 
+                                  DDENUM_ATTACHEDSECONDARYDEVICES );
+    }
+
     /* Initialize DirectDraw now */
-    dxresult = OurDirectDrawCreate( NULL, &p_ddobject, NULL );
+    dxresult = OurDirectDrawCreate( p_vout->p_sys->p_display_driver,
+                                    &p_ddobject, NULL );
     if( dxresult != DD_OK )
     {
         msg_Err( p_vout, "DirectXInitDDraw cannot initialize DDraw" );
@@ -653,13 +723,36 @@ static int DirectXInitDDraw( vout_thread_t *p_vout )
     /* Set DirectDraw Cooperative level, ie what control we want over Windows
      * display */
     dxresult = IDirectDraw2_SetCooperativeLevel( p_vout->p_sys->p_ddobject,
-                                           p_vout->p_sys->hwnd, DDSCL_NORMAL );
+                                                 NULL, DDSCL_NORMAL );
     if( dxresult != DD_OK )
     {
         msg_Err( p_vout, "cannot set direct draw cooperative level" );
         goto error;
     }
 
+    /* Get the size of the current display device */
+    if( p_vout->p_sys->hmonitor && p_vout->p_sys->GetMonitorInfo )
+    {
+        MONITORINFO monitor_info;
+        monitor_info.cbSize = sizeof( MONITORINFO );
+        p_vout->p_sys->GetMonitorInfo( p_vout->p_sys->hmonitor,
+                                       &monitor_info );
+        p_vout->p_sys->rect_display = monitor_info.rcMonitor;
+    }
+    else
+    {
+        p_vout->p_sys->rect_display.left = 0;
+        p_vout->p_sys->rect_display.top = 0;
+        p_vout->p_sys->rect_display.right  = GetSystemMetrics(SM_CXSCREEN);
+        p_vout->p_sys->rect_display.bottom = GetSystemMetrics(SM_CYSCREEN);
+    }
+
+    msg_Dbg( p_vout, "screen dimensions (%ix%i,%ix%i)",
+             p_vout->p_sys->rect_display.left,
+             p_vout->p_sys->rect_display.top,
+             p_vout->p_sys->rect_display.right,
+             p_vout->p_sys->rect_display.bottom );
+
     /* Probe the capabilities of the hardware */
     DirectXGetDDrawCaps( p_vout );
 
@@ -687,7 +780,6 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
     HRESULT              dxresult;
     DDSURFACEDESC        ddsd;
     LPDIRECTDRAWSURFACE  p_display;
-    DDPIXELFORMAT   pixel_format;
 
     msg_Dbg( p_vout, "DirectXCreateDisplay" );
 
@@ -699,8 +791,7 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
     ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
 
     dxresult = IDirectDraw2_CreateSurface( p_vout->p_sys->p_ddobject,
-                                           &ddsd,
-                                           &p_display, NULL );
+                                           &ddsd, &p_display, NULL );
     if( dxresult != DD_OK )
     {
         msg_Err( p_vout, "cannot get primary surface (error %li)", dxresult );
@@ -722,27 +813,20 @@ static int DirectXCreateDisplay( vout_thread_t *p_vout )
     /* The clipper will be used only in non-overlay mode */
     DirectXCreateClipper( p_vout );
 
+    /* Make sure the colorkey will be painted */
+    p_vout->p_sys->i_colorkey = 1;
+    p_vout->p_sys->i_rgb_colorkey =
+        DirectXFindColorkey( p_vout, p_vout->p_sys->i_colorkey );
 
-#if 1
-    /* compute the colorkey pixel value from the RGB value we've got */
-    memset( &pixel_format, 0, sizeof( DDPIXELFORMAT ));
-    pixel_format.dwSize = sizeof( DDPIXELFORMAT );
-    dxresult = IDirectDrawSurface2_GetPixelFormat( p_vout->p_sys->p_display,
-                                                   &pixel_format );
-    if( dxresult != DD_OK )
-    {
-        msg_Warn( p_vout, "DirectXUpdateOverlay GetPixelFormat failed "
-                          "(error %li)", dxresult );
-    }
-    p_vout->p_sys->i_colorkey = (DWORD)((( p_vout->p_sys->i_rgb_colorkey
-                                           * pixel_format.dwRBitMask) / 255)
-                                        & pixel_format.dwRBitMask );
-#endif
+    /* Create the actual brush */
+    SetClassLong( p_vout->p_sys->hvideownd, GCL_HBRBACKGROUND,
+                  (LONG)CreateSolidBrush( p_vout->p_sys->i_rgb_colorkey ) );
+    InvalidateRect( p_vout->p_sys->hvideownd, NULL, TRUE );
+    DirectXUpdateRects( p_vout, VLC_TRUE );
 
     return VLC_SUCCESS;
 }
 
-
 /*****************************************************************************
  * DirectXCreateClipper: Create a clipper that will be used when blitting the
  *                       RGB surface to the main display.
@@ -767,8 +851,8 @@ static int DirectXCreateClipper( vout_thread_t *p_vout )
     }
 
     /* Associate the clipper to the window */
-    dxresult = IDirectDrawClipper_SetHWnd(p_vout->p_sys->p_clipper, 0,
-                                          p_vout->p_sys->hwnd);
+    dxresult = IDirectDrawClipper_SetHWnd( p_vout->p_sys->p_clipper, 0,
+                                           p_vout->p_sys->hvideownd );
     if( dxresult != DD_OK )
     {
         msg_Warn( p_vout, "cannot attach clipper to window (error %li)",
@@ -832,13 +916,9 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
         ddsd.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
         ddsd.ddpfPixelFormat.dwFlags = DDPF_FOURCC;
         ddsd.ddpfPixelFormat.dwFourCC = i_chroma;
-        ddsd.dwFlags = DDSD_CAPS |
-                       DDSD_HEIGHT |
-                       DDSD_WIDTH |
-                       DDSD_PIXELFORMAT;
+        ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH | DDSD_PIXELFORMAT;
         ddsd.dwFlags |= (i_backbuffers ? DDSD_BACKBUFFERCOUNT : 0);
-        ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY |
-                              DDSCAPS_VIDEOMEMORY;
+        ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY;
         ddsd.ddsCaps.dwCaps |= (i_backbuffers ? DDSCAPS_COMPLEX | DDSCAPS_FLIP
                                 : 0 );
         ddsd.dwHeight = p_vout->render.i_height;
@@ -846,8 +926,7 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
         ddsd.dwBackBufferCount = i_backbuffers;
 
         dxresult = IDirectDraw2_CreateSurface( p_vout->p_sys->p_ddobject,
-                                               &ddsd,
-                                               &p_surface, NULL );
+                                               &ddsd, &p_surface, NULL );
         if( dxresult != DD_OK )
         {
             *pp_surface_final = NULL;
@@ -867,9 +946,7 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
         memset( &ddsd, 0, sizeof( DDSURFACEDESC ) );
         ddsd.dwSize = sizeof(DDSURFACEDESC);
         ddsd.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
-        ddsd.dwFlags = DDSD_HEIGHT |
-                       DDSD_WIDTH |
-                       DDSD_CAPS;
+        ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH | DDSD_CAPS;
         ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
         ddsd.dwHeight = p_vout->render.i_height;
         ddsd.dwWidth = p_vout->render.i_width;
@@ -887,8 +964,7 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
         }
 
         dxresult = IDirectDraw2_CreateSurface( p_vout->p_sys->p_ddobject,
-                                               &ddsd,
-                                               &p_surface, NULL );
+                                               &ddsd, &p_surface, NULL );
         if( dxresult != DD_OK )
         {
             *pp_surface_final = NULL;
@@ -909,6 +985,20 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
         return VLC_EGENERIC;
     }
 
+    if( b_overlay )
+    {
+        /* Check the overlay is useable as some graphics cards allow creating
+         * several overlays but only one can be used at one time. */
+        p_vout->p_sys->p_current_surface = *pp_surface_final;
+        if( DirectXUpdateOverlay( p_vout ) != VLC_SUCCESS )
+        {
+            IDirectDrawSurface2_Release( *pp_surface_final );
+            *pp_surface_final = NULL;
+            msg_Err( p_vout, "overlay unuseable (might already be in use)" );
+            return VLC_EGENERIC;
+        }
+    }
+
     return VLC_SUCCESS;
 }
 
@@ -919,7 +1009,7 @@ static int DirectXCreateSurface( vout_thread_t *p_vout,
  * Ususally the overlay is moved by the user and thus, by a move or resize
  * event (in Manage).
  *****************************************************************************/
-void DirectXUpdateOverlay( vout_thread_t *p_vout )
+int DirectXUpdateOverlay( vout_thread_t *p_vout )
 {
     DDOVERLAYFX     ddofx;
     DWORD           dwFlags;
@@ -927,7 +1017,7 @@ void DirectXUpdateOverlay( vout_thread_t *p_vout )
 
     if( p_vout->p_sys->p_current_surface == NULL ||
         !p_vout->p_sys->b_using_overlay )
-        return;
+        return VLC_EGENERIC;
 
     /* The new window dimensions should already have been computed by the
      * caller of this function */
@@ -938,9 +1028,7 @@ void DirectXUpdateOverlay( vout_thread_t *p_vout )
     ddofx.dckDestColorkey.dwColorSpaceLowValue = p_vout->p_sys->i_colorkey;
     ddofx.dckDestColorkey.dwColorSpaceHighValue = p_vout->p_sys->i_colorkey;
 
-    dwFlags = DDOVER_SHOW;
-    if( !p_vout->p_sys->b_caps_overlay_clipping )
-        dwFlags |= DDOVER_KEYDESTOVERRIDE;
+    dwFlags = DDOVER_SHOW | DDOVER_KEYDESTOVERRIDE;
 
     dxresult = IDirectDrawSurface2_UpdateOverlay(
                                          p_vout->p_sys->p_current_surface,
@@ -952,7 +1040,10 @@ void DirectXUpdateOverlay( vout_thread_t *p_vout )
     {
         msg_Warn( p_vout,
                   "DirectXUpdateOverlay cannot move or resize overlay" );
+        return VLC_EGENERIC;
     }
+
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -974,6 +1065,14 @@ static void DirectXCloseDDraw( vout_thread_t *p_vout )
         FreeLibrary( p_vout->p_sys->hddraw_dll );
         p_vout->p_sys->hddraw_dll = NULL;
     }
+
+    if( p_vout->p_sys->p_display_driver != NULL )
+    {
+        free( p_vout->p_sys->p_display_driver );
+        p_vout->p_sys->p_display_driver = NULL;
+    }
+
+    p_vout->p_sys->hmonitor = NULL;
 }
 
 /*****************************************************************************
@@ -1130,11 +1229,11 @@ static int NewPictureVec( vout_thread_t *p_vout, picture_t *p_pic,
              * because a few buggy drivers don't mind creating the surface
              * even if they don't know about the chroma. */
             if( IDirectDraw2_GetFourCCCodes( p_vout->p_sys->p_ddobject,
-                                             &i_codes, NULL ) )
+                                             &i_codes, NULL ) == DD_OK )
             {
                 pi_codes = malloc( i_codes * sizeof(DWORD) );
                 if( pi_codes && IDirectDraw2_GetFourCCCodes(
-                    p_vout->p_sys->p_ddobject, &i_codes, pi_codes ) )
+                    p_vout->p_sys->p_ddobject, &i_codes, pi_codes ) == DD_OK )
                 {
                     for( i = 0; i < (int)i_codes; i++ )
                     {
@@ -1279,6 +1378,8 @@ static void FreePictureVec( vout_thread_t *p_vout, picture_t *p_pic,
             free( p_pic[i].p_sys );
         }
     }
+
+    p_vout->p_sys->p_current_surface = 0;
 }
 
 /*****************************************************************************
@@ -1309,6 +1410,8 @@ static int UpdatePictureStruct( vout_thread_t *p_vout, picture_t *p_pic,
                     p_pic->p->i_pixel_pitch = 2;
                     break;
                 case VLC_FOURCC('R','V','2','4'):
+                    p_pic->p->i_pixel_pitch = 3;
+                    break;
                 case VLC_FOURCC('R','V','3','2'):
                     p_pic->p->i_pixel_pitch = 4;
                     break;
@@ -1422,7 +1525,7 @@ static void DirectXGetDDrawCaps( vout_thread_t *p_vout )
     }
     else
     {
-        BOOL bHasOverlay, bHasOverlayFourCC, bCanClipOverlay,
+        BOOL bHasOverlay, bHasOverlayFourCC, bCanDeinterlace,
              bHasColorKey, bCanStretch, bCanBltFourcc;
 
         /* Determine if the hardware supports overlay surfaces */
@@ -1431,8 +1534,8 @@ static void DirectXGetDDrawCaps( vout_thread_t *p_vout )
         /* Determine if the hardware supports overlay surfaces */
         bHasOverlayFourCC = ((ddcaps.dwCaps & DDCAPS_OVERLAYFOURCC) ==
                        DDCAPS_OVERLAYFOURCC) ? TRUE : FALSE;
-        /* Determine if the hardware supports overlay surfaces */
-        bCanClipOverlay = ((ddcaps.dwCaps & DDCAPS_OVERLAYCANTCLIP) ==
+        /* Determine if the hardware supports overlay deinterlacing */
+        bCanDeinterlace = ((ddcaps.dwCaps & DDCAPS2_CANFLIPODDEVEN) ==
                        0 ) ? TRUE : FALSE;
         /* Determine if the hardware supports colorkeying */
         bHasColorKey = ((ddcaps.dwCaps & DDCAPS_COLORKEY) ==
@@ -1445,15 +1548,11 @@ static void DirectXGetDDrawCaps( vout_thread_t *p_vout )
                         DDCAPS_BLTFOURCC) ? TRUE : FALSE;
 
         msg_Dbg( p_vout, "DirectDraw Capabilities: overlay=%i yuvoverlay=%i "
-                         "can_clip_overlay=%i colorkey=%i stretch=%i "
+                         "can_deinterlace_overlay=%i colorkey=%i stretch=%i "
                          "bltfourcc=%i",
-                         bHasOverlay, bHasOverlayFourCC, bCanClipOverlay,
+                         bHasOverlay, bHasOverlayFourCC, bCanDeinterlace,
                          bHasColorKey, bCanStretch, bCanBltFourcc );
 
-        /* Overlay clipping support is interesting for us as it means we can
-         * get rid of the colorkey alltogether */
-        p_vout->p_sys->b_caps_overlay_clipping = bCanClipOverlay;
-
         /* Don't ask for troubles */
         if( !bCanBltFourcc ) p_vout->p_sys->b_hw_yuv = FALSE; 
     }
@@ -1534,6 +1633,60 @@ static int DirectXUnlockSurface( vout_thread_t *p_vout, picture_t *p_pic )
         return VLC_EGENERIC;
 }
 
+/*****************************************************************************
+ * DirectXFindColorkey: Finds out the 32bits RGB pixel value of the colorkey
+ *****************************************************************************/
+static DWORD DirectXFindColorkey( vout_thread_t *p_vout, uint32_t i_color )
+{
+    DDSURFACEDESC ddsd;
+    HRESULT dxresult;
+    COLORREF i_rgb = 0;
+    uint32_t i_pixel_backup;
+    HDC hdc;
+
+    ddsd.dwSize = sizeof(ddsd);
+    dxresult = IDirectDrawSurface2_Lock( p_vout->p_sys->p_display, NULL,
+                                         &ddsd, DDLOCK_WAIT, NULL );
+    if( dxresult != DD_OK ) return 0;
+
+    i_pixel_backup = *(uint32_t *)ddsd.lpSurface;
+
+    switch( ddsd.ddpfPixelFormat.dwRGBBitCount )
+    {
+    case 4:
+        *(uint8_t *)ddsd.lpSurface = 0x11;
+        break;
+    case 8:
+        *(uint8_t *)ddsd.lpSurface = 0x01;
+        break;
+    case 16:
+        *(uint16_t *)ddsd.lpSurface = 0x01;
+        break;
+    default:
+        *(uint32_t *)ddsd.lpSurface = 0x01;
+        break;
+    }
+
+    IDirectDrawSurface2_Unlock( p_vout->p_sys->p_display, NULL );
+
+    if( IDirectDrawSurface2_GetDC( p_vout->p_sys->p_display, &hdc ) == DD_OK )
+    {
+        i_rgb = GetPixel( hdc, 0, 0 );
+        IDirectDrawSurface2_ReleaseDC( p_vout->p_sys->p_display, hdc );
+    }
+
+    ddsd.dwSize = sizeof(ddsd);
+    dxresult = IDirectDrawSurface2_Lock( p_vout->p_sys->p_display, NULL,
+                                         &ddsd, DDLOCK_WAIT, NULL );
+    if( dxresult != DD_OK ) return i_rgb;
+
+    *(uint32_t *)ddsd.lpSurface = i_pixel_backup;
+
+    IDirectDrawSurface2_Unlock( p_vout->p_sys->p_display, NULL );
+
+    return i_rgb;
+}
+
 /*****************************************************************************
  * object variables callbacks: a bunch of object variables are used by the
  * interfaces to interact with the vout.
