@@ -2,15 +2,15 @@
  * svgalib.c : SVGAlib plugin for vlc
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: svgalib.c,v 1.3 2003/03/30 18:14:38 gbazin Exp $
+ * $Id: svgalib.c,v 1.4 2003/10/25 00:42:31 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
- *      
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -44,12 +44,12 @@ static void End       ( vout_thread_t * );
 static int  Manage    ( vout_thread_t * );
 static void Display   ( vout_thread_t *, picture_t * );
 
-static void SetPalette( vout_thread_t *, u16 *, u16 *, u16 * );
+static void SetPalette( vout_thread_t *, uint16_t *, uint16_t *, uint16_t * );
 
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-vlc_module_begin();                                            
+vlc_module_begin();
     set_description( _("SVGAlib video output") );
     set_capability( "video output", 0 );
     set_callbacks( Create, Destroy );
@@ -166,16 +166,16 @@ static int Init( vout_thread_t *p_vout )
             break;
         }
     }
-    
+
     /* Allocate the picture */
     if( p_pic == NULL )
     {
         return VLC_SUCCESS;
     }
 
-    vout_AllocatePicture( p_vout, p_pic, p_vout->output.i_width,
-                          p_vout->output.i_height,
-                          p_vout->output.i_chroma );
+    vout_AllocatePicture( p_vout, p_pic, p_vout->output.i_chroma,
+                          p_vout->output.i_width, p_vout->output.i_height,
+                          p_vout->output.i_aspect );
 
     if( p_pic->i_planes == 0 )
     {
@@ -250,7 +250,8 @@ static void Display( vout_thread_t *p_vout, picture_t *p_pic )
  *****************************************************************************
  * TODO: support 8 bits clut (for Mach32 cards and others).
  *****************************************************************************/
-static void SetPalette( vout_thread_t *p_vout, u16 *red, u16 *green, u16 *blue )
+static void SetPalette( vout_thread_t *p_vout,
+                        uint16_t *red, uint16_t *green, uint16_t *blue )
 {
     int i = 256;
 
