@@ -2,10 +2,10 @@
  * xcommon.c: Functions common to the X11 and XVideo plugins
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: xcommon.c,v 1.29 2003/08/13 18:39:52 gbazin Exp $
+ * $Id: xcommon.c,v 1.41 2003/12/22 14:32:56 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
- *          Samuel Hocevar <sam@zoy.org>
+ *          Sam Hocevar <sam@zoy.org>
  *          David Kennedy <dkennedy@tinytoad.com>
  *          Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -34,6 +34,7 @@
 #include <vlc/vlc.h>
 #include <vlc/intf.h>
 #include <vlc/vout.h>
+#include <vlc_keys.h>
 
 #ifdef HAVE_MACHINE_PARAM_H
     /* BSD */
@@ -118,6 +119,11 @@ static void SetPalette     ( vout_thread_t *,
 #endif
 
 static void TestNetWMSupport( vout_thread_t * );
+static int ConvertKey( int );
+
+/* Object variables callbacks */
+static int OnTopCallback( vlc_object_t *, char const *,
+                          vlc_value_t, vlc_value_t, void * );
 
 /*****************************************************************************
  * Activate: allocate X11 video thread output method
@@ -129,7 +135,9 @@ static void TestNetWMSupport( vout_thread_t * );
 int E_(Activate) ( vlc_object_t *p_this )
 {
     vout_thread_t *p_vout = (vout_thread_t *)p_this;
-    char *       psz_display;
+    char *        psz_display;
+     vlc_value_t  val, text;
+
 #ifdef MODULE_NAME_IS_xvideo
     char *       psz_chroma;
     vlc_fourcc_t i_chroma = 0;
@@ -273,6 +281,15 @@ int E_(Activate) ( vlc_object_t *p_this )
 
     TestNetWMSupport( p_vout );
 
+    /* Add a variable to indicate if the window should be on top of others */
+    var_Create( p_vout, "video-on-top", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    text.psz_string = _("Always on top");
+    var_Change( p_vout, "video-on-top", VLC_VAR_SETTEXT, &text, NULL );
+    var_AddCallback( p_vout, "video-on-top", OnTopCallback, NULL );
+    /* Trigger a callback right now */
+    var_Get( p_vout, "video-on-top", &val );
+    var_Set( p_vout, "video-on-top", val );
+
     return VLC_SUCCESS;
 }
 
@@ -366,7 +383,7 @@ static int InitVideo( vout_thread_t *p_vout )
         case 24:
             p_vout->output.i_chroma = VLC_FOURCC('R','V','2','4'); break;
         case 32:
-            p_vout->output.i_chroma = VLC_FOURCC('R','V','2','4'); break;
+            p_vout->output.i_chroma = VLC_FOURCC('R','V','3','2'); break;
         default:
             msg_Err( p_vout, "unknown screen depth %i",
                      p_vout->p_sys->i_screen_depth );
@@ -484,8 +501,6 @@ static void DisplayVideo( vout_thread_t *p_vout, picture_t *p_pic )
 static int ManageVideo( vout_thread_t *p_vout )
 {
     XEvent      xevent;                                         /* X11 event */
-    char        i_key;                                    /* ISO Latin-1 key */
-    KeySym      x_key_symbol;
     vlc_value_t val;
 
     /* Handle X11 events: ConfigureNotify events are parsed to know if the
@@ -517,119 +532,44 @@ static int ManageVideo( vout_thread_t *p_vout )
         /* Keyboard event */
         else if( xevent.type == KeyPress )
         {
+            unsigned int state = xevent.xkey.state;
+            KeySym x_key_symbol;
+            char i_key;                                   /* ISO Latin-1 key */
+
             /* We may have keys like F1 trough F12, ESC ... */
             x_key_symbol = XKeycodeToKeysym( p_vout->p_sys->p_display,
                                              xevent.xkey.keycode, 0 );
-            switch( (int)x_key_symbol )
+            val.i_int = ConvertKey( (int)x_key_symbol );
+
+            xevent.xkey.state &= ~ShiftMask;
+            xevent.xkey.state &= ~ControlMask;
+            xevent.xkey.state &= ~Mod1Mask;
+
+            if( !val.i_int &&
+                XLookupString( &xevent.xkey, &i_key, 1, NULL, NULL ) )
             {
-            case XK_Escape:
-                if( p_vout->b_fullscreen )
-                {
-                    p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
-                }
-                else
-                {
-                    /* the user wants to close the window */
-                    playlist_t * p_playlist =
-                        (playlist_t *)vlc_object_find( p_vout,
-                                                       VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-                    if( p_playlist != NULL )
-                    {
-                        playlist_Stop( p_playlist );
-                        vlc_object_release( p_playlist );
-                    }
-                }
-                break;
-            case XK_Menu:
-                {
-                    intf_thread_t *p_intf;
-                    playlist_t * p_playlist;
-
-                    p_intf = vlc_object_find( p_vout, VLC_OBJECT_INTF,
-                                                      FIND_ANYWHERE );
-                    if( p_intf )
-                    {
-                        p_intf->b_menu_change = 1;
-                        vlc_object_release( p_intf );
-                    }
-
-                    p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST,
-                                                           FIND_ANYWHERE );
-                    if( p_playlist != NULL )
-                    {
-                        vlc_value_t val;
-                        var_Set( p_playlist, "intf-popupmenu", val );
-                        vlc_object_release( p_playlist );
-                    }
-                }
-                break;
-            case XK_Left:
-/*                input_Seek( p_vout, -5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "LEFT";
-                var_Set( p_vout, "key-pressed", val );
-                break;
-            case XK_Right:
-/*                input_Seek( p_vout, 5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "RIGHT";
-                var_Set( p_vout, "key-pressed", val );
-                break;
-            case XK_Up:
-/*                input_Seek( p_vout, 60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "UP";
-                var_Set( p_vout, "key-pressed", val );
-                break;
-            case XK_Down:
-/*                input_Seek( p_vout, -60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
-                val.psz_string = "DOWN";
-                var_Set( p_vout, "key-pressed", val );
-                break;
-            case XK_Return:
-            case XK_KP_Enter:
-                val.psz_string = "ENTER";
-                var_Set( p_vout, "key-pressed", val );
-                break;
-            case XK_Home:
-                input_Seek( p_vout, 0, INPUT_SEEK_BYTES | INPUT_SEEK_SET );
-                break;
-            case XK_End:
-                input_Seek( p_vout, 0, INPUT_SEEK_BYTES | INPUT_SEEK_END );
-                break;
-            case XK_Page_Up:
-                input_Seek( p_vout, 10, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
-                break;
-            case XK_Page_Down:
-                input_Seek( p_vout, -10, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
-                break;
-            case XK_space:
-                input_SetStatus( p_vout, INPUT_STATUS_PAUSE );
-                break;
-
-            default:
                 /* "Normal Keys"
                  * The reason why I use this instead of XK_0 is that
                  * with XLookupString, we don't have to care about
                  * keymaps. */
+                val.i_int = i_key;
+            }
 
-                if( XLookupString( &xevent.xkey, &i_key, 1, NULL, NULL ) )
+            if( val.i_int )
+            {
+                if( state & ShiftMask )
                 {
-                    /* FIXME: handle stuff here */
-                    switch( i_key )
-                    {
-                    case 'q':
-                    case 'Q':
-                        p_vout->p_vlc->b_die = 1;
-                        break;
-                    case 'f':
-                    case 'F':
-                        p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
-                        break;
-
-                    default:
-                        break;
-                    }
+                    val.i_int |= KEY_MODIFIER_SHIFT;
                 }
-                break;
+                if( state & ControlMask )
+                {
+                    val.i_int |= KEY_MODIFIER_CTRL;
+                }
+                if( state & Mod1Mask )
+                {
+                    val.i_int |= KEY_MODIFIER_ALT;
+                }
+                var_Set( p_vout->p_vlc, "key-pressed", val );
             }
         }
         /* Mouse click */
@@ -641,7 +581,7 @@ static int ManageVideo( vout_thread_t *p_vout )
                     var_Get( p_vout, "mouse-button-down", &val );
                     val.i_int |= 1;
                     var_Set( p_vout, "mouse-button-down", val );
-                    
+
                     /* detect double-clicks */
                     if( ( ((XButtonEvent *)&xevent)->time -
                           p_vout->p_sys->i_time_button_last_pressed ) < 300 )
@@ -657,13 +597,13 @@ static int ManageVideo( vout_thread_t *p_vout )
                     val.i_int |= 2;
                     var_Set( p_vout, "mouse-button-down", val );
                     break;
-                
+
                 case Button3:
                     var_Get( p_vout, "mouse-button-down", &val );
                     val.i_int |= 4;
                     var_Set( p_vout, "mouse-button-down", val );
                     break;
-                
+
                 case Button4:
                     var_Get( p_vout, "mouse-button-down", &val );
                     val.i_int |= 8;
@@ -692,7 +632,7 @@ static int ManageVideo( vout_thread_t *p_vout )
                     val.b_bool = VLC_TRUE;
                     var_Set( p_vout, "mouse-clicked", val );
                     break;
-                    
+
                 case Button2:
                     {
                         playlist_t *p_playlist;
@@ -714,7 +654,7 @@ static int ManageVideo( vout_thread_t *p_vout )
                         }
                     }
                     break;
-                    
+
                 case Button3:
                     {
                         intf_thread_t *p_intf;
@@ -754,7 +694,7 @@ static int ManageVideo( vout_thread_t *p_vout )
                     val.i_int &= ~16;
                     var_Set( p_vout, "mouse-button-down", val );
                     break;
-                    
+
             }
         }
         /* Mouse move */
@@ -882,11 +822,9 @@ static int ManageVideo( vout_thread_t *p_vout )
                            p_vout->p_sys->p_win->i_height,
                            &i_x, &i_y, &i_width, &i_height );
 
-        XResizeWindow( p_vout->p_sys->p_display,
-                       p_vout->p_sys->p_win->video_window, i_width, i_height );
-
-        XMoveWindow( p_vout->p_sys->p_display,
-                     p_vout->p_sys->p_win->video_window, i_x, i_y );
+        XMoveResizeWindow( p_vout->p_sys->p_display,
+                           p_vout->p_sys->p_win->video_window,
+                           i_x, i_y, i_width, i_height );
     }
 
     /* Autohide Cursour */
@@ -1148,6 +1086,10 @@ static void DestroyWindow( vout_thread_t *p_vout, x11_window_t *p_win )
  *****************************************************************************/
 static int NewPicture( vout_thread_t *p_vout, picture_t *p_pic )
 {
+#ifdef MODULE_NAME_IS_xvideo
+    int i_plane;
+#endif
+
     /* We know the chroma, allocate a buffer which will be used
      * directly by the decoder */
     p_pic->p_sys = malloc( sizeof( picture_sys_t ) );
@@ -1157,6 +1099,11 @@ static int NewPicture( vout_thread_t *p_vout, picture_t *p_pic )
         return -1;
     }
 
+    /* Fill in picture_t fields */
+    vout_InitPicture( VLC_OBJECT(p_vout), p_pic, p_vout->output.i_chroma,
+                      p_vout->output.i_width, p_vout->output.i_height,
+                      p_vout->output.i_aspect );
+
 #ifdef HAVE_SYS_SHM_H
     if( p_vout->p_sys->b_shm )
     {
@@ -1180,6 +1127,7 @@ static int NewPicture( vout_thread_t *p_vout, picture_t *p_pic )
             CreateImage( p_vout, p_vout->p_sys->p_display,
 #ifdef MODULE_NAME_IS_xvideo
                          p_vout->p_sys->i_xvport, p_vout->output.i_chroma,
+                         p_pic->format.i_bits_per_pixel,
 #else
                          p_vout->p_sys->p_visual,
                          p_vout->p_sys->i_screen_depth,
@@ -1198,148 +1146,50 @@ static int NewPicture( vout_thread_t *p_vout, picture_t *p_pic )
     {
 #ifdef MODULE_NAME_IS_xvideo
         case VLC_FOURCC('I','4','2','0'):
-
-            p_pic->Y_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p[Y_PLANE].i_lines = p_vout->output.i_height;
-            p_pic->p[Y_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[0];
-            p_pic->p[Y_PLANE].i_pixel_pitch = 1;
-            p_pic->p[Y_PLANE].i_visible_pitch = p_vout->output.i_width;
-
-            p_pic->U_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[1];
-            p_pic->p[U_PLANE].i_lines = p_vout->output.i_height / 2;
-            p_pic->p[U_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[1];
-            p_pic->p[U_PLANE].i_pixel_pitch = 1;
-            p_pic->p[U_PLANE].i_visible_pitch = p_vout->output.i_width / 2;
-
-            p_pic->V_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[2];
-            p_pic->p[V_PLANE].i_lines = p_vout->output.i_height / 2;
-            p_pic->p[V_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[2];
-            p_pic->p[V_PLANE].i_pixel_pitch = 1;
-            p_pic->p[V_PLANE].i_visible_pitch = p_vout->output.i_width / 2;
-
-            p_pic->i_planes = 3;
-            break;
-
         case VLC_FOURCC('Y','V','1','2'):
-
-            p_pic->Y_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p[Y_PLANE].i_lines = p_vout->output.i_height;
-            p_pic->p[Y_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[0];
-            p_pic->p[Y_PLANE].i_pixel_pitch = 1;
-            p_pic->p[Y_PLANE].i_visible_pitch = p_pic->p[Y_PLANE].i_pitch;
-            p_pic->p[Y_PLANE].i_visible_pitch = p_vout->output.i_width;
-
-            p_pic->U_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[2];
-            p_pic->p[U_PLANE].i_lines = p_vout->output.i_height / 2;
-            p_pic->p[U_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[2];
-            p_pic->p[U_PLANE].i_pixel_pitch = 1;
-            p_pic->p[U_PLANE].i_visible_pitch = p_vout->output.i_width / 2;
-
-            p_pic->V_PIXELS = p_pic->p_sys->p_image->data
-                               + p_pic->p_sys->p_image->offsets[1];
-            p_pic->p[V_PLANE].i_lines = p_vout->output.i_height / 2;
-            p_pic->p[V_PLANE].i_pitch = p_pic->p_sys->p_image->pitches[1];
-            p_pic->p[V_PLANE].i_pixel_pitch = 1;
-            p_pic->p[V_PLANE].i_visible_pitch = p_vout->output.i_width / 2;
-
-            p_pic->i_planes = 3;
-            break;
-
         case VLC_FOURCC('Y','2','1','1'):
-
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p->i_lines = p_vout->output.i_height;
-            /* XXX: this just looks so plain wrong... check it out ! */
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->pitches[0] / 4;
-            p_pic->p->i_pixel_pitch = 4;
-            p_pic->p->i_visible_pitch = p_vout->output.i_width * 4;
-
-            p_pic->i_planes = 1;
-            break;
-
         case VLC_FOURCC('Y','U','Y','2'):
         case VLC_FOURCC('U','Y','V','Y'):
-
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p->i_lines = p_vout->output.i_height;
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->pitches[0];
-            p_pic->p->i_pixel_pitch = 4;
-            p_pic->p->i_visible_pitch = p_vout->output.i_width * 4;
-
-            p_pic->i_planes = 1;
-            break;
-
         case VLC_FOURCC('R','V','1','5'):
-
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p->i_lines = p_vout->output.i_height;
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->pitches[0];
-            p_pic->p->i_pixel_pitch = 2;
-            p_pic->p->i_visible_pitch = p_vout->output.i_width * 2;
-
-            p_pic->i_planes = 1;
-            break;
-
         case VLC_FOURCC('R','V','1','6'):
+        case VLC_FOURCC('R','V','2','4'): /* Fixme: pixel pitch == 4 ? */
+        case VLC_FOURCC('R','V','3','2'):
 
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->offsets[0];
-            p_pic->p->i_lines = p_vout->output.i_height;
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->pitches[0];
-            p_pic->p->i_pixel_pitch = 2;
-            p_pic->p->i_visible_pitch = p_vout->output.i_width * 2;
-
-            p_pic->i_planes = 1;
+            for( i_plane = 0; i_plane < p_pic->p_sys->p_image->num_planes;
+                 i_plane++ )
+            {
+                p_pic->p[i_plane].p_pixels = p_pic->p_sys->p_image->data
+                    + p_pic->p_sys->p_image->offsets[i_plane];
+                p_pic->p[i_plane].i_pitch =
+                    p_pic->p_sys->p_image->pitches[i_plane];
+            }
+            if( p_vout->output.i_chroma == VLC_FOURCC('Y','V','1','2') )
+            {
+                /* U and V inverted compared to I420
+                 * Fixme: this should be handled by the vout core */
+                p_pic->U_PIXELS = p_pic->p_sys->p_image->data
+                    + p_pic->p_sys->p_image->offsets[2];
+                p_pic->V_PIXELS = p_pic->p_sys->p_image->data
+                    + p_pic->p_sys->p_image->offsets[1];
+            }
             break;
 
 #else
         case VLC_FOURCC('R','G','B','2'):
-
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->xoffset;
-            p_pic->p->i_lines = p_pic->p_sys->p_image->height;
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->bytes_per_line;
-            p_pic->p->i_pixel_pitch = p_pic->p_sys->p_image->depth;
-            p_pic->p->i_visible_pitch = p_pic->p_sys->p_image->width;
-
-            p_pic->i_planes = 1;
-
-            break;
-
         case VLC_FOURCC('R','V','1','6'):
         case VLC_FOURCC('R','V','1','5'):
-
-            p_pic->p->p_pixels = p_pic->p_sys->p_image->data
-                                  + p_pic->p_sys->p_image->xoffset;
-            p_pic->p->i_lines = p_pic->p_sys->p_image->height;
-            p_pic->p->i_pitch = p_pic->p_sys->p_image->bytes_per_line;
-            p_pic->p->i_pixel_pitch = p_pic->p_sys->p_image->depth;
-            p_pic->p->i_visible_pitch = 2 * p_pic->p_sys->p_image->width;
-
-            p_pic->i_planes = 1;
-
-            break;
-
-        case VLC_FOURCC('R','V','3','2'):
         case VLC_FOURCC('R','V','2','4'):
+        case VLC_FOURCC('R','V','3','2'):
 
+            p_pic->p->i_lines = p_pic->p_sys->p_image->height;
             p_pic->p->p_pixels = p_pic->p_sys->p_image->data
                                   + p_pic->p_sys->p_image->xoffset;
-            p_pic->p->i_lines = p_pic->p_sys->p_image->height;
             p_pic->p->i_pitch = p_pic->p_sys->p_image->bytes_per_line;
-            p_pic->p->i_pixel_pitch = p_pic->p_sys->p_image->depth;
-            p_pic->p->i_visible_pitch = 4 * p_pic->p_sys->p_image->width;
-
-            p_pic->i_planes = 1;
 
+            /* p_pic->p->i_pixel_pitch = 4 for RV24 but this should be set
+             * properly by vout_InitPicture() */
+            p_pic->p->i_visible_pitch = p_pic->p->i_pixel_pitch
+                                         * p_pic->p_sys->p_image->width;
             break;
 #endif
 
@@ -2150,7 +2000,7 @@ static IMAGE_TYPE * CreateImage( vout_thread_t *p_vout,
 
     /* Allocate memory for image */
 #ifdef MODULE_NAME_IS_xvideo
-    p_data = (byte_t *) malloc( i_width * i_height * 2 ); /* XXX */
+    p_data = (byte_t *) malloc( i_width * i_height * i_bits_per_pixel / 8 );
 #else
     i_bytes_per_line = i_width * i_bytes_per_pixel;
     p_data = (byte_t *) malloc( i_bytes_per_line * i_height );
@@ -2251,7 +2101,7 @@ static void TestNetWMSupport( vout_thread_t *p_vout )
                                 0, 16384, False, AnyPropertyType,
                                 &net_wm_supported,
                                 &i_format, &i_items, &i_bytesafter,
-                                (unsigned char **)&p_args );
+                                (unsigned char **)(intptr_t)&p_args );
 
     if( i_ret != Success || i_items == 0 ) return;
 
@@ -2298,3 +2148,83 @@ static void TestNetWMSupport( vout_thread_t *p_vout )
 
     XFree( p_args );
 }
+
+/*****************************************************************************
+ * Key events handling
+ *****************************************************************************/
+static struct
+{
+    int i_x11key;
+    int i_vlckey;
+
+} x11keys_to_vlckeys[] =
+{
+    { XK_F1, KEY_F1 }, { XK_F2, KEY_F2 }, { XK_F3, KEY_F3 }, { XK_F4, KEY_F4 },
+    { XK_F5, KEY_F5 }, { XK_F6, KEY_F6 }, { XK_F7, KEY_F7 }, { XK_F8, KEY_F8 },
+    { XK_F9, KEY_F9 }, { XK_F10, KEY_F10 }, { XK_F11, KEY_F11 },
+    { XK_F12, KEY_F12 },
+
+    { XK_Return, KEY_ENTER },
+    { XK_KP_Enter, KEY_ENTER },
+    { XK_space, KEY_SPACE },
+    { XK_Escape, KEY_ESC },
+
+    { XK_Menu, KEY_MENU },
+    { XK_Left, KEY_LEFT },
+    { XK_Right, KEY_RIGHT },
+    { XK_Up, KEY_UP },
+    { XK_Down, KEY_DOWN },
+
+    { XK_Home, KEY_HOME },
+    { XK_End, KEY_END },
+    { XK_Page_Up, KEY_PAGEUP },
+    { XK_Page_Down, KEY_PAGEDOWN },
+    { 0, 0 }
+};
+
+static int ConvertKey( int i_key )
+{
+    int i;
+
+    for( i = 0; x11keys_to_vlckeys[i].i_x11key != 0; i++ )
+    {
+        if( x11keys_to_vlckeys[i].i_x11key == i_key )
+        {
+            return x11keys_to_vlckeys[i].i_vlckey;
+        }
+    }
+
+    return 0;
+}
+
+/*****************************************************************************
+ * object variables callbacks: a bunch of object variables are used by the
+ * interfaces to interact with the vout.
+ *****************************************************************************/
+static int OnTopCallback( vlc_object_t *p_this, char const *psz_cmd,
+                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+
+    if( p_vout->p_sys->b_net_wm_state_stays_on_top )
+    {
+        XClientMessageEvent event;
+
+        memset( &event, 0, sizeof( XClientMessageEvent ) );
+
+        event.type = ClientMessage;
+        event.message_type = p_vout->p_sys->net_wm_state;
+        event.display = p_vout->p_sys->p_display;
+        event.window = p_vout->p_sys->p_win->base_window;
+        event.format = 32;
+        event.data.l[ 0 ] = newval.b_bool; /* set property */
+        event.data.l[ 1 ] = p_vout->p_sys->net_wm_state_stays_on_top;
+
+        XSendEvent( p_vout->p_sys->p_display,
+                    DefaultRootWindow( p_vout->p_sys->p_display ),
+                    False, SubstructureRedirectMask,
+                    (XEvent*)&event );
+    }
+
+    return VLC_SUCCESS;
+}
