@@ -2,7 +2,7 @@
  * xvideo.c : Xvideo plugin for vlc
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: xvideo.c,v 1.6 2003/03/30 18:14:39 gbazin Exp $
+ * $Id: xvideo.c,v 1.8 2003/05/25 19:24:53 gbazin Exp $
  *
  * Authors: Shane Harper <shanegh@optusnet.com.au>
  *          Vincent Seguin <seguin@via.ecp.fr>
@@ -46,7 +46,7 @@ extern void E_(Deactivate) ( vlc_object_t * );
     "If you graphics card provides several adaptors, this option allows you " \
     "to choose which one will be used (you shouldn't have to change this).")
 
-#define ALT_FS_TEXT N_("alternate fullscreen method")
+#define ALT_FS_TEXT N_("Alternate fullscreen method")
 #define ALT_FS_LONGTEXT N_( \
     "There are two ways to make a fullscreen window, unfortunately each one " \
     "has its drawbacks.\n" \
@@ -65,10 +65,15 @@ extern void E_(Deactivate) ( vlc_object_t * );
     "Force the XVideo renderer to use a specific chroma format instead of " \
     "trying to improve performances by using the most efficient one.")
 
-#define SHM_TEXT N_("use shared memory")
+#define SHM_TEXT N_("Use shared memory")
 #define SHM_LONGTEXT N_( \
     "Use shared memory to communicate between VLC and the X server.")
 
+#define SCREEN_TEXT N_("choose the screen to be used for fullscreen mode.")
+#define SCREEN_LONGTEXT N_( \
+    "Choose the screen you want to use in fullscreen mode. For instance " \
+    "set it to 0 for first screen, 1 for the second.")
+
 vlc_module_begin();
     add_category_hint( N_("XVideo"), NULL, VLC_TRUE );
     add_string( "xvideo-display", NULL, NULL, DISPLAY_TEXT, DISPLAY_LONGTEXT, VLC_TRUE );
@@ -78,6 +83,10 @@ vlc_module_begin();
 #ifdef HAVE_SYS_SHM_H
     add_bool( "xvideo-shm", 1, NULL, SHM_TEXT, SHM_LONGTEXT, VLC_TRUE );
 #endif
+#ifdef HAVE_XINERAMA
+    add_integer ( "xvideo-xineramascreen", 0, NULL, SCREEN_TEXT, SCREEN_LONGTEXT, VLC_TRUE );
+#endif
+
     set_description( _("XVideo extension video output") );
     set_capability( "video output", 150 );
     set_callbacks( E_(Activate), E_(Deactivate) );
