@@ -2,7 +2,7 @@
  * input_ext-plugins.c: useful functions for access and demux plug-ins
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: input_ext-plugins.c,v 1.31 2003/03/24 17:15:30 gbazin Exp $
+ * $Id: input_ext-plugins.c,v 1.38 2003/11/24 03:30:38 fenrir Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -145,7 +145,7 @@ void input_BuffersEnd( input_thread_t * p_input, input_buffers_t * p_buffers )
 
         if( p_buffers->i_allocated )
         {
-            msg_Err( p_input, "%u bytes have not been freed, "
+            msg_Warn( p_input, "%u bytes have not been freed, "
                               "expect memory leak", p_buffers->i_allocated );
         }
 
@@ -158,13 +158,12 @@ void input_BuffersEnd( input_thread_t * p_input, input_buffers_t * p_buffers )
  * input_NewBuffer: return a pointer to a data buffer of the appropriate size
  *****************************************************************************/
 static inline data_buffer_t * NewBuffer( input_buffers_t * p_buffers,
-                                         size_t i_size,
-                                         vlc_bool_t b_forced )
+                                         size_t i_size )
 {
     data_buffer_t * p_buf;
 
     /* Safety check */
-    if( !b_forced && p_buffers->i_allocated > INPUT_MAX_ALLOCATION )
+    if( p_buffers->i_allocated > INPUT_MAX_ALLOCATION )
     {
         return NULL;
     }
@@ -214,7 +213,7 @@ data_buffer_t * input_NewBuffer( input_buffers_t * p_buffers, size_t i_size )
     data_buffer_t * p_buf;
 
     vlc_mutex_lock( &p_buffers->lock );
-    p_buf = NewBuffer( p_buffers, i_size, VLC_FALSE );
+    p_buf = NewBuffer( p_buffers, i_size );
     vlc_mutex_unlock( &p_buffers->lock );
 
     return p_buf;
@@ -305,13 +304,12 @@ data_packet_t * input_ShareBuffer( input_buffers_t * p_buffers,
  * input_NewPacket: allocate a packet along with a buffer
  *****************************************************************************/
 static inline data_packet_t * NewPacket( input_buffers_t * p_buffers,
-                                         size_t i_size,
-                                         vlc_bool_t b_forced )
+                                         size_t i_size )
 {
     data_buffer_t * p_buf;
     data_packet_t * p_data;
 
-    p_buf = NewBuffer( p_buffers, i_size, b_forced );
+    p_buf = NewBuffer( p_buffers, i_size );
 
     if( p_buf == NULL )
     {
@@ -331,18 +329,7 @@ data_packet_t * input_NewPacket( input_buffers_t * p_buffers, size_t i_size )
     data_packet_t * p_data;
 
     vlc_mutex_lock( &p_buffers->lock );
-    p_data = NewPacket( p_buffers, i_size, VLC_FALSE );
-    vlc_mutex_unlock( &p_buffers->lock );
-
-    return p_data;
-}
-
-data_packet_t * input_NewPacketForce( input_buffers_t * p_buffers, size_t i_size )
-{
-    data_packet_t * p_data;
-
-    vlc_mutex_lock( &p_buffers->lock );
-    p_data = NewPacket( p_buffers, i_size, VLC_TRUE);
+    p_data = NewPacket( p_buffers, i_size );
     vlc_mutex_unlock( &p_buffers->lock );
 
     return p_data;
@@ -483,20 +470,30 @@ void input_DeletePES( input_buffers_t * p_buffers, pes_packet_t * p_pes )
 ssize_t input_FillBuffer( input_thread_t * p_input )
 {
     ptrdiff_t i_remains = p_input->p_last_data - p_input->p_current_data;
-    data_buffer_t * p_buf;
+    data_buffer_t * p_buf = NULL;
     ssize_t i_ret;
 
     vlc_mutex_lock( &p_input->p_method_data->lock );
 
-    p_buf = NewBuffer( p_input->p_method_data,
-                       i_remains + p_input->i_bufsize, VLC_FALSE );
-    if( p_buf == NULL )
+    while( p_buf == NULL )
     {
-        vlc_mutex_unlock( &p_input->p_method_data->lock );
-        msg_Err( p_input, "failed allocating a new buffer (decoder stuck?)" );
-        msleep( INPUT_IDLE_SLEEP );
-        return -1;
+        p_buf = NewBuffer( p_input->p_method_data,
+                           i_remains + p_input->i_bufsize );
+        if( p_buf == NULL )
+        {
+            vlc_mutex_unlock( &p_input->p_method_data->lock );
+            msg_Err( p_input,
+                     "failed allocating a new buffer (decoder stuck?)" );
+            msleep( INPUT_IDLE_SLEEP );
+
+            if( p_input->b_die || p_input->b_error || p_input->b_eof )
+            {
+                return -1;
+            }
+            vlc_mutex_lock( &p_input->p_method_data->lock );
+        }
     }
+
     p_buf->i_refcount = 1;
 
     if( p_input->p_data_buffer != NULL )
@@ -510,6 +507,10 @@ ssize_t input_FillBuffer( input_thread_t * p_input )
         ReleaseBuffer( p_input->p_method_data, p_input->p_data_buffer );
     }
 
+    p_input->p_data_buffer = p_buf;
+    p_input->p_current_data = (byte_t *)p_buf + sizeof(data_buffer_t);
+    p_input->p_last_data = p_input->p_current_data + i_remains;
+
     /* Do not hold the lock during pf_read (blocking call). */
     vlc_mutex_unlock( &p_input->p_method_data->lock );
 
@@ -525,9 +526,7 @@ ssize_t input_FillBuffer( input_thread_t * p_input )
 
     if( i_ret < 0 ) i_ret = 0;
 
-    p_input->p_data_buffer = p_buf;
-    p_input->p_current_data = (byte_t *)p_buf + sizeof(data_buffer_t);
-    p_input->p_last_data = p_input->p_current_data + i_remains + i_ret;
+    p_input->p_last_data += i_ret;
 
     return (ssize_t)i_remains + i_ret;
 }
@@ -581,7 +580,7 @@ ssize_t input_SplitBuffer( input_thread_t * p_input,
         }
     }
 
-    if ( !i_size )
+    if ( i_size < 0)
     {
         return 0;
     }
