@@ -1,9 +1,9 @@
 /*****************************************************************************
- * mtime.c: high rezolution time management functions
+ * mtime.c: high resolution time management functions
  * Functions are prototyped in mtime.h.
  *****************************************************************************
- * Copyright (C) 1998-2001 VideoLAN
- * $Id: mtime.c,v 1.35 2002/11/11 14:39:12 sam Exp $
+ * Copyright (C) 1998-2001, 2003 VideoLAN
+ * $Id: mtime.c,v 1.41 2003/12/11 10:44:32 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -64,18 +64,21 @@ struct timespec
 int nanosleep(struct timespec *, struct timespec *);
 #endif
 
-/*****************************************************************************
- * mstrtime: return a date in a readable format
- *****************************************************************************
- * This functions is provided for any interface function which need to print a
- * date. psz_buffer should be a buffer long enough to store the formatted
- * date.
- *****************************************************************************/
+/**
+ * Return a date in a readable format
+ *
+ * This function converts a mtime date into a string.
+ * psz_buffer should be a buffer long enough to store the formatted
+ * date. 
+ * \param date to be converted
+ * \param psz_buffer should be a buffer at least MSTRTIME_MAX_SIZE characters
+ * \return psz_buffer is returned so this can be used as printf parameter.
+ */
 char *mstrtime( char *psz_buffer, mtime_t date )
 {
     static mtime_t ll1000 = 1000, ll60 = 60, ll24 = 24;
 
-    sprintf( psz_buffer, "%02d:%02d:%02d-%03d.%03d",
+    snprintf( psz_buffer, MSTRTIME_MAX_SIZE, "%02d:%02d:%02d-%03d.%03d",
              (int) (date / (ll1000 * ll1000 * ll60 * ll60) % ll24),
              (int) (date / (ll1000 * ll1000 * ll60) % ll60),
              (int) (date / (ll1000 * ll1000) % ll60),
@@ -84,32 +87,106 @@ char *mstrtime( char *psz_buffer, mtime_t date )
     return( psz_buffer );
 }
 
-/*****************************************************************************
- * mdate: return high precision date
- *****************************************************************************
+/**
+ * Convert seconds to a time in the format h:mm:ss.
+ *
+ * This function is provided for any interface function which need to print a
+ * time string in the format h:mm:ss
+ * date.
+ * \param secs  the date to be converted
+ * \param psz_buffer should be a buffer at least MSTRTIME_MAX_SIZE characters
+ * \return psz_buffer is returned so this can be used as printf parameter.
+ */
+char *secstotimestr( char *psz_buffer, int i_seconds )
+{
+    snprintf( psz_buffer, MSTRTIME_MAX_SIZE, "%d:%2.2d:%2.2d",
+              (int) (i_seconds / (60 *60)), 
+              (int) ((i_seconds / 60) % 60), 
+              (int) (i_seconds % 60) );
+    return( psz_buffer );
+}
+
+/**
+ * Return high precision date
+ *
  * Uses the gettimeofday() function when possible (1 MHz resolution) or the
  * ftime() function (1 kHz resolution).
- *****************************************************************************/
+ */
 mtime_t mdate( void )
 {
 #if defined( HAVE_KERNEL_OS_H )
     return( real_time_clock_usecs() );
 
 #elif defined( WIN32 ) || defined( UNDER_CE )
-    /* We don't get the real date, just the value of a high precision timer.
-     * this is because the usual time functions have at best only a milisecond
-     * resolution */
-    mtime_t freq, usec_time;
+    /* We don't need the real date, just the value of a high precision timer */
+    static mtime_t freq = I64C(-1);
+    mtime_t usec_time;
 
-    if( QueryPerformanceFrequency( (LARGE_INTEGER *)&freq ) )
+    if( freq == I64C(-1) )
+    {
+        /* Extract from the Tcl source code:
+         * (http://www.cs.man.ac.uk/fellowsd-bin/TIP/7.html)
+         *
+         * Some hardware abstraction layers use the CPU clock
+         * in place of the real-time clock as a performance counter
+         * reference.  This results in:
+         *    - inconsistent results among the processors on
+         *      multi-processor systems.
+         *    - unpredictable changes in performance counter frequency
+         *      on "gearshift" processors such as Transmeta and
+         *      SpeedStep.
+         * There seems to be no way to test whether the performance
+         * counter is reliable, but a useful heuristic is that
+         * if its frequency is 1.193182 MHz or 3.579545 MHz, it's
+         * derived from a colorburst crystal and is therefore
+         * the RTC rather than the TSC.  If it's anything else, we
+         * presume that the performance counter is unreliable.
+         */
+
+        freq = ( QueryPerformanceFrequency( (LARGE_INTEGER *)&freq ) &&
+                 (freq == I64C(1193182) || freq == I64C(3579545) ) )
+               ? freq : 0;
+    }
+
+    if( freq != 0 )
     {
         /* Microsecond resolution */
         QueryPerformanceCounter( (LARGE_INTEGER *)&usec_time );
         return ( usec_time * 1000000 ) / freq;
     }
+    else
+    {
+        /* Fallback on GetTickCount() which has a milisecond resolution
+         * (actually, best case is about 10 ms resolution)
+         * GetTickCount() only returns a DWORD thus will wrap after
+         * about 49.7 days so we try to detect the wrapping. */
 
-    /* Milisecond resolution */
-    return 1000 * GetTickCount();
+        static CRITICAL_SECTION date_lock;
+        static mtime_t i_previous_time = I64C(-1);
+        static int i_wrap_counts = -1;
+
+        if( i_wrap_counts == -1 )
+        {
+            /* Initialization */
+            i_previous_time = I64C(1000) * GetTickCount();
+            InitializeCriticalSection( &date_lock );
+            i_wrap_counts = 0;
+        }
+
+        EnterCriticalSection( &date_lock );
+        usec_time = I64C(1000) *
+            (i_wrap_counts * I64C(0x100000000) + GetTickCount());
+        if( i_previous_time > usec_time )
+        {
+            /* Counter wrapped */
+            i_wrap_counts++;
+            usec_time += I64C(0x100000000000);
+        }
+        i_previous_time = usec_time;
+        LeaveCriticalSection( &date_lock );
+
+        return usec_time;
+    }
 
 #else
     struct timeval tv_date;
@@ -123,13 +200,14 @@ mtime_t mdate( void )
 #endif
 }
 
-/*****************************************************************************
- * mwait: wait for a date
- *****************************************************************************
+/**
+ * Wait for a date
+ *
  * This function uses select() and an system date function to wake up at a
  * precise date. It should be used for process synchronization. If current date
  * is posterior to wished date, the function returns immediately.
- *****************************************************************************/
+ * \param date The date to wake up at
+ */
 void mwait( mtime_t date )
 {
 #if defined( HAVE_KERNEL_OS_H )
@@ -202,11 +280,12 @@ void mwait( mtime_t date )
 #endif
 }
 
-/*****************************************************************************
- * msleep: more precise sleep()
- *****************************************************************************
+/**
+ * More precise sleep()
+ *
  * Portable usleep() function.
- *****************************************************************************/
+ * \param delay the amount of time to sleep
+ */
 void msleep( mtime_t delay )
 {
 #if defined( HAVE_KERNEL_OS_H )
