@@ -2,7 +2,7 @@
  * stream_output.c : stream output module
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: stream_output.c,v 1.33 2003/08/09 14:59:24 gbazin Exp $
+ * $Id: stream_output.c,v 1.36 2003/12/07 17:09:33 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -73,6 +73,42 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
                                        char * psz_dest )
 {
     sout_instance_t *p_sout;
+    vlc_value_t keep;
+
+    if( var_Get( p_parent, "sout-keep", &keep ) < 0 )
+    {
+        msg_Warn( p_parent, "cannot get sout-keep value" );
+        keep.b_bool = VLC_FALSE;
+    }
+    else if( keep.b_bool )
+    {
+        msg_Warn( p_parent, "sout-keep true" );
+        if( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT, FIND_ANYWHERE ) ) )
+        {
+            if( !strcmp( p_sout->psz_sout, psz_dest ) )
+            {
+                msg_Warn( p_parent, "sout keep : reusing sout" );
+                msg_Warn( p_parent, "sout keep : you probably want to use gather stream_out" );
+                vlc_object_detach( p_sout );
+                vlc_object_attach( p_sout, p_parent );
+                vlc_object_release( p_sout );
+                return p_sout;
+            }
+            else
+            {
+                msg_Warn( p_parent, "sout keep : destroying unusable sout" );
+                sout_DeleteInstance( p_sout );
+            }
+        }
+    }
+    else if( !keep.b_bool )
+    {
+        while( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT, FIND_PARENT ) ) )
+        {
+            msg_Warn( p_parent, "sout keep : destroying old sout" );
+            sout_DeleteInstance( p_sout );
+        }
+    }
 
     /* *** Allocate descriptor *** */
     p_sout = vlc_object_create( p_parent, VLC_OBJECT_SOUT );
@@ -139,30 +175,17 @@ void sout_DeleteInstance( sout_instance_t * p_sout )
  * Packetizer/Input
  *****************************************************************************/
 sout_packetizer_input_t *__sout_InputNew( vlc_object_t  *p_this,
-                                          sout_format_t *p_fmt )
+                                          es_format_t *p_fmt )
 {
     sout_instance_t         *p_sout = NULL;
     sout_packetizer_input_t *p_input;
 
-    int             i_try;
-
     /* search an stream output */
-    for( i_try = 0; i_try < 12; i_try++ )
-    {
-        p_sout = vlc_object_find( p_this, VLC_OBJECT_SOUT, FIND_ANYWHERE );
-        if( p_sout )
-        {
-            break;
-        }
-
-        msleep( 100*1000 );
-        msg_Dbg( p_this, "waiting for sout" );
-    }
-
-    if( !p_sout )
+    if( !( p_sout = vlc_object_find( p_this, VLC_OBJECT_SOUT, FIND_ANYWHERE ) ) )
     {
+        /* can't happen ... */
         msg_Err( p_this, "cannot find any stream ouput" );
-        return( NULL );
+        return NULL;
     }
 
     msg_Dbg( p_sout, "adding a new input" );
@@ -172,7 +195,7 @@ sout_packetizer_input_t *__sout_InputNew( vlc_object_t  *p_this,
     p_input->p_sout = p_sout;
     p_input->p_fmt  = p_fmt;
 
-    if( p_fmt->i_fourcc == VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
+    if( p_fmt->i_codec == VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
     {
         vlc_object_release( p_sout );
         return p_input;
@@ -202,7 +225,7 @@ int sout_InputDelete( sout_packetizer_input_t *p_input )
 
     msg_Dbg( p_sout, "removing an input" );
 
-    if( p_input->p_fmt->i_fourcc != VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
+    if( p_input->p_fmt->i_codec != VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
     {
         vlc_mutex_lock( &p_sout->lock );
         p_sout->p_stream->pf_del( p_sout->p_stream, p_input->id );
@@ -220,7 +243,7 @@ int sout_InputSendBuffer( sout_packetizer_input_t *p_input, sout_buffer_t *p_buf
     sout_instance_t     *p_sout = p_input->p_sout;
     int                 i_ret;
 
-    if( p_input->p_fmt->i_fourcc == VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
+    if( p_input->p_fmt->i_codec == VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
     {
         sout_BufferDelete( p_input->p_sout, p_buffer );
         return VLC_SUCCESS;
@@ -359,8 +382,8 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
     {
         int b_answer;
         if( p_mux->pf_capacity( p_mux,
-                                SOUT_MUX_CAP_GET_ADD_STREAM_ANY_TIME,
-                                NULL, (void*)&b_answer ) != SOUT_MUX_CAP_ERR_OK )
+                                SOUT_MUX_CAP_GET_ADD_STREAM_ANY_TIME, NULL,
+                                (void*)&b_answer ) != SOUT_MUX_CAP_ERR_OK )
         {
             b_answer = VLC_FALSE;
         }
@@ -369,6 +392,19 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
             msg_Dbg( p_sout, "muxer support adding stream at any time" );
             p_mux->b_add_stream_any_time = VLC_TRUE;
             p_mux->b_waiting_stream = VLC_FALSE;
+
+            if( p_mux->pf_capacity( p_mux,
+                                    SOUT_MUX_CAP_GET_ADD_STREAM_WAIT, NULL,
+                                    (void*)&b_answer ) != SOUT_MUX_CAP_ERR_OK )
+            {
+                b_answer = VLC_FALSE;
+            }
+            if( b_answer )
+            {
+                msg_Dbg( p_sout, "muxer prefers waiting for all ES before "
+                         "starting muxing" );
+                p_mux->b_waiting_stream = VLC_TRUE;
+            }
         }
         else
         {
@@ -386,7 +422,7 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
     return p_mux;
 }
 
-void sout_MuxDelete              ( sout_mux_t *p_mux )
+void sout_MuxDelete( sout_mux_t *p_mux )
 {
     if( p_mux->p_module )
     {
@@ -399,14 +435,14 @@ void sout_MuxDelete              ( sout_mux_t *p_mux )
     vlc_object_destroy( p_mux );
 }
 
-sout_input_t *sout_MuxAddStream( sout_mux_t *p_mux,
-                                 sout_format_t *p_fmt )
+sout_input_t *sout_MuxAddStream( sout_mux_t *p_mux, es_format_t *p_fmt )
 {
     sout_input_t *p_input;
 
-    if( !p_mux->b_add_stream_any_time && !p_mux->b_waiting_stream)
+    if( !p_mux->b_add_stream_any_time && !p_mux->b_waiting_stream )
     {
-        msg_Err( p_mux, "cannot add a new stream (unsuported while muxing for this format)" );
+        msg_Err( p_mux, "cannot add a new stream (unsuported while muxing "
+                        "for this format)" );
         return NULL;
     }
     if( p_mux->i_add_stream_start < 0 )
@@ -1200,60 +1236,22 @@ static char *_sout_stream_url_to_chain( vlc_object_t *p_this, char *psz_url )
 {
     mrl_t       mrl;
     char        *psz_chain, *p;
-    char        *psz_vcodec, *psz_acodec;
 
     mrl_Parse( &mrl, psz_url );
-    p = psz_chain = malloc( 500 + strlen( mrl.psz_way ) + strlen( mrl.psz_access ) + strlen( mrl.psz_name ) );
-
-    psz_vcodec = config_GetPsz( p_this, "sout-vcodec" );
-    if( psz_vcodec && *psz_vcodec == '\0')
-    {
-        FREE( psz_vcodec );
-    }
-    psz_acodec = config_GetPsz( p_this, "sout-acodec" );
-    if( psz_acodec && *psz_acodec == '\0' )
-    {
-        FREE( psz_acodec );
-    }
-    /* set transcoding */
-    if( psz_vcodec || psz_acodec )
-    {
-        p += sprintf( p, "transcode{" );
-        if( psz_vcodec )
-        {
-            int br;
-
-            p += sprintf( p, "vcodec=%s,", psz_vcodec );
-
-            if( ( br = config_GetInt( p_this, "sout-vbitrate" ) ) > 0 )
-            {
-                p += sprintf( p, "vb=%d,", br * 1000 );
-            }
-            free( psz_vcodec );
-        }
-        if( psz_acodec )
-        {
-            int br;
-
-            p += sprintf( p, "acodec=%s,", psz_acodec );
-            if( ( br = config_GetInt( p_this, "sout-abitrate" ) ) > 0 )
-            {
-                p += sprintf( p, "ab=%d,", br * 1000 );
-            }
-
-            free( psz_acodec );
-        }
-        p += sprintf( p, "}:" );
-    }
+    p = psz_chain = malloc( 500 + strlen( mrl.psz_way ) +
+                                  strlen( mrl.psz_access ) +
+                                  strlen( mrl.psz_name ) );
 
 
     if( config_GetInt( p_this, "sout-display" ) )
     {
-        p += sprintf( p, "duplicate{dst=display,dst=std{mux=\"%s\",access=\"%s\",url=\"%s\"}}", mrl.psz_way, mrl.psz_access, mrl.psz_name );
+        p += sprintf( p, "duplicate{dst=display,dst=std{mux=\"%s\",access=\"%s\",url=\"%s\"}}",
+                      mrl.psz_way, mrl.psz_access, mrl.psz_name );
     }
     else
     {
-        p += sprintf( p, "std{mux=\"%s\",access=\"%s\",url=\"%s\"}", mrl.psz_way, mrl.psz_access, mrl.psz_name );
+        p += sprintf( p, "std{mux=\"%s\",access=\"%s\",url=\"%s\"}",
+                      mrl.psz_way, mrl.psz_access, mrl.psz_name );
     }
 
     mrl_Clean( &mrl );
