@@ -5,7 +5,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000-2001 VideoLAN
- * $Id: video_output.c,v 1.231 2003/07/29 22:20:53 gbazin Exp $
+ * $Id: video_output.c,v 1.242 2003/12/11 23:12:46 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -62,6 +62,8 @@ static int FullscreenCallback( vlc_object_t *, char const *,
                                vlc_value_t, vlc_value_t, void * );
 static int DeinterlaceCallback( vlc_object_t *, char const *,
                                 vlc_value_t, vlc_value_t, void * );
+static int FilterCallback( vlc_object_t *, char const *,
+                           vlc_value_t, vlc_value_t, void * );
 
 /*****************************************************************************
  * vout_Request: find a video output thread, create one, or destroy one.
@@ -78,21 +80,22 @@ vout_thread_t * __vout_Request ( vlc_object_t *p_this, vout_thread_t *p_vout,
         /* Reattach video output to input before bailing out */
         if( p_vout )
         {
-            vlc_object_t *p_input;
+            vlc_object_t *p_playlist;
 
-            p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_PARENT );
+            p_playlist = vlc_object_find( p_this,
+                                          VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
 
-            if( p_input )
+            if( p_playlist )
             {
                 vlc_object_detach( p_vout );
-                vlc_object_attach( p_vout, p_input );
+                vlc_object_attach( p_vout, p_playlist );
 
-                vlc_object_release( p_input );
+                vlc_object_release( p_playlist );
             }
             else
             {
+                msg_Dbg( p_this, "cannot find playlist destroying vout" );
                 vlc_object_detach( p_vout );
-                /* vlc_object_release( p_vout ); */
                 vout_Destroy( p_vout );
             }
         }
@@ -110,20 +113,21 @@ vout_thread_t * __vout_Request ( vlc_object_t *p_this, vout_thread_t *p_vout,
 
         if( !p_vout )
         {
-            vlc_object_t *p_input;
+            vlc_object_t *p_playlist;
 
-            p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_PARENT );
-            if( p_input )
+            p_playlist = vlc_object_find( p_this,
+                                          VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+            if( p_playlist )
             {
-                p_vout = vlc_object_find( p_input, VLC_OBJECT_VOUT,
-                                          FIND_CHILD );
+                p_vout = vlc_object_find( p_playlist,
+                                          VLC_OBJECT_VOUT, FIND_CHILD );
                 /* only first children of p_input for unused vout */
-                if( p_vout && p_vout->p_parent != p_input )
+                if( p_vout && p_vout->p_parent != p_playlist )
                 {
                     vlc_object_release( p_vout );
                     p_vout = NULL;
                 }
-                vlc_object_release( p_input );
+                vlc_object_release( p_playlist );
             }
         }
     }
@@ -268,6 +272,7 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     p_vout->b_interface  = 0;
     p_vout->b_scale      = 1;
     p_vout->b_fullscreen = 0;
+    p_vout->i_alignment  = 0;
     p_vout->render_time  = 10;
     p_vout->c_fps_samples = 0;
     p_vout->b_filter_change = 0;
@@ -278,7 +283,6 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     var_Create( p_vout, "mouse-button-down", VLC_VAR_INTEGER );
     var_Create( p_vout, "mouse-moved", VLC_VAR_BOOL );
     var_Create( p_vout, "mouse-clicked", VLC_VAR_INTEGER );
-    var_Create( p_vout, "key-pressed", VLC_VAR_STRING );
 
     var_Create( p_vout, "intf-change", VLC_VAR_BOOL );
     val.b_bool = VLC_TRUE;
@@ -297,6 +301,7 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     var_Create( p_vout, "width", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Create( p_vout, "height", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Create( p_vout, "zoom", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "align", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
 
     p_vout->b_override_aspect = VLC_FALSE;
 
@@ -395,12 +400,12 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
         return NULL;
     }
 
-    p_vout->p_text_renderer_module = module_Need( p_vout, "text renderer", 
-						  NULL );
+    p_vout->p_text_renderer_module = module_Need( p_vout, "text renderer",
+                                                  NULL );
     if( p_vout->p_text_renderer_module == NULL )
     {
-	msg_Warn( p_vout, "no suitable text renderer module" );
-	p_vout->pf_add_string = NULL;
+        msg_Warn( p_vout, "no suitable text renderer module" );
+        p_vout->pf_add_string = NULL;
     }
 
     /* Create a few object variables for interface interaction */
@@ -433,9 +438,22 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     if( var_Get( p_vout, "deinterlace-mode", &val ) == VLC_SUCCESS )
     {
         var_Set( p_vout, "deinterlace", val );
+        if( val.psz_string ) free( val.psz_string );
     }
     var_AddCallback( p_vout, "deinterlace", DeinterlaceCallback, NULL );
 
+
+    var_Create( p_vout, "filter", VLC_VAR_STRING );
+    text.psz_string = _("Filters");
+    var_Change( p_vout, "filter", VLC_VAR_SETTEXT, &text, NULL );
+    var_Change( p_vout, "filter", VLC_VAR_INHERITVALUE, &val, NULL );
+    if( val.psz_string )
+    {
+        var_Set( p_vout, "filter", val );
+        free( val.psz_string );
+    }
+    var_AddCallback( p_vout, "filter", FilterCallback, NULL );
+
     /* Calculate delay created by internal caching */
     p_input_thread = (input_thread_t *)vlc_object_find( p_vout,
                                            VLC_OBJECT_INPUT, FIND_ANYWHERE );
@@ -482,6 +500,8 @@ void vout_Destroy( vout_thread_t *p_vout )
     p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST, 
                                   FIND_ANYWHERE );
 
+    if( p_vout->psz_filter_chain ) free( p_vout->psz_filter_chain );
+
     /* Free structure */
     vlc_object_destroy( p_vout );
 
@@ -812,7 +832,7 @@ static void RunThread( vout_thread_t *p_vout)
             }
 
             if( display_date >
-                current_date + p_vout->i_pts_delay +  VOUT_BOGUS_DELAY )
+                current_date + p_vout->i_pts_delay + VOUT_BOGUS_DELAY )
             {
                 /* Picture is waaay too early: it will be destroyed */
                 vlc_mutex_lock( &p_vout->picture_lock );
@@ -892,9 +912,17 @@ static void RunThread( vout_thread_t *p_vout)
          */
         if( display_date != 0 && p_directbuffer != NULL )
         {
-            /* Store render time using a sliding mean */
-            p_vout->render_time += mdate() - current_date;
-            p_vout->render_time >>= 1;
+            mtime_t current_render_time = mdate() - current_date;
+            /* if render time is very large we don't include it in the mean */
+            if( current_render_time < p_vout->render_time +
+                VOUT_DISPLAY_DELAY )
+            {
+                /* Store render time using a sliding mean weighting to
+                 * current value in a 3 to 1 ratio*/
+                p_vout->render_time *= 3;
+                p_vout->render_time += current_render_time;
+                p_vout->render_time >>= 2;
+            }
         }
 
         /* Give back change lock */
@@ -903,7 +931,12 @@ static void RunThread( vout_thread_t *p_vout)
         /* Sleep a while or until a given date */
         if( display_date != 0 )
         {
-            mwait( display_date - VOUT_MWAIT_TOLERANCE );
+            /* If there are filters in the chain, better give them the picture
+             * in advance */
+            if( !p_vout->psz_filter_chain || !*p_vout->psz_filter_chain )
+            {
+                mwait( display_date - VOUT_MWAIT_TOLERANCE );
+            }
         }
         else
         {
@@ -968,9 +1001,37 @@ static void RunThread( vout_thread_t *p_vout)
             }
 
             /* Need to reinitialise the chroma plugin */
-            if( p_vout->chroma.p_module->pf_deactivate )
-                p_vout->chroma.p_module->pf_deactivate( VLC_OBJECT(p_vout) );
-            p_vout->chroma.p_module->pf_activate( VLC_OBJECT(p_vout) );
+            if( p_vout->chroma.p_module )
+            {
+                if( p_vout->chroma.p_module->pf_deactivate )
+                    p_vout->chroma.p_module->pf_deactivate( VLC_OBJECT(p_vout) );
+                p_vout->chroma.p_module->pf_activate( VLC_OBJECT(p_vout) );
+            }
+        }
+
+        if( p_vout->i_changes & VOUT_PICTURE_BUFFERS_CHANGE )
+        {
+            /* This happens when the picture buffers need to be recreated.
+             * This is useful on multimonitor displays for instance.
+             *
+             * Warning: This only works when the vout creates only 1 picture
+             * buffer!! */
+            p_vout->i_changes &= ~VOUT_PICTURE_BUFFERS_CHANGE;
+
+            if( !p_vout->b_direct )
+            {
+                module_Unneed( p_vout, p_vout->chroma.p_module );
+            }
+
+            vlc_mutex_lock( &p_vout->picture_lock );
+
+            p_vout->pf_end( p_vout );
+
+            I_OUTPUTPICTURES = I_RENDERPICTURES = 0;
+
+            p_vout->b_error = InitThread( p_vout );
+
+            vlc_mutex_unlock( &p_vout->picture_lock );
         }
     }
 
@@ -1178,6 +1239,9 @@ static void InitWindowSize( vout_thread_t *p_vout, int *pi_width,
 
 #define FP_FACTOR 1000                             /* our fixed point factor */
 
+    var_Get( p_vout, "align", &val );
+    p_vout->i_alignment = val.i_int;
+
     var_Get( p_vout, "width", &val );
     i_width = val.i_int;
     var_Get( p_vout, "height", &val );
@@ -1325,7 +1389,7 @@ static int DeinterlaceCallback( vlc_object_t *p_this, char const *psz_cmd,
 
     for( i = 0 ; i < p_input->stream.i_es_number ; i++ )
     {
-        if( ( ES->i_cat == VIDEO_ES ) && ES->p_decoder_fifo != NULL )
+        if( ( ES->i_cat == VIDEO_ES ) && ES->p_dec != NULL )
         {
             input_UnselectES( p_input, ES );
             input_SelectES( p_input, ES );
@@ -1340,3 +1404,45 @@ static int DeinterlaceCallback( vlc_object_t *p_this, char const *psz_cmd,
     var_Set( p_vout, "intf-change", val );
     return VLC_SUCCESS;
 }
+
+static int FilterCallback( vlc_object_t *p_this, char const *psz_cmd,
+                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    vout_thread_t *p_vout = (vout_thread_t *)p_this;
+    input_thread_t *p_input;
+    vlc_value_t val;
+    unsigned int i;
+
+    p_input = (input_thread_t *)vlc_object_find( p_this, VLC_OBJECT_INPUT,
+                                                 FIND_PARENT );
+
+    if (!p_input)
+    {
+        msg_Err( p_vout, "Input not found" );
+        return( VLC_EGENERIC );
+    }
+    /* Restart the video stream */
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
+    p_vout->b_filter_change = VLC_TRUE;
+
+#define ES p_input->stream.pp_es[i]
+
+    for( i = 0 ; i < p_input->stream.i_es_number ; i++ )
+    {
+        if( ( ES->i_cat == VIDEO_ES ) && ES->p_dec != NULL )
+        {
+            input_UnselectES( p_input, ES );
+            input_SelectES( p_input, ES );
+        }
+#undef ES
+    }
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
+    vlc_object_release( p_input );
+
+    val.b_bool = VLC_TRUE;
+    var_Set( p_vout, "intf-change", val );
+    return VLC_SUCCESS;
+}
+
